@@ -22,7 +22,6 @@ import time
 import six
 from six.moves import configparser
 
-from swift import gettext_ as _
 from swift.common.utils import get_valid_utf8_str
 
 
@@ -58,7 +57,7 @@ class ContainerSyncRealms(object):
                     log_func = self.logger.debug
                 else:
                     log_func = self.logger.error
-                log_func(_('Could not load %(conf)r: %(error)s') % {
+                log_func('Could not load %(conf)r: %(error)s', {
                          'conf': self.conf_path, 'error': err})
             else:
                 if mtime != self.conf_path_mtime:
@@ -68,11 +67,11 @@ class ContainerSyncRealms(object):
                         conf.read(self.conf_path)
                     except configparser.ParsingError as err:
                         self.logger.error(
-                            _('Could not load %(conf)r: %(error)s')
-                            % {'conf': self.conf_path, 'error': err})
+                            'Could not load %(conf)r: %(error)s',
+                            {'conf': self.conf_path, 'error': err})
                     else:
                         try:
-                            self.mtime_check_interval = conf.getint(
+                            self.mtime_check_interval = conf.getfloat(
                                 'DEFAULT', 'mtime_check_interval')
                             self.next_mtime_check = \
                                 now + self.mtime_check_interval
@@ -82,9 +81,9 @@ class ContainerSyncRealms(object):
                                 now + self.mtime_check_interval
                         except (configparser.ParsingError, ValueError) as err:
                             self.logger.error(
-                                _('Error in %(conf)r with '
-                                  'mtime_check_interval: %(error)s')
-                                % {'conf': self.conf_path, 'error': err})
+                                'Error in %(conf)r with '
+                                'mtime_check_interval: %(error)s',
+                                {'conf': self.conf_path, 'error': err})
                         realms = {}
                         for section in conf.sections():
                             realm = {}
