@@ -23,7 +23,6 @@ import uuid
 import errno
 import re
 from contextlib import contextmanager
-from swift import gettext_ as _
 
 from eventlet import GreenPool, sleep, Timeout
 from eventlet.green import subprocess
@@ -34,7 +33,7 @@ from swift.common.utils import get_logger, whataremyips, storage_directory, \
     renamer, mkdirs, lock_parent_directory, config_true_value, \
     unlink_older_than, dump_recon_cache, rsync_module_interpolation, \
     parse_override_options, round_robin_iter, Everything, get_db_files, \
-    parse_db_filename, quote, RateLimitedIterator
+    parse_db_filename, quote, RateLimitedIterator, config_auto_int_value
 from swift.common import ring
 from swift.common.ring.utils import is_local_device
 from swift.common.http import HTTP_NOT_FOUND, HTTP_INSUFFICIENT_STORAGE, \
@@ -174,8 +173,9 @@ class ReplConnection(BufferedHTTPConnection):
             response.data = response.read()
             return response
         except (Exception, Timeout):
+            self.close()
             self.logger.exception(
-                _('ERROR reading HTTP response from %s'), self.node)
+                'ERROR reading HTTP response from %s', self.node)
             return None
 
 
@@ -195,7 +195,7 @@ class Replicator(Daemon):
         self.cpool = GreenPool(size=concurrency)
         swift_dir = conf.get('swift_dir', '/etc/swift')
         self.ring = ring.Ring(swift_dir, ring_name=self.server_type)
-        self._local_device_ids = set()
+        self._local_device_ids = {}
         self.per_diff = int(conf.get('per_diff', 1000))
         self.max_diffs = int(conf.get('max_diffs') or 100)
         self.interval = float(conf.get('interval') or
@@ -238,6 +238,8 @@ class Replicator(Daemon):
         self.extract_device_re = re.compile('%s%s([^%s]+)' % (
             self.root, os.path.sep, os.path.sep))
         self.handoffs_only = config_true_value(conf.get('handoffs_only', 'no'))
+        self.handoff_delete = config_auto_int_value(
+            conf.get('handoff_delete', 'auto'), 0)
 
     def _zero_stats(self):
         """Zero out the stats."""
@@ -251,15 +253,15 @@ class Replicator(Daemon):
         """Report the current stats to the logs."""
         now = time.time()
         self.logger.info(
-            _('Attempted to replicate %(count)d dbs in %(time).5f seconds '
-              '(%(rate).5f/s)'),
+            'Attempted to replicate %(count)d dbs in %(time).5f seconds '
+            '(%(rate).5f/s)',
             {'count': self.stats['attempted'],
              'time': now - self.stats['start'],
              'rate': self.stats['attempted'] /
                 (now - self.stats['start'] + 0.0000001)})
-        self.logger.info(_('Removed %(remove)d dbs') % self.stats)
-        self.logger.info(_('%(success)s successes, %(failure)s failures')
-                         % self.stats)
+        self.logger.info('Removed %(remove)d dbs', self.stats)
+        self.logger.info('%(success)s successes, %(failure)s failures',
+                         self.stats)
         dump_recon_cache(
             {'replication_stats': self.stats,
              'replication_time': now - self.stats['start'],
@@ -305,7 +307,7 @@ class Replicator(Daemon):
         proc = subprocess.Popen(popen_args)
         proc.communicate()
         if proc.returncode != 0:
-            self.logger.error(_('ERROR rsync failed with %(code)s: %(args)s'),
+            self.logger.error('ERROR rsync failed with %(code)s: %(args)s',
                               {'code': proc.returncode, 'args': popen_args})
         return proc.returncode == 0
 
@@ -554,7 +556,13 @@ class Replicator(Daemon):
             reason = '%s new rows' % max_row_delta
             self.logger.debug(log_template, reason)
             return True
-        if not (responses and all(responses)):
+        if self.handoff_delete:
+            # delete handoff if we have had handoff_delete successes
+            successes_count = len([resp for resp in responses if resp])
+            delete_handoff = successes_count >= self.handoff_delete
+        else:
+            delete_handoff = responses and all(responses)
+        if not delete_handoff:
             reason = '%s/%s success' % (responses.count(True), len(responses))
             self.logger.debug(log_template, reason)
             return True
@@ -616,10 +624,10 @@ class Replicator(Daemon):
                     'replicate out and remove.' % (object_file, name, bpart))
         except (Exception, Timeout) as e:
             if 'no such table' in str(e):
-                self.logger.error(_('Quarantining DB %s'), object_file)
+                self.logger.error('Quarantining DB %s', object_file)
                 quarantine_db(broker.db_file, broker.db_type)
             else:
-                self.logger.exception(_('ERROR reading db %s'), object_file)
+                self.logger.exception('ERROR reading db %s', object_file)
             nodes = self.ring.get_part_nodes(int(partition))
             self._add_failure_stats([(failure_dev['replication_ip'],
                                       failure_dev['device'])
@@ -671,13 +679,13 @@ class Replicator(Daemon):
                     repl_nodes.append(next(more_nodes))
                 except StopIteration:
                     self.logger.error(
-                        _('ERROR There are not enough handoff nodes to reach '
-                          'replica count for partition %s'),
+                        'ERROR There are not enough handoff nodes to reach '
+                        'replica count for partition %s',
                         partition)
-                self.logger.error(_('ERROR Remote drive not mounted %s'), node)
+                self.logger.error('ERROR Remote drive not mounted %s', node)
             except (Exception, Timeout):
-                self.logger.exception(_('ERROR syncing %(file)s with node'
-                                        ' %(node)s'),
+                self.logger.exception('ERROR syncing %(file)s with node'
+                                      ' %(node)s',
                                       {'file': object_file, 'node': node})
             if not success:
                 failure_devs_info.add((node['replication_ip'], node['device']))
@@ -776,16 +784,17 @@ class Replicator(Daemon):
         dirs = []
         ips = whataremyips(self.bind_ip)
         if not ips:
-            self.logger.error(_('ERROR Failed to get my own IPs?'))
+            self.logger.error('ERROR Failed to get my own IPs?')
             return
 
-        if self.handoffs_only:
+        if self.handoffs_only or self.handoff_delete:
             self.logger.warning(
-                'Starting replication pass with handoffs_only enabled. '
-                'This mode is not intended for normal '
-                'operation; use handoffs_only with care.')
+                'Starting replication pass with handoffs_only '
+                'and/or handoffs_delete enabled. '
+                'These modes are not intended for normal '
+                'operation; use these options with care.')
 
-        self._local_device_ids = set()
+        self._local_device_ids = {}
         found_local = False
         for node in self.ring.devs:
             if node and is_local_device(ips, self.port,
@@ -812,7 +821,7 @@ class Replicator(Daemon):
                     time.time() - self.reclaim_age)
                 datadir = os.path.join(self.root, node['device'], self.datadir)
                 if os.path.isdir(datadir):
-                    self._local_device_ids.add(node['id'])
+                    self._local_device_ids[node['id']] = node
                     part_filt = self._partition_dir_filter(
                         node['id'], partitions_to_replicate)
                     dirs.append((datadir, node['id'], part_filt))
@@ -820,16 +829,17 @@ class Replicator(Daemon):
             self.logger.error("Can't find itself %s with port %s in ring "
                               "file, not replicating",
                               ", ".join(ips), self.port)
-        self.logger.info(_('Beginning replication run'))
+        self.logger.info('Beginning replication run')
         for part, object_file, node_id in self.roundrobin_datadirs(dirs):
             self.cpool.spawn_n(
                 self._replicate_object, part, object_file, node_id)
         self.cpool.waitall()
-        self.logger.info(_('Replication run OVER'))
-        if self.handoffs_only:
+        self.logger.info('Replication run OVER')
+        if self.handoffs_only or self.handoff_delete:
             self.logger.warning(
-                'Finished replication pass with handoffs_only enabled. '
-                'If handoffs_only is no longer required, disable it.')
+                'Finished replication pass with handoffs_only and/or '
+                'handoffs_delete enabled. If these are no longer required, '
+                'disable them.')
         self._report_stats()
 
     def run_forever(self, *args, **kwargs):
@@ -842,7 +852,7 @@ class Replicator(Daemon):
             try:
                 self.run_once()
             except (Exception, Timeout):
-                self.logger.exception(_('ERROR trying to replicate'))
+                self.logger.exception('ERROR trying to replicate')
             elapsed = time.time() - begin
             if elapsed < self.interval:
                 sleep(self.interval - elapsed)
@@ -946,7 +956,7 @@ class ReplicatorRpc(object):
                 info = self._get_synced_replication_info(broker, remote_info)
             except (Exception, Timeout) as e:
                 if 'no such table' in str(e):
-                    self.logger.error(_("Quarantining DB %s"), broker)
+                    self.logger.error("Quarantining DB %s", broker)
                     quarantine_db(broker.db_file, broker.db_type)
                     return HTTPNotFound()
                 raise
