@@ -27,13 +27,15 @@ import six
 import six.moves.cPickle as pickle
 from six.moves.http_client import HTTPException
 
-from swift.common.bufferedhttp import http_connect
+from swift.common.bufferedhttp import http_connect, http_connect_raw
 from swift.common.exceptions import ClientException
-from swift.common.utils import Timestamp, FileLikeIter
+from swift.common.request_helpers import USE_REPLICATION_NETWORK_HEADER, \
+    get_ip_port
+from swift.common.swob import normalize_etag
+from swift.common.utils import Timestamp, FileLikeIter, quote
 from swift.common.http import HTTP_NO_CONTENT, HTTP_INSUFFICIENT_STORAGE, \
     is_success, is_server_error
 from swift.common.header_key_dict import HeaderKeyDict
-from swift.common.utils import quote
 
 
 class DirectClientException(ClientException):
@@ -54,22 +56,94 @@ class DirectClientException(ClientException):
             http_reason=resp.reason, http_headers=headers)
 
 
-def _make_req(node, part, method, path, _headers, stype,
-              conn_timeout=5, response_timeout=15):
+class DirectClientReconException(ClientException):
+
+    def __init__(self, method, node, path, resp):
+        if not isinstance(path, six.text_type):
+            path = path.decode("utf-8")
+        msg = 'server %s:%s direct %s %r gave status %s' % (
+            node['ip'], node['port'], method, path, resp.status)
+        headers = HeaderKeyDict(resp.getheaders())
+        super(DirectClientReconException, self).__init__(
+            msg, http_host=node['ip'], http_port=node['port'],
+            http_status=resp.status, http_reason=resp.reason,
+            http_headers=headers)
+
+
+def _make_path(*components):
+    return u'/' + u'/'.join(
+        x.decode('utf-8') if isinstance(x, six.binary_type) else x
+        for x in components)
+
+
+def _make_req(node, part, method, path, headers, stype,
+              conn_timeout=5, response_timeout=15, send_timeout=15,
+              contents=None, content_length=None, chunk_size=65535):
     """
     Make request to backend storage node.
     (i.e. 'Account', 'Container', 'Object')
     :param node: a node dict from a ring
-    :param part: an integer, the partion number
+    :param part: an integer, the partition number
     :param method: a string, the HTTP method (e.g. 'PUT', 'DELETE', etc)
     :param path: a string, the request path
     :param headers: a dict, header name => value
     :param stype: a string, describing the type of service
+    :param conn_timeout: timeout while waiting for connection; default is 5
+        seconds
+    :param response_timeout: timeout while waiting for response; default is 15
+        seconds
+    :param send_timeout: timeout for sending request body; default is 15
+        seconds
+    :param contents: an iterable or string to read object data from
+    :param content_length: value to send as content-length header
+    :param chunk_size: if defined, chunk size of data to send
     :returns: an HTTPResponse object
+    :raises DirectClientException: if the response status is not 2xx
+    :raises eventlet.Timeout: if either conn_timeout or response_timeout is
+        exceeded
     """
+    if contents is not None:
+        if content_length is not None:
+            headers['Content-Length'] = str(content_length)
+        else:
+            for n, v in headers.items():
+                if n.lower() == 'content-length':
+                    content_length = int(v)
+        if not contents:
+            headers['Content-Length'] = '0'
+        if isinstance(contents, six.string_types):
+            contents = [contents]
+        if content_length is None:
+            headers['Transfer-Encoding'] = 'chunked'
+
+    ip, port = get_ip_port(node, headers)
+    headers.setdefault('X-Backend-Allow-Reserved-Names', 'true')
     with Timeout(conn_timeout):
-        conn = http_connect(node['ip'], node['port'], node['device'], part,
-                            method, path, headers=_headers)
+        conn = http_connect(ip, port, node['device'], part,
+                            method, path, headers=headers)
+
+    if contents is not None:
+        contents_f = FileLikeIter(contents)
+
+        with Timeout(send_timeout):
+            if content_length is None:
+                chunk = contents_f.read(chunk_size)
+                while chunk:
+                    conn.send(b'%x\r\n%s\r\n' % (len(chunk), chunk))
+                    chunk = contents_f.read(chunk_size)
+                conn.send(b'0\r\n\r\n')
+            else:
+                left = content_length
+                while left > 0:
+                    size = chunk_size
+                    if size > left:
+                        size = left
+                    chunk = contents_f.read(size)
+                    if not chunk:
+                        break
+                    conn.send(chunk)
+                    left -= len(chunk)
+
     with Timeout(response_timeout):
         resp = conn.getresponse()
         resp.read()
@@ -82,30 +156,52 @@ def _get_direct_account_container(path, stype, node, part,
                                   marker=None, limit=None,
                                   prefix=None, delimiter=None,
                                   conn_timeout=5, response_timeout=15,
-                                  end_marker=None, reverse=None):
-    """Base class for get direct account and container.
+                                  end_marker=None, reverse=None, headers=None,
+                                  extra_params=None):
+    """Base function for get direct account and container.
 
-    Do not use directly use the get_direct_account or
-    get_direct_container instead.
+    Do not use directly use the direct_get_account or
+    direct_get_container instead.
     """
-    params = ['format=json']
+    if headers is None:
+        headers = {}
+
+    params = {'format': 'json'}
+    if extra_params:
+        for key, value in extra_params.items():
+            if value is not None:
+                params[key] = value
     if marker:
-        params.append('marker=%s' % quote(marker))
+        if 'marker' in params:
+            raise TypeError('duplicate values for keyword arg: marker')
+        params['marker'] = quote(marker)
     if limit:
-        params.append('limit=%d' % limit)
+        if 'limit' in params:
+            raise TypeError('duplicate values for keyword arg: limit')
+        params['limit'] = '%d' % limit
     if prefix:
-        params.append('prefix=%s' % quote(prefix))
+        if 'prefix' in params:
+            raise TypeError('duplicate values for keyword arg: prefix')
+        params['prefix'] = quote(prefix)
     if delimiter:
-        params.append('delimiter=%s' % quote(delimiter))
+        if 'delimiter' in params:
+            raise TypeError('duplicate values for keyword arg: delimiter')
+        params['delimiter'] = quote(delimiter)
     if end_marker:
-        params.append('end_marker=%s' % quote(end_marker))
+        if 'end_marker' in params:
+            raise TypeError('duplicate values for keyword arg: end_marker')
+        params['end_marker'] = quote(end_marker)
     if reverse:
-        params.append('reverse=%s' % quote(reverse))
-    qs = '&'.join(params)
+        if 'reverse' in params:
+            raise TypeError('duplicate values for keyword arg: reverse')
+        params['reverse'] = quote(reverse)
+    qs = '&'.join('%s=%s' % (k, v) for k, v in params.items())
+
+    ip, port = get_ip_port(node, headers)
     with Timeout(conn_timeout):
-        conn = http_connect(node['ip'], node['port'], node['device'], part,
+        conn = http_connect(ip, port, node['device'], part,
                             'GET', path, query_string=qs,
-                            headers=gen_headers())
+                            headers=gen_headers(hdrs_in=headers))
     with Timeout(response_timeout):
         resp = conn.getresponse()
     if not is_success(resp.status):
@@ -121,17 +217,30 @@ def _get_direct_account_container(path, stype, node, part,
     return resp_headers, json.loads(resp.read())
 
 
-def gen_headers(hdrs_in=None, add_ts=False):
+def gen_headers(hdrs_in=None, add_ts=True):
+    """
+    Get the headers ready for a request. All requests should have a User-Agent
+    string, but if one is passed in don't over-write it. Not all requests will
+    need an X-Timestamp, but if one is passed in do not over-write it.
+
+    :param headers: dict or None, base for HTTP headers
+    :param add_ts: boolean, should be True for any "unsafe" HTTP request
+
+    :returns: HeaderKeyDict based on headers and ready for the request
+    """
     hdrs_out = HeaderKeyDict(hdrs_in) if hdrs_in else HeaderKeyDict()
-    if add_ts:
+    if add_ts and 'X-Timestamp' not in hdrs_out:
         hdrs_out['X-Timestamp'] = Timestamp.now().internal
-    hdrs_out['User-Agent'] = 'direct-client %s' % os.getpid()
+    if 'user-agent' not in hdrs_out:
+        hdrs_out['User-Agent'] = 'direct-client %s' % os.getpid()
+    hdrs_out.setdefault('X-Backend-Allow-Reserved-Names', 'true')
     return hdrs_out
 
 
 def direct_get_account(node, part, account, marker=None, limit=None,
                        prefix=None, delimiter=None, conn_timeout=5,
-                       response_timeout=15, end_marker=None, reverse=None):
+                       response_timeout=15, end_marker=None, reverse=None,
+                       headers=None):
     """
     Get listings directly from the account server.
 
@@ -149,8 +258,9 @@ def direct_get_account(node, part, account, marker=None, limit=None,
     :returns: a tuple of (response headers, a list of containers) The response
               headers will HeaderKeyDict.
     """
-    path = '/' + account
+    path = _make_path(account)
     return _get_direct_account_container(path, "Account", node, part,
+                                         headers=headers,
                                          marker=marker,
                                          limit=limit, prefix=prefix,
                                          delimiter=delimiter,
@@ -165,13 +275,13 @@ def direct_delete_account(node, part, account, conn_timeout=5,
     if headers is None:
         headers = {}
 
-    path = '/%s' % account
+    path = _make_path(account)
     _make_req(node, part, 'DELETE', path, gen_headers(headers, True),
               'Account', conn_timeout, response_timeout)
 
 
 def direct_head_container(node, part, account, container, conn_timeout=5,
-                          response_timeout=15):
+                          response_timeout=15, headers=None):
     """
     Request container information directly from the container server.
 
@@ -184,8 +294,11 @@ def direct_head_container(node, part, account, container, conn_timeout=5,
     :returns: a dict containing the response's headers in a HeaderKeyDict
     :raises ClientException: HTTP HEAD request failed
     """
-    path = '/%s/%s' % (account, container)
-    resp = _make_req(node, part, 'HEAD', path, gen_headers(),
+    if headers is None:
+        headers = {}
+
+    path = _make_path(account, container)
+    resp = _make_req(node, part, 'HEAD', path, gen_headers(headers),
                      'Container', conn_timeout, response_timeout)
 
     resp_headers = HeaderKeyDict()
@@ -197,7 +310,7 @@ def direct_head_container(node, part, account, container, conn_timeout=5,
 def direct_get_container(node, part, account, container, marker=None,
                          limit=None, prefix=None, delimiter=None,
                          conn_timeout=5, response_timeout=15, end_marker=None,
-                         reverse=None):
+                         reverse=None, headers=None, extra_params=None):
     """
     Get container listings directly from the container server.
 
@@ -213,10 +326,17 @@ def direct_get_container(node, part, account, container, marker=None,
     :param response_timeout: timeout in seconds for getting the response
     :param end_marker: end_marker query
     :param reverse: reverse the returned listing
+    :param headers: headers to be included in the request
+    :param extra_params: a dict of extra parameters to be included in the
+        request. It can be used to pass additional parameters, e.g,
+        {'states':'updating'} can be used with shard_range/namespace listing.
+        It can also be used to pass the existing keyword args, like 'marker' or
+        'limit', but if the same parameter appears twice in both keyword arg
+        (not None) and extra_params, this function will raise TypeError.
     :returns: a tuple of (response headers, a list of objects) The response
               headers will be a HeaderKeyDict.
     """
-    path = '/%s/%s' % (account, container)
+    path = _make_path(account, container)
     return _get_direct_account_container(path, "Container", node,
                                          part, marker=marker,
                                          limit=limit, prefix=prefix,
@@ -224,7 +344,9 @@ def direct_get_container(node, part, account, container, marker=None,
                                          end_marker=end_marker,
                                          reverse=reverse,
                                          conn_timeout=conn_timeout,
-                                         response_timeout=response_timeout)
+                                         response_timeout=response_timeout,
+                                         headers=headers,
+                                         extra_params=extra_params)
 
 
 def direct_delete_container(node, part, account, container, conn_timeout=5,
@@ -244,12 +366,67 @@ def direct_delete_container(node, part, account, container, conn_timeout=5,
     if headers is None:
         headers = {}
 
-    path = '/%s/%s' % (account, container)
+    path = _make_path(account, container)
     add_timestamp = 'x-timestamp' not in (k.lower() for k in headers)
     _make_req(node, part, 'DELETE', path, gen_headers(headers, add_timestamp),
               'Container', conn_timeout, response_timeout)
 
 
+def direct_put_container(node, part, account, container, conn_timeout=5,
+                         response_timeout=15, headers=None, contents=None,
+                         content_length=None, chunk_size=65535):
+    """
+    Make a PUT request to a container server.
+
+    :param node: node dictionary from the ring
+    :param part: partition the container is on
+    :param account: account name
+    :param container: container name
+    :param conn_timeout: timeout in seconds for establishing the connection
+    :param response_timeout: timeout in seconds for getting the response
+    :param headers: additional headers to include in the request
+    :param contents: an iterable or string to send in request body (optional)
+    :param content_length: value to send as content-length header (optional)
+    :param chunk_size: chunk size of data to send (optional)
+    :raises ClientException: HTTP PUT request failed
+    """
+    if headers is None:
+        headers = {}
+
+    lower_headers = set(k.lower() for k in headers)
+    headers_out = gen_headers(headers,
+                              add_ts='x-timestamp' not in lower_headers)
+    path = _make_path(account, container)
+    _make_req(node, part, 'PUT', path, headers_out, 'Container', conn_timeout,
+              response_timeout, contents=contents,
+              content_length=content_length, chunk_size=chunk_size)
+
+
+def direct_post_container(node, part, account, container, conn_timeout=5,
+                          response_timeout=15, headers=None):
+    """
+    Make a POST request to a container server.
+
+    :param node: node dictionary from the ring
+    :param part: partition the container is on
+    :param account: account name
+    :param container: container name
+    :param conn_timeout: timeout in seconds for establishing the connection
+    :param response_timeout: timeout in seconds for getting the response
+    :param headers: additional headers to include in the request
+    :raises ClientException: HTTP PUT request failed
+    """
+    if headers is None:
+        headers = {}
+
+    lower_headers = set(k.lower() for k in headers)
+    headers_out = gen_headers(headers,
+                              add_ts='x-timestamp' not in lower_headers)
+    path = _make_path(account, container)
+    return _make_req(node, part, 'POST', path, headers_out, 'Container',
+                     conn_timeout, response_timeout)
+
+
 def direct_put_container_object(node, part, account, container, obj,
                                 conn_timeout=5, response_timeout=15,
                                 headers=None):
@@ -258,7 +435,7 @@ def direct_put_container_object(node, part, account, container, obj,
 
     have_x_timestamp = 'x-timestamp' in (k.lower() for k in headers)
 
-    path = '/%s/%s/%s' % (account, container, obj)
+    path = _make_path(account, container, obj)
     _make_req(node, part, 'PUT', path,
               gen_headers(headers, add_ts=(not have_x_timestamp)),
               'Container', conn_timeout, response_timeout)
@@ -273,7 +450,7 @@ def direct_delete_container_object(node, part, account, container, obj,
     headers = gen_headers(headers, add_ts='x-timestamp' not in (
         k.lower() for k in headers))
 
-    path = '/%s/%s/%s' % (account, container, obj)
+    path = _make_path(account, container, obj)
     _make_req(node, part, 'DELETE', path, headers,
               'Container', conn_timeout, response_timeout)
 
@@ -299,7 +476,7 @@ def direct_head_object(node, part, account, container, obj, conn_timeout=5,
 
     headers = gen_headers(headers)
 
-    path = '/%s/%s/%s' % (account, container, obj)
+    path = _make_path(account, container, obj)
     resp = _make_req(node, part, 'HEAD', path, headers,
                      'Object', conn_timeout, response_timeout)
 
@@ -330,9 +507,10 @@ def direct_get_object(node, part, account, container, obj, conn_timeout=5,
     if headers is None:
         headers = {}
 
-    path = '/%s/%s/%s' % (account, container, obj)
+    ip, port = get_ip_port(node, headers)
+    path = _make_path(account, container, obj)
     with Timeout(conn_timeout):
-        conn = http_connect(node['ip'], node['port'], node['device'], part,
+        conn = http_connect(ip, port, node['device'], part,
                             'GET', path, headers=gen_headers(headers))
     with Timeout(response_timeout):
         resp = conn.getresponse()
@@ -380,62 +558,24 @@ def direct_put_object(node, part, account, container, name, contents,
     :raises ClientException: HTTP PUT request failed
     """
 
-    path = '/%s/%s/%s' % (account, container, name)
+    path = _make_path(account, container, name)
     if headers is None:
         headers = {}
     if etag:
-        headers['ETag'] = etag.strip('"')
-    if content_length is not None:
-        headers['Content-Length'] = str(content_length)
-    else:
-        for n, v in headers.items():
-            if n.lower() == 'content-length':
-                content_length = int(v)
+        headers['ETag'] = normalize_etag(etag)
     if content_type is not None:
         headers['Content-Type'] = content_type
     else:
         headers['Content-Type'] = 'application/octet-stream'
-    if not contents:
-        headers['Content-Length'] = '0'
-    if isinstance(contents, six.string_types):
-        contents = [contents]
     # Incase the caller want to insert an object with specific age
     add_ts = 'X-Timestamp' not in headers
 
-    if content_length is None:
-        headers['Transfer-Encoding'] = 'chunked'
+    resp = _make_req(
+        node, part, 'PUT', path, gen_headers(headers, add_ts=add_ts),
+        'Object', conn_timeout, response_timeout, contents=contents,
+        content_length=content_length, chunk_size=chunk_size)
 
-    with Timeout(conn_timeout):
-        conn = http_connect(node['ip'], node['port'], node['device'], part,
-                            'PUT', path, headers=gen_headers(headers, add_ts))
-
-    contents_f = FileLikeIter(contents)
-
-    if content_length is None:
-        chunk = contents_f.read(chunk_size)
-        while chunk:
-            conn.send('%x\r\n%s\r\n' % (len(chunk), chunk))
-            chunk = contents_f.read(chunk_size)
-        conn.send('0\r\n\r\n')
-    else:
-        left = content_length
-        while left > 0:
-            size = chunk_size
-            if size > left:
-                size = left
-            chunk = contents_f.read(size)
-            if not chunk:
-                break
-            conn.send(chunk)
-            left -= len(chunk)
-
-    with Timeout(response_timeout):
-        resp = conn.getresponse()
-        resp.read()
-    if not is_success(resp.status):
-        raise DirectClientException('Object', 'PUT',
-                                    node, part, path, resp)
-    return resp.getheader('etag').strip('"')
+    return normalize_etag(resp.getheader('etag'))
 
 
 def direct_post_object(node, part, account, container, name, headers,
@@ -453,7 +593,7 @@ def direct_post_object(node, part, account, container, name, headers,
     :param response_timeout: timeout in seconds for getting the response
     :raises ClientException: HTTP POST request failed
     """
-    path = '/%s/%s/%s' % (account, container, name)
+    path = _make_path(account, container, name)
     _make_req(node, part, 'POST', path, gen_headers(headers, True),
               'Object', conn_timeout, response_timeout)
 
@@ -478,7 +618,7 @@ def direct_delete_object(node, part, account, container, obj,
     headers = gen_headers(headers, add_ts='x-timestamp' not in (
         k.lower() for k in headers))
 
-    path = '/%s/%s/%s' % (account, container, obj)
+    path = _make_path(account, container, obj)
     _make_req(node, part, 'DELETE', path, headers,
               'Object', conn_timeout, response_timeout)
 
@@ -488,6 +628,9 @@ def direct_get_suffix_hashes(node, part, suffixes, conn_timeout=5,
     """
     Get suffix hashes directly from the object server.
 
+    Note that unlike other ``direct_client`` functions, this one defaults
+    to using the replication network to make requests.
+
     :param node: node dictionary from the ring
     :param part: partition the container is on
     :param conn_timeout: timeout in seconds for establishing the connection
@@ -499,9 +642,11 @@ def direct_get_suffix_hashes(node, part, suffixes, conn_timeout=5,
     if headers is None:
         headers = {}
 
+    headers.setdefault(USE_REPLICATION_NETWORK_HEADER, 'true')
+    ip, port = get_ip_port(node, headers)
     path = '/%s' % '-'.join(suffixes)
     with Timeout(conn_timeout):
-        conn = http_connect(node['replication_ip'], node['replication_port'],
+        conn = http_connect(ip, port,
                             node['device'], part, 'REPLICATE', path,
                             headers=gen_headers(headers))
     with Timeout(response_timeout):
@@ -558,3 +703,31 @@ def retry(func, *args, **kwargs):
                               http_device=args[0]['device'])
     else:
         raise ClientException('Raise too many retries')
+
+
+def direct_get_recon(node, recon_command, conn_timeout=5, response_timeout=15,
+                     headers=None):
+    """
+    Get recon json directly from the storage server.
+
+    :param node: node dictionary from the ring
+    :param recon_command: recon string (post /recon/)
+    :param conn_timeout: timeout in seconds for establishing the connection
+    :param response_timeout: timeout in seconds for getting the response
+    :param headers: dict to be passed into HTTPConnection headers
+    :returns: deserialized json response
+    :raises DirectClientReconException: HTTP GET request failed
+    """
+    if headers is None:
+        headers = {}
+
+    ip, port = get_ip_port(node, headers)
+    path = '/recon/%s' % recon_command
+    with Timeout(conn_timeout):
+        conn = http_connect_raw(ip, port, 'GET', path,
+                                headers=gen_headers(headers))
+    with Timeout(response_timeout):
+        resp = conn.getresponse()
+    if not is_success(resp.status):
+        raise DirectClientReconException('GET', node, path, resp)
+    return json.loads(resp.read())
