@@ -23,8 +23,12 @@ import time
 import subprocess
 import re
 import six
-from swift import gettext_ as _
 import tempfile
+try:
+    from shutil import which
+except ImportError:
+    # py2
+    from distutils.spawn import find_executable as which
 
 from swift.common.utils import search_tree, remove_file, write_file, readconf
 from swift.common.exceptions import InvalidPidFileException
@@ -67,22 +71,22 @@ def setup_env():
         resource.setrlimit(resource.RLIMIT_NOFILE,
                            (MAX_DESCRIPTORS, MAX_DESCRIPTORS))
     except ValueError:
-        print(_("WARNING: Unable to modify file descriptor limit.  "
-                "Running as non-root?"))
+        print("WARNING: Unable to modify file descriptor limit.  "
+              "Running as non-root?")
 
     try:
         resource.setrlimit(resource.RLIMIT_DATA,
                            (MAX_MEMORY, MAX_MEMORY))
     except ValueError:
-        print(_("WARNING: Unable to modify memory limit.  "
-                "Running as non-root?"))
+        print("WARNING: Unable to modify memory limit.  "
+              "Running as non-root?")
 
     try:
         resource.setrlimit(resource.RLIMIT_NPROC,
                            (MAX_PROCS, MAX_PROCS))
     except ValueError:
-        print(_("WARNING: Unable to modify max process limit.  "
-                "Running as non-root?"))
+        print("WARNING: Unable to modify max process limit.  "
+              "Running as non-root?")
 
     # Set PYTHON_EGG_CACHE if it isn't already set
     os.environ.setdefault('PYTHON_EGG_CACHE', tempfile.gettempdir())
@@ -98,8 +102,10 @@ def command(func):
     func.publicly_accessible = True
 
     @functools.wraps(func)
-    def wrapped(*a, **kw):
-        rv = func(*a, **kw)
+    def wrapped(self, *a, **kw):
+        rv = func(self, *a, **kw)
+        if len(self.servers) == 0:
+            return 1
         return 1 if rv else 0
     return wrapped
 
@@ -174,6 +180,50 @@ def kill_group(pid, sig):
     os.kill(-pid, sig)
 
 
+def get_child_pids(pid):
+    """
+    Get the current set of all child PIDs for a PID.
+
+    :param pid: process id
+    """
+    output = subprocess.check_output(
+        ["ps", "--ppid", str(pid), "--no-headers", "-o", "pid"])
+    return {int(pid) for pid in output.split()}
+
+
+def format_server_name(servername):
+    """
+    Formats server name as swift compatible server names
+    E.g. swift-object-server
+
+    :param servername: server name
+    :returns: swift compatible server name and its binary name
+    """
+    if '.' in servername:
+        servername = servername.split('.', 1)[0]
+    if '-' not in servername:
+        servername = '%s-server' % servername
+    cmd = 'swift-%s' % servername
+    return servername, cmd
+
+
+def verify_server(server):
+    """
+    Check whether the server is among swift servers or not, and also
+    checks whether the server's binaries are installed or not.
+
+    :param server: name of the server
+    :returns: True, when the server name is valid and its binaries are found.
+              False, otherwise.
+    """
+    if not server:
+        return False
+    _, cmd = format_server_name(server)
+    if which(cmd) is None:
+        return False
+    return True
+
+
 class UnknownCommandError(Exception):
     pass
 
@@ -203,7 +253,8 @@ class Manager(object):
 
         self.servers = set()
         for name in self.server_names:
-            self.servers.add(Server(name, run_dir))
+            if verify_server(name):
+                self.servers.add(Server(name, run_dir))
 
     def __iter__(self):
         return iter(self.servers)
@@ -240,7 +291,7 @@ class Manager(object):
                 try:
                     status += server.interact(**kwargs)
                 except KeyboardInterrupt:
-                    print(_('\nuser quit'))
+                    print('\nuser quit')
                     self.stop(**kwargs)
                     break
         elif kwargs.get('wait', True):
@@ -277,7 +328,7 @@ class Manager(object):
         for server in self.servers:
             signaled_pids = server.stop(**kwargs)
             if not signaled_pids:
-                print(_('No %s running') % server)
+                print('No %s running' % server)
             else:
                 server_pids[server] = signaled_pids
 
@@ -290,7 +341,7 @@ class Manager(object):
         for server, killed_pid in watch_server_pids(server_pids,
                                                     interval=kill_wait,
                                                     **kwargs):
-            print(_("%(server)s (%(pid)s) appears to have stopped") %
+            print("%(server)s (%(pid)s) appears to have stopped" %
                   {'server': server, 'pid': killed_pid})
             killed_pids.add(killed_pid)
             if not killed_pids.symmetric_difference(signaled_pids):
@@ -303,15 +354,15 @@ class Manager(object):
             if not killed_pids.issuperset(pids):
                 # some pids of this server were not killed
                 if kill_after_timeout:
-                    print(_('Waited %(kill_wait)s seconds for %(server)s '
-                            'to die; killing') %
+                    print('Waited %(kill_wait)s seconds for %(server)s '
+                          'to die; killing' %
                           {'kill_wait': kill_wait, 'server': server})
                     # Send SIGKILL to all remaining pids
                     for pid in set(pids.keys()) - killed_pids:
-                        print(_('Signal %(server)s  pid: %(pid)s  signal: '
-                                '%(signal)s') % {'server': server,
-                                                 'pid': pid,
-                                                 'signal': signal.SIGKILL})
+                        print('Signal %(server)s  pid: %(pid)s  signal: '
+                              '%(signal)s' % {'server': server,
+                                              'pid': pid,
+                                              'signal': signal.SIGKILL})
                         # Send SIGKILL to process group
                         try:
                             kill_group(pid, signal.SIGKILL)
@@ -320,8 +371,8 @@ class Manager(object):
                             if e.errno != errno.ESRCH:
                                 raise
                 else:
-                    print(_('Waited %(kill_wait)s seconds for %(server)s '
-                            'to die; giving up') %
+                    print('Waited %(kill_wait)s seconds for %(server)s '
+                          'to die; giving up' %
                           {'kill_wait': kill_wait, 'server': server})
         return 1
 
@@ -377,7 +428,17 @@ class Manager(object):
         for server in self.servers:
             signaled_pids = server.stop(**kwargs)
             if not signaled_pids:
-                print(_('No %s running') % server)
+                print('No %s running' % server)
+                status += 1
+        return status
+
+    def kill_child_pids(self, **kwargs):
+        """kill child pids, optionally servicing accepted connections"""
+        status = 0
+        for server in self.servers:
+            signaled_pids = server.kill_child_pids(**kwargs)
+            if not signaled_pids:
+                print('No %s running' % server)
                 status += 1
         return status
 
@@ -436,10 +497,8 @@ class Server(object):
             self.server, self.conf = self.server.rsplit('.', 1)
         else:
             self.conf = None
-        if '-' not in self.server:
-            self.server = '%s-server' % self.server
+        self.server, self.cmd = format_server_name(self.server)
         self.type = self.server.rsplit('-', 1)[0]
-        self.cmd = 'swift-%s' % self.server
         self.procs = []
         self.run_dir = run_dir
 
@@ -539,7 +598,7 @@ class Server(object):
 
         def dump_found_configs():
             if found_conf_files:
-                print(_('Found configs:'))
+                print('Found configs:')
             for i, conf_file in enumerate(found_conf_files):
                 print('  %d) %s' % (i + 1, conf_file))
 
@@ -547,18 +606,18 @@ class Server(object):
             # maybe there's a config file(s) out there, but I couldn't find it!
             if not kwargs.get('quiet'):
                 if number:
-                    print(_('Unable to locate config number %(number)s for'
-                            ' %(server)s') %
+                    print('Unable to locate config number %(number)s for'
+                          ' %(server)s' %
                           {'number': number, 'server': self.server})
                 else:
-                    print(_('Unable to locate config for %s') % self.server)
+                    print('Unable to locate config for %s' % self.server)
             if kwargs.get('verbose') and not kwargs.get('quiet'):
                 dump_found_configs()
         elif any(["object-expirer" in name for name in conf_files]) and \
                 not kwargs.get('quiet'):
-            print(_("WARNING: object-expirer.conf is deprecated. "
-                    "Move object-expirers' configuration into "
-                    "object-server.conf."))
+            print("WARNING: object-expirer.conf is deprecated. "
+                  "Move object-expirers' configuration into "
+                  "object-server.conf.")
             if kwargs.get('verbose'):
                 dump_found_configs()
 
@@ -594,6 +653,32 @@ class Server(object):
                 pid = None
             yield pid_file, pid
 
+    def _signal_pid(self, sig, pid, pid_file, verbose):
+        try:
+            if sig != signal.SIG_DFL:
+                print('Signal %(server)s  pid: %(pid)s  signal: '
+                      '%(signal)s' %
+                      {'server': self.server, 'pid': pid, 'signal': sig})
+            safe_kill(pid, sig, 'swift-%s' % self.server)
+        except InvalidPidFileException:
+            if verbose:
+                print('Removing pid file %(pid_file)s with wrong pid '
+                      '%(pid)d' % {'pid_file': pid_file, 'pid': pid})
+            remove_file(pid_file)
+            return False
+        except OSError as e:
+            if e.errno == errno.ESRCH:
+                # pid does not exist
+                if verbose:
+                    print("Removing stale pid file %s" % pid_file)
+                remove_file(pid_file)
+            elif e.errno == errno.EPERM:
+                print("No permission to signal PID %d" % pid)
+            return False
+        else:
+            # process exists
+            return True
+
     def signal_pids(self, sig, **kwargs):
         """Send a signal to pids for this server
 
@@ -605,33 +690,32 @@ class Server(object):
         pids = {}
         for pid_file, pid in self.iter_pid_files(**kwargs):
             if not pid:  # Catches None and 0
-                print(_('Removing pid file %s with invalid pid') % pid_file)
+                print('Removing pid file %s with invalid pid' % pid_file)
                 remove_file(pid_file)
                 continue
-            try:
-                if sig != signal.SIG_DFL:
-                    print(_('Signal %(server)s  pid: %(pid)s  signal: '
-                            '%(signal)s') %
-                          {'server': self.server, 'pid': pid, 'signal': sig})
-                safe_kill(pid, sig, 'swift-%s' % self.server)
-            except InvalidPidFileException as e:
-                if kwargs.get('verbose'):
-                    print(_('Removing pid file %(pid_file)s with wrong pid '
-                            '%(pid)d') % {'pid_file': pid_file, 'pid': pid})
-                remove_file(pid_file)
-            except OSError as e:
-                if e.errno == errno.ESRCH:
-                    # pid does not exist
-                    if kwargs.get('verbose'):
-                        print(_("Removing stale pid file %s") % pid_file)
-                    remove_file(pid_file)
-                elif e.errno == errno.EPERM:
-                    print(_("No permission to signal PID %d") % pid)
-            else:
-                # process exists
+            if self._signal_pid(sig, pid, pid_file, kwargs.get('verbose')):
                 pids[pid] = pid_file
         return pids
 
+    def signal_children(self, sig, **kwargs):
+        """Send a signal to child pids for this server
+
+        :param sig: signal to send
+
+        :returns: a dict mapping pids (ints) to pid_files (paths)
+
+        """
+        pids = {}
+        for pid_file, pid in self.iter_pid_files(**kwargs):
+            if not pid:  # Catches None and 0
+                print('Removing pid file %s with invalid pid' % pid_file)
+                remove_file(pid_file)
+                continue
+            for pid in get_child_pids(pid):
+                if self._signal_pid(sig, pid, pid_file, kwargs.get('verbose')):
+                    pids[pid] = pid_file
+        return pids
+
     def get_running_pids(self, **kwargs):
         """Get running pids
 
@@ -659,6 +743,25 @@ class Server(object):
             sig = signal.SIGTERM
         return self.signal_pids(sig, **kwargs)
 
+    def kill_child_pids(self, **kwargs):
+        """Kill child pids, leaving server overseer to respawn them
+
+        :param graceful: if True, attempt SIGHUP on supporting servers
+        :param seamless: if True, attempt SIGUSR1 on supporting servers
+
+        :returns: a dict mapping pids (ints) to pid_files (paths)
+
+        """
+        graceful = kwargs.get('graceful')
+        seamless = kwargs.get('seamless')
+        if graceful and self.server in GRACEFUL_SHUTDOWN_SERVERS:
+            sig = signal.SIGHUP
+        elif seamless and self.server in SEAMLESS_SHUTDOWN_SERVERS:
+            sig = signal.SIGUSR1
+        else:
+            sig = signal.SIGTERM
+        return self.signal_children(sig, **kwargs)
+
     def status(self, pids=None, **kwargs):
         """Display status of server
 
@@ -675,15 +778,15 @@ class Server(object):
                 kwargs['quiet'] = True
                 conf_files = self.conf_files(**kwargs)
                 if conf_files:
-                    print(_("%(server)s #%(number)d not running (%(conf)s)") %
+                    print("%(server)s #%(number)d not running (%(conf)s)" %
                           {'server': self.server, 'number': number,
                            'conf': conf_files[0]})
             else:
-                print(_("No %s running") % self.server)
+                print("No %s running" % self.server)
             return 1
         for pid, pid_file in pids.items():
             conf_file = self.get_conf_file_name(pid_file)
-            print(_("%(server)s running (%(pid)s - %(conf)s)") %
+            print("%(server)s running (%(pid)s - %(conf)s)" %
                   {'server': self.server, 'pid': pid, 'conf': conf_file})
         return 0
 
@@ -791,16 +894,16 @@ class Server(object):
             # any unstarted instances
             if conf_file in conf_files:
                 already_started = True
-                print(_("%(server)s running (%(pid)s - %(conf)s)") %
+                print("%(server)s running (%(pid)s - %(conf)s)" %
                       {'server': self.server, 'pid': pid, 'conf': conf_file})
             elif not kwargs.get('number', 0):
                 already_started = True
-                print(_("%(server)s running (%(pid)s - %(pid_file)s)") %
+                print("%(server)s running (%(pid)s - %(pid_file)s)" %
                       {'server': self.server, 'pid': pid,
                        'pid_file': pid_file})
 
         if already_started:
-            print(_("%s already started...") % self.server)
+            print("%s already started..." % self.server)
             return {}
 
         if self.server not in START_ONCE_SERVERS:
@@ -809,16 +912,16 @@ class Server(object):
         pids = {}
         for conf_file in conf_files:
             if kwargs.get('once'):
-                msg = _('Running %s once') % self.server
+                msg = 'Running %s once' % self.server
             else:
-                msg = _('Starting %s') % self.server
+                msg = 'Starting %s' % self.server
             print('%s...(%s)' % (msg, conf_file))
             try:
                 pid = self.spawn(conf_file, **kwargs)
             except OSError as e:
                 if e.errno == errno.ENOENT:
                     # TODO(clayg): should I check if self.cmd exists earlier?
-                    print(_("%s does not exist") % self.cmd)
+                    print("%s does not exist" % self.cmd)
                     break
                 else:
                     raise
