@@ -122,16 +122,17 @@ import json
 
 import six
 
-from hashlib import md5
 from swift.common import constraints
 from swift.common.exceptions import ListingIterError, SegmentError
 from swift.common.http import is_success
-from swift.common.swob import Request, Response, \
+from swift.common.swob import Request, Response, HTTPException, \
     HTTPRequestedRangeNotSatisfiable, HTTPBadRequest, HTTPConflict, \
-    str_to_wsgi, wsgi_to_str, wsgi_quote, wsgi_unquote
+    str_to_wsgi, wsgi_to_str, wsgi_quote, wsgi_unquote, normalize_etag
 from swift.common.utils import get_logger, \
-    RateLimitedIterator, quote, close_if_possible, closing_if_possible
-from swift.common.request_helpers import SegmentedIterable
+    RateLimitedIterator, quote, close_if_possible, closing_if_possible, \
+    drain_and_close, md5
+from swift.common.request_helpers import SegmentedIterable, \
+    update_ignore_range_header
 from swift.common.wsgi import WSGIContext, make_subrequest, load_app_config
 
 
@@ -331,9 +332,9 @@ class GetContext(WSGIContext):
         if have_complete_listing:
             response_headers = [(h, v) for h, v in response_headers
                                 if h.lower() != "etag"]
-            etag = md5()
+            etag = md5(usedforsecurity=False)
             for seg_dict in segments:
-                etag.update(seg_dict['hash'].strip('"').encode('utf8'))
+                etag.update(normalize_etag(seg_dict['hash']).encode('utf8'))
             response_headers.append(('Etag', '"%s"' % etag.hexdigest()))
 
         app_iter = None
@@ -353,6 +354,8 @@ class GetContext(WSGIContext):
 
             try:
                 app_iter.validate_first_segment()
+            except HTTPException as err_resp:
+                return err_resp
             except (SegmentError, ListingIterError):
                 return HTTPConflict(request=req)
 
@@ -369,11 +372,16 @@ class GetContext(WSGIContext):
 
         Otherwise, simply pass it through.
         """
+        update_ignore_range_header(req, 'X-Object-Manifest')
         resp_iter = self._app_call(req.environ)
 
         # make sure this response is for a dynamic large object manifest
         for header, value in self._response_headers:
             if (header.lower() == 'x-object-manifest'):
+                content_length = self._response_header_value('content-length')
+                if content_length is not None and int(content_length) < 1024:
+                    # Go ahead and consume small bodies
+                    drain_and_close(resp_iter)
                 close_if_possible(resp_iter)
                 response = self.get_or_head_response(
                     req, wsgi_to_str(wsgi_unquote(value)))
