@@ -23,7 +23,8 @@ from swift.common import swob
 from swift.common.http import HTTP_OK
 from swift.common.middleware.versioned_writes.object_versioning import \
     DELETE_MARKER_CONTENT_TYPE
-from swift.common.utils import json, public, config_true_value, Timestamp
+from swift.common.utils import json, public, config_true_value, Timestamp, \
+    cap_length
 from swift.common.registry import get_swift_info
 
 from swift.common.middleware.s3api.controllers.base import Controller
@@ -32,8 +33,9 @@ from swift.common.middleware.s3api.etree import Element, SubElement, \
 from swift.common.middleware.s3api.s3response import \
     HTTPOk, S3NotImplemented, InvalidArgument, \
     MalformedXML, InvalidLocationConstraint, NoSuchBucket, \
-    BucketNotEmpty, InternalError, ServiceUnavailable, NoSuchKey
-from swift.common.middleware.s3api.utils import MULTIUPLOAD_SUFFIX
+    BucketNotEmpty, VersionedBucketNotEmpty, InternalError, \
+    ServiceUnavailable, NoSuchKey
+from swift.common.middleware.s3api.utils import MULTIUPLOAD_SUFFIX, S3Timestamp
 
 MAX_PUT_BUCKET_BODY_SIZE = 10240
 
@@ -53,7 +55,10 @@ class BucketController(Controller):
         try:
             resp = req.get_response(self.app, 'HEAD')
             if int(resp.sw_headers['X-Container-Object-Count']) > 0:
-                raise BucketNotEmpty()
+                if resp.sw_headers.get('X-Container-Sysmeta-Versions-Enabled'):
+                    raise VersionedBucketNotEmpty()
+                else:
+                    raise BucketNotEmpty()
             # FIXME: This extra HEAD saves unexpected segment deletion
             # but if a complete multipart upload happen while cleanup
             # segment container below, completed object may be missing its
@@ -287,7 +292,7 @@ class BucketController(Controller):
             contents = SubElement(elem, 'Contents')
             SubElement(contents, 'Key').text = name
         SubElement(contents, 'LastModified').text = \
-            o['last_modified'][:-3] + 'Z'
+            S3Timestamp.from_isoformat(o['last_modified']).s3xmlformat
         if contents.tag != 'DeleteMarker':
             if 's3_etag' in o:
                 # New-enough MUs are already in the right format
@@ -339,7 +344,12 @@ class BucketController(Controller):
 
         resp = req.get_response(self.app, query=query)
 
-        objects = json.loads(resp.body)
+        try:
+            objects = json.loads(resp.body)
+        except (TypeError, ValueError):
+            self.logger.error('Got non-JSON response trying to list %s: %r',
+                              req.path, cap_length(resp.body, 60))
+            raise
 
         is_truncated = max_keys > 0 and len(objects) > max_keys
         objects = objects[:max_keys]
