@@ -64,7 +64,7 @@ def translate_swift_to_s3(key, val):
 
     if _key.startswith('x-object-meta-'):
         return translate_meta_key(_key), val
-    elif _key in ('content-length', 'content-type',
+    elif _key in ('accept-ranges', 'content-length', 'content-type',
                   'content-range', 'content-encoding',
                   'content-disposition', 'content-language',
                   'etag', 'last-modified', 'x-robots-tag',
@@ -111,6 +111,7 @@ class S3Response(S3ResponseBase, swob.Response):
     headers instead of Swift's HeaderKeyDict.  This also translates Swift
     specific headers to S3 headers.
     """
+
     def __init__(self, *args, **kwargs):
         swob.Response.__init__(self, *args, **kwargs)
 
@@ -229,16 +230,17 @@ class ErrorResponse(S3ResponseBase, swob.HTTPException):
     _code = ''
     xml_declaration = True
 
-    def __init__(self, msg=None, *args, **kwargs):
+    def __init__(self, msg=None, reason=None, *args, **kwargs):
         if msg:
             self._msg = msg
         if not self._code:
             self._code = self.__class__.__name__
+        self.reason = reason
 
         self.info = kwargs.copy()
         for reserved_key in ('headers', 'body'):
             if self.info.get(reserved_key):
-                del(self.info[reserved_key])
+                del (self.info[reserved_key])
 
         swob.HTTPException.__init__(
             self, status=kwargs.pop('status', self._status),
@@ -247,6 +249,14 @@ class ErrorResponse(S3ResponseBase, swob.HTTPException):
             **kwargs)
         self.headers = HeaderKeyDict(self.headers)
 
+    @property
+    def metric_name(self):
+        parts = [str(self.status_int), self._code]
+        if self.reason:
+            parts.append(self.reason)
+        metric = '.'.join(parts)
+        return metric.replace(' ', '_')
+
     def _body_iter(self):
         error_elem = Element('Error')
         SubElement(error_elem, 'Code').text = self._code
@@ -334,6 +344,12 @@ class BucketNotEmpty(ErrorResponse):
     _msg = 'The bucket you tried to delete is not empty'
 
 
+class VersionedBucketNotEmpty(BucketNotEmpty):
+    _msg = 'The bucket you tried to delete is not empty. ' \
+           'You must delete all versions in the bucket.'
+    _code = 'BucketNotEmpty'
+
+
 class CredentialsNotSupported(ErrorResponse):
     _status = '400 Bad Request'
     _msg = 'This request does not support credentials.'
@@ -598,6 +614,16 @@ class NoSuchKey(ErrorResponse):
         ErrorResponse.__init__(self, msg, key=key, *args, **kwargs)
 
 
+class ObjectLockConfigurationNotFoundError(ErrorResponse):
+    _status = '404 Not found'
+    _msg = 'Object Lock configuration does not exist for this bucket'
+
+    def __init__(self, bucket, msg=None, *args, **kwargs):
+        if not bucket:
+            raise InternalError()
+        ErrorResponse.__init__(self, msg, bucket_name=bucket, *args, **kwargs)
+
+
 class NoSuchLifecycleConfiguration(ErrorResponse):
     _status = '404 Not Found'
     _msg = 'The lifecycle configuration does not exist. .'
