@@ -152,12 +152,13 @@ from cgi import parse_header
 from six.moves.urllib.parse import unquote
 
 from swift.common.constraints import MAX_FILE_SIZE, valid_api_version, \
-    ACCOUNT_LISTING_LIMIT
+    ACCOUNT_LISTING_LIMIT, CONTAINER_LISTING_LIMIT
 from swift.common.http import is_success, is_client_error, HTTP_NOT_FOUND, \
     HTTP_CONFLICT
 from swift.common.request_helpers import get_sys_meta_prefix, \
     copy_header_subset, get_reserved_name, split_reserved_name, \
     constrain_req_limit
+from swift.common.middleware import app_property
 from swift.common.middleware.symlink import TGT_OBJ_SYMLINK_HDR, \
     TGT_ETAG_SYSMETA_SYMLINK_HDR, SYMLOOP_EXTEND, ALLOW_RESERVED_NAMES, \
     TGT_BYTES_SYSMETA_SYMLINK_HDR, TGT_ACCT_SYMLINK_HDR
@@ -329,16 +330,17 @@ class ObjectContext(ObjectVersioningContext):
 
         # do the write
         put_resp = put_req.get_response(self.app)
-        drain_and_close(put_resp)
         close_if_possible(put_req.environ['wsgi.input'])
 
         if put_resp.status_int == HTTP_NOT_FOUND:
+            drain_and_close(put_resp)
             raise HTTPInternalServerError(
                 request=req, content_type='text/plain',
                 body=b'The versions container does not exist. You may '
                      b'want to re-enable object versioning.')
 
         self._check_response_error(req, put_resp)
+        drain_and_close(put_resp)
         put_bytes = byte_counter.bytes_read
         # N.B. this is essentially the same hack that symlink does in
         # _validate_etag_and_update_sysmeta to deal with SLO
@@ -392,12 +394,13 @@ class ObjectContext(ObjectVersioningContext):
         """
         if is_success(resp.status_int):
             return
+        body = resp.body
         drain_and_close(resp)
         if is_client_error(resp.status_int):
             # missing container or bad permissions
             if resp.status_int == 404:
                 raise HTTPPreconditionFailed(request=req)
-            raise HTTPException(body=resp.body, status=resp.status,
+            raise HTTPException(body=body, status=resp.status,
                                 headers=resp.headers)
         # could not version the data, bail
         raise HTTPServiceUnavailable(request=req)
@@ -1163,12 +1166,22 @@ class ContainerContext(ObjectVersioningContext):
             params['prefix'] = get_reserved_name(params['prefix'])
 
         # NB: no end_marker support (yet)
-        versions_req.params = {
-            k: params.get(k, '')
-            for k in ('prefix', 'marker', 'limit', 'delimiter', 'reverse')}
-        versions_resp = versions_req.get_response(self.app)
+        if get_container_info(versions_req.environ, self.app,
+                              swift_source='OV')['status'] == 404:
+            # we don't usually like to LBYL like this, but 404s tend to be
+            # expensive (since we check all primaries and a bunch of handoffs)
+            # and we expect this to be a reasonably common way to listing
+            # objects since it's more complete from the user's perspective
+            # (see also: s3api and that client ecosystem)
+            versions_resp = None
+        else:
+            versions_req.params = {
+                k: params.get(k, '') for k in (
+                    'prefix', 'marker', 'limit', 'delimiter', 'reverse')}
+            versions_resp = versions_req.get_response(self.app)
 
-        if versions_resp.status_int == HTTP_NOT_FOUND:
+        if versions_resp is None \
+                or versions_resp.status_int == HTTP_NOT_FOUND:
             subdir_listing = [{'subdir': s} for s in subdir_set]
             broken_listing = []
             for item in current_versions.values():
@@ -1191,7 +1204,7 @@ class ContainerContext(ObjectVersioningContext):
                     'hash': item['hash'],
                     'last_modified': item['last_modified'],
                 })
-            limit = constrain_req_limit(req, ACCOUNT_LISTING_LIMIT)
+            limit = constrain_req_limit(req, CONTAINER_LISTING_LIMIT)
             body = build_listing(
                 null_listing, subdir_listing, broken_listing,
                 reverse=config_true_value(params.get('reverse', 'no')),
@@ -1256,7 +1269,7 @@ class ContainerContext(ObjectVersioningContext):
                         'last_modified': item['last_modified'],
                     })
 
-                limit = constrain_req_limit(req, ACCOUNT_LISTING_LIMIT)
+                limit = constrain_req_limit(req, CONTAINER_LISTING_LIMIT)
                 body = build_listing(
                     null_listing, versions_listing,
                     subdir_listing, broken_listing,
@@ -1377,6 +1390,12 @@ class ObjectVersioningMiddleware(object):
         self.conf = conf
         self.logger = get_logger(conf, log_route='object_versioning')
 
+    # Pass these along so get_container_info will have the configured
+    # odds to skip cache
+    _pipeline_final_app = app_property('_pipeline_final_app')
+    _pipeline_request_logging_app = app_property(
+        '_pipeline_request_logging_app')
+
     def account_request(self, req, api_version, account, start_response):
         account_ctx = AccountContext(self.app, self.logger)
         if req.method == 'GET':
