@@ -31,8 +31,8 @@ from swift.common.bufferedhttp import http_connect
 from swift.common.exceptions import ConnectionTimeout, LockTimeout
 from swift.common.ring import Ring
 from swift.common.utils import get_logger, config_true_value, \
-    dump_recon_cache, majority_size, Timestamp, ratelimit_sleep, \
-    eventlet_monkey_patch
+    dump_recon_cache, majority_size, Timestamp, EventletRateLimiter, \
+    eventlet_monkey_patch, node_to_string
 from swift.common.daemon import Daemon
 from swift.common.http import is_success, HTTP_INTERNAL_SERVER_ERROR
 from swift.common.recon import RECON_CONTAINER_FILE, DEFAULT_RECON_CACHE_PATH
@@ -59,10 +59,10 @@ class ContainerUpdater(Daemon):
                 float(conf.get('slowdown', '0.01')) + 0.01)
         else:
             containers_per_second = 50
-        self.containers_running_time = 0
         self.max_containers_per_second = \
             float(conf.get('containers_per_second',
                            containers_per_second))
+        self.rate_limiter = EventletRateLimiter(self.max_containers_per_second)
         self.node_timeout = float(conf.get('node_timeout', 3))
         self.conn_timeout = float(conf.get('conn_timeout', 0.5))
         self.no_changes = 0
@@ -158,6 +158,7 @@ class ContainerUpdater(Daemon):
                     pid2filename[pid] = tmpfilename
                 else:
                     signal.signal(signal.SIGTERM, signal.SIG_DFL)
+                    os.environ.pop('NOTIFY_SOCKET', None)
                     eventlet_monkey_patch()
                     self.no_changes = 0
                     self.successes = 0
@@ -226,9 +227,7 @@ class ContainerUpdater(Daemon):
                         self.logger.exception(
                             "Error processing container %s: %s", dbfile, e)
 
-                    self.containers_running_time = ratelimit_sleep(
-                        self.containers_running_time,
-                        self.max_containers_per_second)
+                    self.rate_limiter.wait()
 
     def process_container(self, dbfile):
         """
@@ -342,9 +341,8 @@ class ContainerUpdater(Daemon):
                     node['device'], part, 'PUT', container, headers=headers)
             except (Exception, Timeout):
                 self.logger.exception(
-                    'ERROR account update failed with '
-                    '%(replication_ip)s:%(replication_port)s/%(device)s '
-                    '(will retry later): ', node)
+                    'ERROR account update failed with %s (will retry later):',
+                    node_to_string(node, replication=True))
                 return HTTP_INTERNAL_SERVER_ERROR
         with Timeout(self.node_timeout):
             try:
@@ -354,9 +352,8 @@ class ContainerUpdater(Daemon):
             except (Exception, Timeout):
                 if self.logger.getEffectiveLevel() <= logging.DEBUG:
                     self.logger.exception(
-                        'Exception with '
-                        '%(replication_ip)s:%(replication_port)s/%(device)s',
-                        node)
+                        'Exception with %s',
+                        node_to_string(node, replication=True))
                 return HTTP_INTERNAL_SERVER_ERROR
             finally:
                 conn.close()
