@@ -15,18 +15,18 @@
 
 """ In-Memory Disk File Interface for Swift Object Server"""
 
+import io
 import time
-import hashlib
 from contextlib import contextmanager
 
 from eventlet import Timeout
-from six import moves
 
 from swift.common.utils import Timestamp
 from swift.common.exceptions import DiskFileQuarantined, DiskFileNotExist, \
     DiskFileCollision, DiskFileDeleted, DiskFileNotOpen
 from swift.common.request_helpers import is_sys_meta
 from swift.common.swob import multi_range_iterator
+from swift.common.utils import md5
 from swift.obj.diskfile import DATAFILE_SYSTEM_META, RESERVED_DATAFILE_META
 
 
@@ -35,7 +35,7 @@ class InMemoryFileSystem(object):
     A very simplistic in-memory file system scheme.
 
     There is one dictionary mapping a given object name to a tuple. The first
-    entry in the tuple is the cStringIO buffer representing the file contents,
+    entry in the tuple is the BytesIO buffer representing the file contents,
     the second entry is the metadata dictionary.
     """
 
@@ -47,7 +47,7 @@ class InMemoryFileSystem(object):
         Return back an file-like object and its metadata
 
         :param name: standard object name
-        :return: (fp, metadata) fp is `StringIO` in-memory representation
+        :return: (fp, metadata) fp is ``BytesIO`` in-memory representation
                                 object (or None). metadata is a dictionary
                                 of metadata (or None)
         """
@@ -63,7 +63,7 @@ class InMemoryFileSystem(object):
         Store object into memory
 
         :param name: standard object name
-        :param fp: `StringIO` in-memory representation object
+        :param fp: ``BytesIO`` in-memory representation object
         :param metadata: dictionary of metadata to be written
         """
         self._filesystem[name] = (fp, metadata)
@@ -103,16 +103,16 @@ class DiskFileWriter(object):
         self._name = name
         self._fp = None
         self._upload_size = 0
-        self._chunks_etag = hashlib.md5()
+        self._chunks_etag = md5(usedforsecurity=False)
 
     def open(self):
         """
         Prepare to accept writes.
 
-        Create a new ``StringIO`` object for a started-but-not-yet-finished
+        Create a new ``BytesIO`` object for a started-but-not-yet-finished
         PUT.
         """
-        self._fp = moves.cStringIO()
+        self._fp = io.BytesIO()
         return self
 
     def close(self):
@@ -125,7 +125,7 @@ class DiskFileWriter(object):
 
     def write(self, chunk):
         """
-        Write a chunk of data into the `StringIO` object.
+        Write a chunk of data into the ``BytesIO`` object.
 
         :param chunk: the chunk of data to write as a string object
         """
@@ -144,7 +144,7 @@ class DiskFileWriter(object):
     def put(self, metadata):
         """
         Make the final association in the in-memory file system for this name
-        with the `StringIO` object.
+        with the ``BytesIO`` object.
 
         :param metadata: dictionary of metadata to be written
         """
@@ -197,7 +197,7 @@ class DiskFileReader(object):
             self._read_to_eof = False
             if self._fp.tell() == 0:
                 self._started_at_0 = True
-                self._iter_etag = hashlib.md5()
+                self._iter_etag = md5(usedforsecurity=False)
             while True:
                 chunk = self._fp.read()
                 if chunk:
@@ -426,13 +426,14 @@ class DiskFile(object):
         with self.open(current_time=current_time):
             return self.get_metadata()
 
-    def reader(self, keep_cache=False):
+    def reader(self, keep_cache=False, cooperative_period=0):
         """
         Return a swift.common.swob.Response class compatible "app_iter"
         object. The responsibility of closing the open file is passed to the
         DiskFileReader object.
 
         :param keep_cache:
+        :param cooperative_period:
         """
         dr = DiskFileReader(self._name, self._fp,
                             int(self._metadata['Content-Length']),
