@@ -33,7 +33,8 @@ from swift.common.ring import Ring
 from swift.common.utils import get_logger, renamer, write_pickle, \
     dump_recon_cache, config_true_value, RateLimitedIterator, split_path, \
     eventlet_monkey_patch, get_redirect_data, ContextPool, hash_path, \
-    non_negative_float, config_positive_int_value, non_negative_int
+    non_negative_float, config_positive_int_value, non_negative_int, \
+    EventletRateLimiter, node_to_string
 from swift.common.daemon import Daemon
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.storage_policy import split_policy_string, PolicyError
@@ -43,16 +44,17 @@ from swift.common.http import is_success, HTTP_INTERNAL_SERVER_ERROR, \
     HTTP_MOVED_PERMANENTLY
 
 
-class RateLimiterBucket(object):
-    def __init__(self, update_delta):
-        self.update_delta = update_delta
-        self.last_time = 0
+class RateLimiterBucket(EventletRateLimiter):
+    """
+    Extends EventletRateLimiter to also maintain a deque of items that have
+    been deferred due to rate-limiting, and to provide a comparator for sorting
+    instanced by readiness.
+    """
+    def __init__(self, max_updates_per_second):
+        super(RateLimiterBucket, self).__init__(max_updates_per_second,
+                                                rate_buffer=0)
         self.deque = deque()
 
-    @property
-    def wait_until(self):
-        return self.last_time + self.update_delta
-
     def __len__(self):
         return len(self.deque)
 
@@ -62,10 +64,10 @@ class RateLimiterBucket(object):
     __nonzero__ = __bool__  # py2
 
     def __lt__(self, other):
-        # used to sort buckets by readiness
+        # used to sort RateLimiterBuckets by readiness
         if isinstance(other, RateLimiterBucket):
-            return self.wait_until < other.wait_until
-        return self.wait_until < other
+            return self.running_time < other.running_time
+        return self.running_time < other
 
 
 class BucketizedUpdateSkippingLimiter(object):
@@ -84,15 +86,15 @@ class BucketizedUpdateSkippingLimiter(object):
 
     The iterator increments stats as follows:
 
-      * The `deferrals` stat is incremented for each update that is
-        rate-limited. Note that a individual update is rate-limited at most
-        once.
-      * The `skips` stat is incremented for each rate-limited update that is
-        not eventually yielded. This includes updates that are evicted from the
-        deferral queue and all updates that remain in the deferral queue when
-        ``drain_until`` time is reached and the iterator terminates.
-      * The `drains` stat is incremented for each rate-limited update that is
-        eventually yielded.
+    * The `deferrals` stat is incremented for each update that is
+      rate-limited. Note that a individual update is rate-limited at most
+      once.
+    * The `skips` stat is incremented for each rate-limited update that is
+      not eventually yielded. This includes updates that are evicted from the
+      deferral queue and all updates that remain in the deferral queue when
+      ``drain_until`` time is reached and the iterator terminates.
+    * The `drains` stat is incremented for each rate-limited update that is
+      eventually yielded.
 
     Consequently, when this iterator terminates, the sum of `skips` and
     `drains` is equal to the number of `deferrals`.
@@ -124,15 +126,11 @@ class BucketizedUpdateSkippingLimiter(object):
         self.stats = stats
         # if we want a smaller "blast radius" we could make this number bigger
         self.num_buckets = max(num_buckets, 1)
-        try:
-            self.bucket_update_delta = 1.0 / max_elements_per_group_per_second
-        except ZeroDivisionError:
-            self.bucket_update_delta = -1
         self.max_deferred_elements = max_deferred_elements
         self.deferred_buckets = deque()
         self.drain_until = drain_until
         self.salt = str(uuid.uuid4())
-        self.buckets = [RateLimiterBucket(self.bucket_update_delta)
+        self.buckets = [RateLimiterBucket(max_elements_per_group_per_second)
                         for _ in range(self.num_buckets)]
         self.buckets_ordered_by_readiness = None
 
@@ -151,9 +149,8 @@ class BucketizedUpdateSkippingLimiter(object):
         for update_ctx in self.iterator:
             bucket = self.buckets[self._bucket_key(update_ctx['update'])]
             now = self._get_time()
-            if now >= bucket.wait_until:
+            if bucket.is_allowed(now=now):
                 # no need to ratelimit, just return next update
-                bucket.last_time = now
                 return update_ctx
 
             self.stats.deferrals += 1
@@ -194,13 +191,12 @@ class BucketizedUpdateSkippingLimiter(object):
             bucket = self.buckets_ordered_by_readiness.get_nowait()
             if now < self.drain_until:
                 # wait for next element to be ready
-                time.sleep(max(0, bucket.wait_until - now))
+                bucket.wait(now=now)
                 # drain the most recently deferred element
                 item = bucket.deque.pop()
                 if bucket:
                     # bucket has more deferred elements, re-insert in queue in
                     # correct chronological position
-                    bucket.last_time = self._get_time()
                     self.buckets_ordered_by_readiness.put(bucket)
                 self.stats.drains += 1
                 self.logger.increment("drains")
@@ -223,12 +219,12 @@ class SweepStats(object):
     """
     Stats bucket for an update sweep
 
-    A measure of the rate at which updates are being rate-limited is:
+    A measure of the rate at which updates are being rate-limited is::
 
         deferrals / (deferrals + successes + failures - drains)
 
     A measure of the rate at which updates are not being sent during a sweep
-    is:
+    is::
 
         skips / (skips + successes + failures)
     """
@@ -385,6 +381,7 @@ class ObjectUpdater(Daemon):
                     pids.append(pid)
                 else:
                     signal.signal(signal.SIGTERM, signal.SIG_DFL)
+                    os.environ.pop('NOTIFY_SOCKET', None)
                     eventlet_monkey_patch()
                     self.stats.reset()
                     forkbegin = time.time()
@@ -739,15 +736,13 @@ class ObjectUpdater(Daemon):
             if not success:
                 self.logger.debug(
                     'Error code %(status)d is returned from remote '
-                    'server %(ip)s: %(port)s / %(device)s',
-                    {'status': resp.status, 'ip': node['replication_ip'],
-                     'port': node['replication_port'],
-                     'device': node['device']})
+                    'server %(node)s',
+                    {'status': resp.status,
+                     'node': node_to_string(node, replication=True)})
             return success, node['id'], redirect
         except Exception:
-            self.logger.exception(
-                'ERROR with remote server '
-                '%(replication_ip)s:%(replication_port)s/%(device)s', node)
+            self.logger.exception('ERROR with remote server %s',
+                                  node_to_string(node, replication=True))
         except Timeout as exc:
             action = 'connecting to'
             if not isinstance(exc, ConnectionTimeout):
@@ -756,9 +751,8 @@ class ObjectUpdater(Daemon):
                 status = 499
                 action = 'waiting on'
             self.logger.info(
-                'Timeout %(action)s remote server '
-                '%(replication_ip)s:%(replication_port)s/%(device)s: %(exc)s',
-                dict(node, exc=exc, action=action))
+                'Timeout %s remote server %s: %s',
+                action, node_to_string(node, replication=True), exc)
         finally:
             elapsed = time.time() - start
             self.logger.timing('updater.timing.status.%s' % status,
