@@ -13,8 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# See http://code.google.com/p/python-nose/issues/detail?id=373
-# The code below enables nosetests to work with i18n _() blocks
 from __future__ import print_function
 import sys
 from contextlib import contextmanager
@@ -22,20 +20,7 @@ from contextlib import contextmanager
 import os
 from six import reraise
 
-try:
-    from unittest.util import safe_repr
-except ImportError:
-    # Probably py26
-    _MAX_LENGTH = 80
-
-    def safe_repr(obj, short=False):
-        try:
-            result = repr(obj)
-        except Exception:
-            result = object.__repr__(obj)
-        if not short or len(result) < _MAX_LENGTH:
-            return result
-        return result[:_MAX_LENGTH] + ' [truncated]...'
+from unittest.util import safe_repr
 
 import warnings
 warnings.filterwarnings('ignore', module='cryptography|OpenSSL', message=(
@@ -51,11 +36,13 @@ warnings.filterwarnings('ignore', message=(
     'Therefore, support for it is deprecated in cryptography '
     'and will be removed in a future release.'))
 
-from eventlet.green import socket
+import unittest
 
-# make unittests pass on all locale
-import swift
-setattr(swift, 'gettext_', lambda x: x)
+if sys.version_info < (3, 2):
+    unittest.TestCase.assertRaisesRegex = unittest.TestCase.assertRaisesRegexp
+    unittest.TestCase.assertRegex = unittest.TestCase.assertRegexpMatches
+
+from eventlet.green import socket
 
 from swift.common.utils import readconf
 
@@ -127,3 +114,34 @@ def annotate_failure(msg):
             err_val = '%s Failed with %s' % (msg, err)
             err_typ = AssertionError
         reraise(err_typ, err_val, err_tb)
+
+
+class BaseTestCase(unittest.TestCase):
+    def _assertDictContainsSubset(self, subset, dictionary, msg=None):
+        """Checks whether dictionary is a superset of subset."""
+        # This is almost identical to the method in python3.4 version of
+        # unitest.case.TestCase.assertDictContainsSubset, reproduced here to
+        # avoid the deprecation warning in the original when using python3.
+        missing = []
+        mismatched = []
+        for key, value in subset.items():
+            if key not in dictionary:
+                missing.append(key)
+            elif value != dictionary[key]:
+                mismatched.append('%s, expected: %s, actual: %s' %
+                                  (safe_repr(key), safe_repr(value),
+                                   safe_repr(dictionary[key])))
+
+        if not (missing or mismatched):
+            return
+
+        standardMsg = ''
+        if missing:
+            standardMsg = 'Missing: %s' % ','.join(safe_repr(m) for m in
+                                                   missing)
+        if mismatched:
+            if standardMsg:
+                standardMsg += '; '
+            standardMsg += 'Mismatched values: %s' % ','.join(mismatched)
+
+        self.fail(self._formatMessage(msg, standardMsg))
