@@ -19,6 +19,8 @@ from contextlib import contextmanager
 import logging
 from unittest import SkipTest
 
+import os
+
 import test.functional as tf
 from test.functional.s3api.s3_test_client import (
     Connection, get_boto3_conn, tear_down_s3)
@@ -46,6 +48,8 @@ class S3ApiBase(unittest.TestCase):
             logging.getLogger('boto').setLevel(logging.DEBUG)
 
     def setUp(self):
+        if not tf.config.get('s3_access_key'):
+            raise SkipTest('no s3api user configured')
         if 's3api' not in tf.cluster_info:
             raise SkipTest('s3api middleware is not enabled')
         if tf.config.get('account'):
@@ -83,6 +87,8 @@ class S3ApiBase(unittest.TestCase):
 
 class S3ApiBaseBoto3(S3ApiBase):
     def setUp(self):
+        if not tf.config.get('s3_access_key'):
+            raise SkipTest('no s3api user configured')
         if 's3api' not in tf.cluster_info:
             raise SkipTest('s3api middleware is not enabled')
         try:
@@ -100,3 +106,29 @@ class S3ApiBaseBoto3(S3ApiBase):
 
     def tearDown(self):
         tear_down_s3(self.conn)
+
+
+def skip_boto2_sort_header_bug(m):
+    def wrapped(self, *args, **kwargs):
+        if os.environ.get('S3_USE_SIGV4') == "True":
+            # boto doesn't sort headers for v4 sigs properly; see
+            # https://github.com/boto/boto/pull/3032
+            # or https://github.com/boto/boto/pull/3176
+            # or https://github.com/boto/boto/pull/3751
+            # or https://github.com/boto/boto/pull/3824
+            self.skipTest('This stuff got the issue of boto<=2.x')
+        return m(self, *args, **kwargs)
+    return wrapped
+
+
+class SigV4Mixin(object):
+    @classmethod
+    def setUpClass(cls):
+        os.environ['S3_USE_SIGV4'] = "True"
+
+    @classmethod
+    def tearDownClass(cls):
+        del os.environ['S3_USE_SIGV4']
+
+    def setUp(self):
+        super(SigV4Mixin, self).setUp()
