@@ -16,12 +16,6 @@
 import base64
 import binascii
 import unittest
-import os
-import boto
-
-# For an issue with venv and distutils, disable pylint message here
-# pylint: disable-msg=E0611,F0401
-from distutils.version import StrictVersion
 
 import six
 from six.moves import urllib, zip, zip_longest
@@ -29,10 +23,12 @@ from six.moves import urllib, zip, zip_longest
 import test.functional as tf
 from swift.common.middleware.s3api.etree import fromstring, tostring, \
     Element, SubElement
-from swift.common.middleware.s3api.utils import MULTIUPLOAD_SUFFIX, mktime
+from swift.common.middleware.s3api.utils import MULTIUPLOAD_SUFFIX, mktime, \
+    S3Timestamp
 from swift.common.utils import md5
 
-from test.functional.s3api import S3ApiBase
+from test.functional.s3api import S3ApiBase, SigV4Mixin, \
+    skip_boto2_sort_header_bug
 from test.functional.s3api.s3_test_client import Connection
 from test.functional.s3api.utils import get_error_code, get_error_msg, \
     calculate_md5
@@ -51,7 +47,7 @@ class TestS3ApiMultiUpload(S3ApiBase):
     def setUp(self):
         super(TestS3ApiMultiUpload, self).setUp()
         if not tf.cluster_info['s3api'].get('allow_multipart_uploads', False):
-            raise tf.SkipTest('multipart upload is not enebled')
+            self.skipTest('multipart upload is not enebled')
 
         self.min_segment_size = int(tf.cluster_info['s3api'].get(
             'min_segment_size', 5242880))
@@ -115,7 +111,11 @@ class TestS3ApiMultiUpload(S3ApiBase):
         bucket = 'bucket'
         keys = [u'obj1\N{SNOWMAN}', u'obj2\N{SNOWMAN}', 'obj3']
         bad_content_md5 = base64.b64encode(b'a' * 16).strip().decode('ascii')
-        headers = [{'Content-Type': 'foo/bar', 'x-amz-meta-baz': 'quux'},
+        headers = [{'Content-Type': 'foo/bar', 'x-amz-meta-baz': 'quux',
+                    'Content-Encoding': 'gzip', 'Content-Language': 'en-US',
+                    'Expires': 'Thu, 01 Dec 1994 16:00:00 GMT',
+                    'Cache-Control': 'no-cache',
+                    'Content-Disposition': 'attachment'},
                    {'Content-MD5': bad_content_md5},
                    {'Etag': 'nonsense'}]
         uploads = []
@@ -209,7 +209,8 @@ class TestS3ApiMultiUpload(S3ApiBase):
         self.assertEqual(headers['content-type'], 'text/html; charset=UTF-8')
         self.assertTrue('content-length' in headers)
         self.assertEqual(headers['content-length'], '0')
-        expected_parts_list = [(headers['etag'], mktime(headers['date']))]
+        expected_parts_list = [(headers['etag'],
+                                mktime(headers['last-modified']))]
 
         # Upload Part Copy
         key, upload_id = uploads[1]
@@ -238,8 +239,8 @@ class TestS3ApiMultiUpload(S3ApiBase):
         self.assertTrue('etag' not in headers)
         elem = fromstring(body, 'CopyPartResult')
 
-        last_modified = elem.find('LastModified').text
-        self.assertTrue(last_modified is not None)
+        copy_resp_last_modified = elem.find('LastModified').text
+        self.assertIsNotNone(copy_resp_last_modified)
 
         self.assertEqual(resp_etag, etag)
 
@@ -252,15 +253,10 @@ class TestS3ApiMultiUpload(S3ApiBase):
         self.assertEqual(200, status)
         elem = fromstring(body, 'ListPartsResult')
 
-        # FIXME: COPY result drops milli/microseconds but GET doesn't
-        last_modified_gets = [p.find('LastModified').text
-                              for p in elem.iterfind('Part')]
-        self.assertEqual(
-            last_modified_gets[0].rsplit('.', 1)[0],
-            last_modified.rsplit('.', 1)[0],
-            '%r != %r' % (last_modified_gets[0], last_modified))
-        # There should be *exactly* two parts in the result
-        self.assertEqual(1, len(last_modified_gets))
+        listing_last_modified = [p.find('LastModified').text
+                                 for p in elem.iterfind('Part')]
+        # There should be *exactly* one parts in the result
+        self.assertEqual(listing_last_modified, [copy_resp_last_modified])
 
         # List Parts
         key, upload_id = uploads[0]
@@ -295,15 +291,10 @@ class TestS3ApiMultiUpload(S3ApiBase):
         for (expected_etag, expected_date), p in \
                 zip(expected_parts_list, elem.findall('Part')):
             last_modified = p.find('LastModified').text
-            self.assertTrue(last_modified is not None)
-            # TODO: sanity check
-            #       (kota_) How do we check the sanity?
-            #       the last-modified header drops milli-seconds info
-            #       by the constraint of the format.
-            #       For now, we can do either the format check or round check
-            # last_modified_from_xml = mktime(last_modified)
-            # self.assertEqual(expected_date,
-            #                   last_modified_from_xml)
+            self.assertIsNotNone(last_modified)
+            last_modified_from_xml = S3Timestamp.from_s3xmlformat(
+                last_modified)
+            self.assertEqual(expected_date, float(last_modified_from_xml))
             self.assertEqual(expected_etag, p.find('ETag').text)
             self.assertEqual(self.min_segment_size, int(p.find('Size').text))
             etags.append(p.find('ETag').text)
@@ -346,6 +337,11 @@ class TestS3ApiMultiUpload(S3ApiBase):
         self.assertEqual(status, 200)
         self.assertEqual(headers['content-length'], str(exp_size))
         self.assertEqual(headers['content-type'], 'foo/bar')
+        self.assertEqual(headers['content-encoding'], 'gzip')
+        self.assertEqual(headers['content-language'], 'en-US')
+        self.assertEqual(headers['content-disposition'], 'attachment')
+        self.assertEqual(headers['expires'], 'Thu, 01 Dec 1994 16:00:00 GMT')
+        self.assertEqual(headers['cache-control'], 'no-cache')
         self.assertEqual(headers['x-amz-meta-baz'], 'quux')
 
         swift_etag = '"%s"' % md5(
@@ -485,9 +481,9 @@ class TestS3ApiMultiUpload(S3ApiBase):
             expected_key = keys[0]
         self.assertEqual(o.find('Key').text, expected_key)
         self.assertIsNotNone(o.find('LastModified').text)
-        self.assertRegexpMatches(
+        self.assertRegex(
             o.find('LastModified').text,
-            r'^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{3}Z$')
+            r'^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}.000Z$')
         self.assertEqual(o.find('ETag').text, exp_etag)
         self.assertEqual(o.find('Size').text, str(exp_size))
         self.assertIsNotNone(o.find('StorageClass').text)
@@ -866,9 +862,8 @@ class TestS3ApiMultiUpload(S3ApiBase):
                                    query=query)
         self.assertEqual(status, 200)
 
-    def test_object_multi_upload_part_copy_range(self):
-        bucket = 'bucket'
-        keys = ['obj1']
+    def _initiate_mpu_upload(self, bucket, key):
+        keys = [key]
         uploads = []
 
         results_generator = self._initiate_multi_uploads_result_generator(
@@ -892,10 +887,13 @@ class TestS3ApiMultiUpload(S3ApiBase):
             self.assertTrue((key, upload_id) not in uploads)
             uploads.append((key, upload_id))
 
-        self.assertEqual(len(uploads), len(keys))  # sanity
+        # sanity, there's just one multi-part upload
+        self.assertEqual(1, len(uploads))
+        self.assertEqual(1, len(keys))
+        _, upload_id = uploads[0]
+        return upload_id
 
-        # Upload Part Copy Range
-        key, upload_id = uploads[0]
+    def _copy_part_from_new_src_range(self, bucket, key, upload_id):
         src_bucket = 'bucket2'
         src_obj = 'obj4'
         src_content = b'y' * (self.min_segment_size // 2) + b'z' * \
@@ -922,33 +920,153 @@ class TestS3ApiMultiUpload(S3ApiBase):
         self.assertEqual(headers['content-length'], str(len(body)))
         self.assertTrue('etag' not in headers)
         elem = fromstring(body, 'CopyPartResult')
+        etags = [elem.find('ETag').text]
 
-        last_modified = elem.find('LastModified').text
-        self.assertTrue(last_modified is not None)
+        copy_resp_last_modified = elem.find('LastModified').text
+        self.assertIsNotNone(copy_resp_last_modified)
 
         self.assertEqual(resp_etag, etag)
 
         # Check last-modified timestamp
-        key, upload_id = uploads[0]
         query = 'uploadId=%s' % upload_id
         status, headers, body = \
             self.conn.make_request('GET', bucket, key, query=query)
 
         elem = fromstring(body, 'ListPartsResult')
 
-        # FIXME: COPY result drops milli/microseconds but GET doesn't
-        last_modified_gets = [p.find('LastModified').text
-                              for p in elem.iterfind('Part')]
-        self.assertEqual(
-            last_modified_gets[0].rsplit('.', 1)[0],
-            last_modified.rsplit('.', 1)[0],
-            '%r != %r' % (last_modified_gets[0], last_modified))
-
+        listing_last_modified = [p.find('LastModified').text
+                                 for p in elem.iterfind('Part')]
         # There should be *exactly* one parts in the result
-        self.assertEqual(1, len(last_modified_gets))
+        self.assertEqual(listing_last_modified, [copy_resp_last_modified])
+
+        # sanity, there's just one etag
+        self.assertEqual(1, len(etags))
+        return etags[0]
+
+    def _complete_mpu_upload(self, bucket, key, upload_id, etags):
+        # Complete Multipart Upload
+        query = 'uploadId=%s' % upload_id
+        xml = self._gen_comp_xml(etags)
+        status, headers, body = \
+            self.conn.make_request('POST', bucket, key, body=xml,
+                                   query=query)
+        self.assertEqual(status, 200)
+        self.assertCommonResponseHeaders(headers)
+        self.assertTrue('content-type' in headers)
+        self.assertEqual(headers['content-type'], 'application/xml')
+        if 'content-length' in headers:
+            self.assertEqual(headers['content-length'], str(len(body)))
+        else:
+            self.assertIn('transfer-encoding', headers)
+            self.assertEqual(headers['transfer-encoding'], 'chunked')
+        lines = body.split(b'\n')
+        self.assertTrue(lines[0].startswith(b'<?xml'), body)
+        self.assertTrue(lines[0].endswith(b'?>'), body)
+        elem = fromstring(body, 'CompleteMultipartUploadResult')
+        self.assertEqual(
+            '%s/%s/%s' %
+            (tf.config['s3_storage_url'].rstrip('/'), bucket, key),
+            elem.find('Location').text)
+        self.assertEqual(elem.find('Bucket').text, bucket)
+        self.assertEqual(elem.find('Key').text, key)
+        concatted_etags = b''.join(
+            etag.strip('"').encode('ascii') for etag in etags)
+        exp_etag = '"%s-%s"' % (
+            md5(binascii.unhexlify(concatted_etags),
+                usedforsecurity=False).hexdigest(), len(etags))
+        etag = elem.find('ETag').text
+        self.assertEqual(etag, exp_etag)
+
+    @skip_boto2_sort_header_bug
+    def test_mpu_copy_part_from_range_then_complete(self):
+        bucket = 'mpu-copy-range'
+        key = 'obj-complete'
+        upload_id = self._initiate_mpu_upload(bucket, key)
+        etag = self._copy_part_from_new_src_range(bucket, key, upload_id)
+        self._complete_mpu_upload(bucket, key, upload_id, [etag])
+
+    @skip_boto2_sort_header_bug
+    def test_mpu_copy_part_from_range_then_abort(self):
+        bucket = 'mpu-copy-range'
+        key = 'obj-abort'
+        upload_id = self._initiate_mpu_upload(bucket, key)
+        self._copy_part_from_new_src_range(bucket, key, upload_id)
+
+        # Abort Multipart Upload
+        query = 'uploadId=%s' % upload_id
+        status, headers, body = \
+            self.conn.make_request('DELETE', bucket, key, query=query)
+
+        # sanity checks
+        self.assertEqual(status, 204)
+        self.assertCommonResponseHeaders(headers)
+        self.assertTrue('content-type' in headers)
+        self.assertEqual(headers['content-type'], 'text/html; charset=UTF-8')
+        self.assertTrue('content-length' in headers)
+        self.assertEqual(headers['content-length'], '0')
+
+    def _copy_part_from_new_mpu_range(self, bucket, key, upload_id):
+        src_bucket = 'bucket2'
+        src_obj = 'mpu2'
+        src_upload_id = self._initiate_mpu_upload(src_bucket, src_obj)
+        # upload parts
+        etags = []
+        for part_num in range(2):
+            # Upload Part
+            content = (chr(97 + part_num) * self.min_segment_size).encode()
+            etag = md5(content, usedforsecurity=False).hexdigest()
+            status, headers, body = \
+                self._upload_part(src_bucket, src_obj, src_upload_id,
+                                  content, part_num=part_num + 1)
+            self.assertEqual(status, 200)
+            self.assertCommonResponseHeaders(headers, etag)
+            self.assertTrue('content-type' in headers)
+            self.assertEqual(headers['content-type'],
+                             'text/html; charset=UTF-8')
+            self.assertTrue('content-length' in headers)
+            self.assertEqual(headers['content-length'], '0')
+            self.assertEqual(headers['etag'], '"%s"' % etag)
+            etags.append(etag)
+        self._complete_mpu_upload(src_bucket, src_obj, src_upload_id, etags)
+
+        # Upload Part Copy -- MPU as source
+        src_range = 'bytes=0-%d' % (self.min_segment_size - 1)
+        status, headers, body, resp_etag = \
+            self._upload_part_copy(src_bucket, src_obj, bucket,
+                                   key, upload_id, part_num=1,
+                                   src_range=src_range)
+        self.assertEqual(status, 200)
+        self.assertCommonResponseHeaders(headers)
+        self.assertIn('content-type', headers)
+        self.assertEqual(headers['content-type'], 'application/xml')
+        self.assertIn('content-length', headers)
+        self.assertEqual(headers['content-length'], str(len(body)))
+        self.assertNotIn('etag', headers)
+        elem = fromstring(body, 'CopyPartResult')
+
+        last_modified = elem.find('LastModified').text
+        self.assertIsNotNone(last_modified)
+        # use copied with src_range from src_obj?part-number=1
+        self.assertEqual(resp_etag, etags[0])
+
+        return resp_etag
+
+    @skip_boto2_sort_header_bug
+    def test_mpu_copy_part_from_mpu_part_number_then_complete(self):
+        bucket = 'mpu-copy-range'
+        key = 'obj-complete'
+        upload_id = self._initiate_mpu_upload(bucket, key)
+        etag = self._copy_part_from_new_mpu_range(bucket, key, upload_id)
+        self._complete_mpu_upload(bucket, key, upload_id, [etag])
+
+    @skip_boto2_sort_header_bug
+    def test_mpu_copy_part_from_mpu_part_number_then_abort(self):
+        bucket = 'mpu-copy-range'
+        key = 'obj-abort'
+        upload_id = self._initiate_mpu_upload(bucket, key)
+        self._copy_part_from_new_mpu_range(bucket, key, upload_id)
 
         # Abort Multipart Upload
-        key, upload_id = uploads[0]
         query = 'uploadId=%s' % upload_id
         status, headers, body = \
             self.conn.make_request('DELETE', bucket, key, query=query)
@@ -1035,8 +1153,8 @@ class TestS3ApiMultiUpload(S3ApiBase):
         self.assertTrue('etag' not in headers)
         elem = fromstring(body, 'CopyPartResult')
 
-        last_modifieds = [elem.find('LastModified').text]
-        self.assertTrue(last_modifieds[0] is not None)
+        copy_resp_last_modifieds = [elem.find('LastModified').text]
+        self.assertTrue(copy_resp_last_modifieds[0] is not None)
 
         self.assertEqual(resp_etag, etags[0])
 
@@ -1053,8 +1171,8 @@ class TestS3ApiMultiUpload(S3ApiBase):
         self.assertTrue('etag' not in headers)
         elem = fromstring(body, 'CopyPartResult')
 
-        last_modifieds.append(elem.find('LastModified').text)
-        self.assertTrue(last_modifieds[1] is not None)
+        copy_resp_last_modifieds.append(elem.find('LastModified').text)
+        self.assertTrue(copy_resp_last_modifieds[1] is not None)
 
         self.assertEqual(resp_etag, etags[1])
 
@@ -1066,15 +1184,9 @@ class TestS3ApiMultiUpload(S3ApiBase):
 
         elem = fromstring(body, 'ListPartsResult')
 
-        # FIXME: COPY result drops milli/microseconds but GET doesn't
-        last_modified_gets = [p.find('LastModified').text
-                              for p in elem.iterfind('Part')]
-        self.assertEqual(
-            [lm.rsplit('.', 1)[0] for lm in last_modified_gets],
-            [lm.rsplit('.', 1)[0] for lm in last_modifieds])
-
-        # There should be *exactly* two parts in the result
-        self.assertEqual(2, len(last_modified_gets))
+        listing_last_modified = [p.find('LastModified').text
+                                 for p in elem.iterfind('Part')]
+        self.assertEqual(listing_last_modified, copy_resp_last_modifieds)
 
         # Abort Multipart Upload
         key, upload_id = uploads[0]
@@ -1090,28 +1202,6 @@ class TestS3ApiMultiUpload(S3ApiBase):
         self.assertTrue('content-length' in headers)
         self.assertEqual(headers['content-length'], '0')
 
-
-class TestS3ApiMultiUploadSigV4(TestS3ApiMultiUpload):
-    @classmethod
-    def setUpClass(cls):
-        os.environ['S3_USE_SIGV4'] = "True"
-
-    @classmethod
-    def tearDownClass(cls):
-        del os.environ['S3_USE_SIGV4']
-
-    def setUp(self):
-        super(TestS3ApiMultiUploadSigV4, self).setUp()
-
-    def test_object_multi_upload_part_copy_range(self):
-        if StrictVersion(boto.__version__) < StrictVersion('3.0'):
-            # boto 2 doesn't sort headers properly; see
-            # https://github.com/boto/boto/pull/3032
-            # or https://github.com/boto/boto/pull/3176
-            # or https://github.com/boto/boto/pull/3751
-            # or https://github.com/boto/boto/pull/3824
-            self.skipTest('This stuff got the issue of boto<=2.x')
-
     def test_delete_bucket_multi_upload_object_exisiting(self):
         bucket = 'bucket'
         keys = ['obj1']
@@ -1189,5 +1279,9 @@ class TestS3ApiMultiUploadSigV4(TestS3ApiMultiUpload):
         self.assertEqual(status, 204)  # sanity
 
 
+class TestS3ApiMultiUploadSigV4(TestS3ApiMultiUpload, SigV4Mixin):
+    pass
+
+
 if __name__ == '__main__':
     unittest.main()
