@@ -15,26 +15,25 @@
 # limitations under the License.
 
 import unittest
-import os
-import boto
-
-# For an issue with venv and distutils, disable pylint message here
-# pylint: disable-msg=E0611,F0401
-from distutils.version import StrictVersion
 
+import calendar
 import email.parser
 from email.utils import formatdate, parsedate
 from time import mktime
 import six
 
 import test.functional as tf
+from swift.common import utils
 
 from swift.common.middleware.s3api.etree import fromstring
+from swift.common.middleware.s3api.utils import S3Timestamp
 from swift.common.utils import md5, quote
 
-from test.functional.s3api import S3ApiBase
+from test.functional.s3api import S3ApiBase, SigV4Mixin, \
+    skip_boto2_sort_header_bug
 from test.functional.s3api.s3_test_client import Connection
-from test.functional.s3api.utils import get_error_code, calculate_md5
+from test.functional.s3api.utils import get_error_code, calculate_md5, \
+    get_error_msg
 
 DAY = 86400.0  # 60 * 60 * 24 (sec)
 
@@ -98,21 +97,32 @@ class TestS3ApiObject(S3ApiBase):
 
         elem = fromstring(body, 'CopyObjectResult')
         self.assertTrue(elem.find('LastModified').text is not None)
-        last_modified_xml = elem.find('LastModified').text
+        copy_resp_last_modified_xml = elem.find('LastModified').text
         self.assertTrue(elem.find('ETag').text is not None)
         self.assertEqual(etag, elem.find('ETag').text.strip('"'))
         self._assertObjectEtag(dst_bucket, dst_obj, etag)
 
-        # Check timestamp on Copy:
+        # Check timestamp on Copy in listing:
         status, headers, body = \
             self.conn.make_request('GET', dst_bucket)
         self.assertEqual(status, 200)
         elem = fromstring(body, 'ListBucketResult')
-
-        # FIXME: COPY result drops milli/microseconds but GET doesn't
         self.assertEqual(
-            elem.find('Contents').find("LastModified").text.rsplit('.', 1)[0],
-            last_modified_xml.rsplit('.', 1)[0])
+            elem.find('Contents').find("LastModified").text,
+            copy_resp_last_modified_xml)
+
+        # GET Object copy
+        status, headers, body = \
+            self.conn.make_request('GET', dst_bucket, dst_obj)
+        self.assertEqual(status, 200)
+
+        self.assertCommonResponseHeaders(headers, etag)
+        self.assertTrue(headers['last-modified'] is not None)
+        self.assertEqual(
+            float(S3Timestamp.from_s3xmlformat(copy_resp_last_modified_xml)),
+            calendar.timegm(parsedate(headers['last-modified'])))
+        self.assertTrue(headers['content-type'] is not None)
+        self.assertEqual(headers['content-length'], str(len(content)))
 
         # GET Object
         status, headers, body = \
@@ -123,6 +133,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertTrue(headers['last-modified'] is not None)
         self.assertTrue(headers['content-type'] is not None)
         self.assertEqual(headers['content-length'], str(len(content)))
+        self.assertEqual(headers['accept-ranges'], 'bytes')
 
         # HEAD Object
         status, headers, body = \
@@ -133,6 +144,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertTrue(headers['last-modified'] is not None)
         self.assertTrue('content-type' in headers)
         self.assertEqual(headers['content-length'], str(len(content)))
+        self.assertEqual(headers['accept-ranges'], 'bytes')
 
         # DELETE Object
         status, headers, body = \
@@ -353,6 +365,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertCommonResponseHeaders(headers)
         self._assertObjectEtag(self.bucket, obj, etag)
 
+    @skip_boto2_sort_header_bug
     def test_put_object_metadata(self):
         self._test_put_object_headers({
             'X-Amz-Meta-Bar': 'foo',
@@ -404,6 +417,53 @@ class TestS3ApiObject(S3ApiBase):
         self.assertCommonResponseHeaders(headers)
         self._assertObjectEtag(self.bucket, obj, etag)
 
+    def test_put_object_valid_delete_headers(self):
+        obj = 'object'
+        content = b'abcdefghij'
+        ts = utils.Timestamp.now()
+        delete_at = {'X-Delete-At': str(int(ts) + 70)}
+        delete_after = {'X-Delete-After': str(int(ts) + 130)}
+        status, delete_at, body = \
+            self.conn.make_request('PUT', self.bucket, obj, delete_at, content)
+        self.assertEqual(status, 200)
+        status, delete_after, body = \
+            self.conn.make_request('PUT', self.bucket, obj, delete_after,
+                                   content)
+        self.assertEqual(status, 200)
+
+    def test_put_object_invalid_x_delete_at(self):
+        obj = 'object'
+        content = b'abcdefghij'
+        ts = utils.Timestamp.now()
+        headers = {'X-Delete-At': str(int(ts) - 140)}
+        status, headers, body = \
+            self.conn.make_request('PUT', self.bucket, obj, headers, content)
+        self.assertEqual(status, 400)
+        self.assertEqual(get_error_code(body), 'InvalidArgument')
+        self.assertEqual(get_error_msg(body), 'X-Delete-At in past')
+        headers = {'X-Delete-At': 'test'}
+        status, headers, body = \
+            self.conn.make_request('PUT', self.bucket, obj, headers, content)
+        self.assertEqual(status, 400)
+        self.assertEqual(get_error_code(body), 'InvalidArgument')
+        self.assertEqual(get_error_msg(body), 'Non-integer X-Delete-At')
+
+    def test_put_object_invalid_x_delete_after(self):
+        obj = 'object'
+        content = b'abcdefghij'
+        headers = {'X-Delete-After': 'test'}
+        status, headers, body = \
+            self.conn.make_request('PUT', self.bucket, obj, headers, content)
+        self.assertEqual(status, 400)
+        self.assertEqual(get_error_code(body), 'InvalidArgument')
+        self.assertEqual(get_error_msg(body), 'Non-integer X-Delete-After')
+        headers = {'X-Delete-After': '-140'}
+        status, headers, body = \
+            self.conn.make_request('PUT', self.bucket, obj, headers, content)
+        self.assertEqual(status, 400)
+        self.assertEqual(get_error_code(body), 'InvalidArgument')
+        self.assertEqual(get_error_msg(body), 'X-Delete-After in past')
+
     def test_put_object_copy_source_params(self):
         obj = 'object'
         src_headers = {'X-Amz-Meta-Test': 'src'}
@@ -524,6 +584,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('PUT', dst_bucket, dst_obj, headers)
         self.assertEqual(status, 400)
 
+    @skip_boto2_sort_header_bug
     def test_put_object_copy_source_if_modified_since(self):
         obj = 'object'
         dst_bucket = 'dst-bucket'
@@ -544,6 +605,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertCommonResponseHeaders(headers)
         self._assertObjectEtag(self.bucket, obj, etag)
 
+    @skip_boto2_sort_header_bug
     def test_put_object_copy_source_if_unmodified_since(self):
         obj = 'object'
         dst_bucket = 'dst-bucket'
@@ -564,6 +626,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertCommonResponseHeaders(headers)
         self._assertObjectEtag(self.bucket, obj, etag)
 
+    @skip_boto2_sort_header_bug
     def test_put_object_copy_source_if_match(self):
         obj = 'object'
         dst_bucket = 'dst-bucket'
@@ -583,6 +646,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertCommonResponseHeaders(headers)
         self._assertObjectEtag(self.bucket, obj, etag)
 
+    @skip_boto2_sort_header_bug
     def test_put_object_copy_source_if_none_match(self):
         obj = 'object'
         dst_bucket = 'dst-bucket'
@@ -609,6 +673,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
         self.assertEqual(headers['content-type'], 'text/plain')
+        self.assertTrue('accept-ranges' in headers)
 
     def test_get_object_response_content_language(self):
         obj = 'object'
@@ -620,6 +685,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
         self.assertEqual(headers['content-language'], 'en')
+        self.assertTrue('accept-ranges' in headers)
 
     def test_get_object_response_cache_control(self):
         obj = 'object'
@@ -630,6 +696,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('GET', self.bucket, obj, query=query)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
         self.assertEqual(headers['cache-control'], 'private')
 
     def test_get_object_response_content_disposition(self):
@@ -642,6 +709,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
         self.assertEqual(headers['content-disposition'], 'inline')
+        self.assertTrue('accept-ranges' in headers)
 
     def test_get_object_response_content_encoding(self):
         obj = 'object'
@@ -653,6 +721,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
         self.assertEqual(headers['content-encoding'], 'gzip')
+        self.assertTrue('accept-ranges' in headers)
 
     def test_get_object_range(self):
         obj = 'object'
@@ -668,6 +737,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertEqual(status, 206)
         self.assertCommonResponseHeaders(headers)
         self.assertTrue('content-length' in headers)
+        self.assertTrue('accept-ranges' in headers)
         self.assertEqual(headers['content-length'], '5')
         self.assertTrue('x-amz-meta-test' in headers)
         self.assertEqual('swift', headers['x-amz-meta-test'])
@@ -680,6 +750,7 @@ class TestS3ApiObject(S3ApiBase):
         self.assertEqual(status, 206)
         self.assertCommonResponseHeaders(headers)
         self.assertTrue('content-length' in headers)
+        self.assertTrue('accept-ranges' in headers)
         self.assertEqual(headers['content-length'], '5')
         self.assertTrue('x-amz-meta-test' in headers)
         self.assertEqual('swift', headers['x-amz-meta-test'])
@@ -690,6 +761,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('GET', self.bucket, obj, headers=headers)
         self.assertEqual(status, 206)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
         self.assertTrue('content-length' in headers)
         self.assertEqual(headers['content-length'], '5')
         self.assertTrue('x-amz-meta-test' in headers)
@@ -703,6 +775,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('GET', self.bucket, obj, headers=headers)
         self.assertEqual(status, 206)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
         self.assertIn('content-length', headers)
 
         self.assertIn('content-type', headers)  # sanity
@@ -754,6 +827,7 @@ class TestS3ApiObject(S3ApiBase):
         status, headers, body = \
             self.conn.make_request('GET', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
+        self.assertTrue('accept-ranges' in headers)
         self.assertCommonResponseHeaders(headers)
 
     def test_get_object_if_unmodified_since(self):
@@ -769,6 +843,30 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('GET', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
+        # check we can use the last modified time from the listing...
+        status, headers, body = \
+            self.conn.make_request('GET', self.bucket)
+        elem = fromstring(body, 'ListBucketResult')
+        last_modified = elem.find('./Contents/LastModified').text
+        listing_datetime = S3Timestamp.from_s3xmlformat(last_modified)
+        # Make sure there's no fractions of a second
+        self.assertEqual(int(listing_datetime), float(listing_datetime))
+        header_datetime = formatdate(int(listing_datetime))
+
+        headers = {'If-Unmodified-Since': header_datetime}
+        status, headers, body = \
+            self.conn.make_request('GET', self.bucket, obj, headers=headers)
+        self.assertEqual(status, 200)
+        self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
+
+        headers = {'If-Modified-Since': header_datetime}
+        status, headers, body = \
+            self.conn.make_request('GET', self.bucket, obj, headers=headers)
+        self.assertEqual(status, 304)
+        self.assertTrue('accept-ranges' in headers)
+        self.assertCommonResponseHeaders(headers)
 
     def test_get_object_if_match(self):
         obj = 'object'
@@ -783,6 +881,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('GET', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
     def test_get_object_if_none_match(self):
         obj = 'object'
@@ -792,6 +891,7 @@ class TestS3ApiObject(S3ApiBase):
         status, headers, body = \
             self.conn.make_request('GET', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
+        self.assertTrue('accept-ranges' in headers)
         self.assertCommonResponseHeaders(headers)
 
     def test_head_object_range(self):
@@ -804,18 +904,21 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('HEAD', self.bucket, obj, headers=headers)
         self.assertEqual(headers['content-length'], '5')
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
         headers = {'Range': 'bytes=5-'}
         status, headers, body = \
             self.conn.make_request('HEAD', self.bucket, obj, headers=headers)
         self.assertEqual(headers['content-length'], '5')
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
         headers = {'Range': 'bytes=-5'}
         status, headers, body = \
             self.conn.make_request('HEAD', self.bucket, obj, headers=headers)
         self.assertEqual(headers['content-length'], '5')
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
     def test_head_object_if_modified_since(self):
         obj = 'object'
@@ -830,6 +933,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('HEAD', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
     def test_head_object_if_unmodified_since(self):
         obj = 'object'
@@ -844,6 +948,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('HEAD', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
     def test_head_object_if_match(self):
         obj = 'object'
@@ -858,6 +963,7 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('HEAD', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
     def test_head_object_if_none_match(self):
         obj = 'object'
@@ -868,48 +974,11 @@ class TestS3ApiObject(S3ApiBase):
             self.conn.make_request('HEAD', self.bucket, obj, headers=headers)
         self.assertEqual(status, 200)
         self.assertCommonResponseHeaders(headers)
+        self.assertTrue('accept-ranges' in headers)
 
 
-class TestS3ApiObjectSigV4(TestS3ApiObject):
-    @classmethod
-    def setUpClass(cls):
-        os.environ['S3_USE_SIGV4'] = "True"
-
-    @classmethod
-    def tearDownClass(cls):
-        del os.environ['S3_USE_SIGV4']
-
-    def setUp(self):
-        super(TestS3ApiObjectSigV4, self).setUp()
-
-    @unittest.skipIf(StrictVersion(boto.__version__) < StrictVersion('3.0'),
-                     'This stuff got the signing issue of boto<=2.x')
-    def test_put_object_metadata(self):
-        super(TestS3ApiObjectSigV4, self).test_put_object_metadata()
-
-    @unittest.skipIf(StrictVersion(boto.__version__) < StrictVersion('3.0'),
-                     'This stuff got the signing issue of boto<=2.x')
-    def test_put_object_copy_source_if_modified_since(self):
-        super(TestS3ApiObjectSigV4, self).\
-            test_put_object_copy_source_if_modified_since()
-
-    @unittest.skipIf(StrictVersion(boto.__version__) < StrictVersion('3.0'),
-                     'This stuff got the signing issue of boto<=2.x')
-    def test_put_object_copy_source_if_unmodified_since(self):
-        super(TestS3ApiObjectSigV4, self).\
-            test_put_object_copy_source_if_unmodified_since()
-
-    @unittest.skipIf(StrictVersion(boto.__version__) < StrictVersion('3.0'),
-                     'This stuff got the signing issue of boto<=2.x')
-    def test_put_object_copy_source_if_match(self):
-        super(TestS3ApiObjectSigV4,
-              self).test_put_object_copy_source_if_match()
-
-    @unittest.skipIf(StrictVersion(boto.__version__) < StrictVersion('3.0'),
-                     'This stuff got the signing issue of boto<=2.x')
-    def test_put_object_copy_source_if_none_match(self):
-        super(TestS3ApiObjectSigV4,
-              self).test_put_object_copy_source_if_none_match()
+class TestS3ApiObjectSigV4(TestS3ApiObject, SigV4Mixin):
+    pass
 
 
 if __name__ == '__main__':
