@@ -407,6 +407,13 @@ class TestContainer(unittest.TestCase):
             {'X-Container-Meta-' + ('k' * self.max_meta_name_length): 'v'})
         resp.read()
         self.assertEqual(resp.status, 204)
+        # Clear it, so the value-length checking doesn't accidentally trip
+        # the overall max
+        resp = retry(
+            post,
+            {'X-Container-Meta-' + ('k' * self.max_meta_name_length): ''})
+        resp.read()
+        self.assertEqual(resp.status, 204)
         resp = retry(
             post,
             {'X-Container-Meta-' + (
