@@ -23,16 +23,30 @@ from copy import deepcopy
 import six
 from six.moves import urllib
 from time import time, strftime, gmtime
+from unittest import SkipTest
 
 import test.functional as tf
 from swift.common.middleware import tempurl
 from test.functional import cluster_info
 from test.functional.tests import Utils, Base, Base2, BaseEnv
-from test.functional import requires_acls, SkipTest
+from test.functional import requires_acls
 from test.functional.swift_test_client import Account, Connection, \
     ResponseError
 
 
+def tempurl_parms(method, expires, path, key, digest=None):
+    path = urllib.parse.unquote(path)
+    if not six.PY2:
+        method = method.encode('utf8')
+        path = path.encode('utf8')
+        key = key.encode('utf8')
+    sig = hmac.new(
+        key,
+        b'%s\n%d\n%s' % (method, expires, path),
+        digest or hashlib.sha256).hexdigest()
+    return {'temp_url_sig': sig, 'temp_url_expires': str(expires)}
+
+
 def setUpModule():
     tf.setup_package()
 
@@ -90,7 +104,7 @@ class TestTempurlEnv(TestTempurlBaseEnv):
 
 class TestTempurl(Base):
     env = TestTempurlEnv
-    digest_name = 'sha1'
+    digest_name = 'sha256'
 
     def setUp(self):
         super(TestTempurl, self).setUp()
@@ -102,6 +116,8 @@ class TestTempurl(Base):
                 "Expected tempurl_enabled to be True/False, got %r" %
                 (self.env.tempurl_enabled,))
 
+        # N.B. The default to 'sha1' in case the info has nothing is for
+        # extremely old clusters, which presumably use SHA1.
         if self.digest_name not in cluster_info['tempurl'].get(
                 'allowed_digests', ['sha1']):
             raise SkipTest("tempurl does not support %s signatures" %
@@ -116,16 +132,7 @@ class TestTempurl(Base):
             self.env.tempurl_key)
 
     def tempurl_parms(self, method, expires, path, key):
-        path = urllib.parse.unquote(path)
-        if not six.PY2:
-            method = method.encode('utf8')
-            path = path.encode('utf8')
-            key = key.encode('utf8')
-        sig = hmac.new(
-            key,
-            b'%s\n%d\n%s' % (method, expires, path),
-            self.digest).hexdigest()
-        return {'temp_url_sig': sig, 'temp_url_expires': str(expires)}
+        return tempurl_parms(method, expires, path, key, self.digest)
 
     def test_GET(self):
         for e in (str(self.expires), self.expires_8601):
@@ -347,17 +354,12 @@ class TestTempurlPrefix(TestTempurl):
             else:
                 prefix = path_parts[4][:4]
         prefix_to_hash = '/'.join(path_parts[0:4]) + '/' + prefix
-        if not six.PY2:
-            method = method.encode('utf8')
-            prefix_to_hash = prefix_to_hash.encode('utf8')
-            key = key.encode('utf8')
-        sig = hmac.new(
-            key,
-            b'%s\n%d\nprefix:%s' % (method, expires, prefix_to_hash),
-            self.digest).hexdigest()
-        return {
-            'temp_url_sig': sig, 'temp_url_expires': str(expires),
-            'temp_url_prefix': prefix}
+        parms = tempurl_parms(
+            method, expires,
+            'prefix:' + prefix_to_hash,
+            key, self.digest)
+        parms['temp_url_prefix'] = prefix
+        return parms
 
     def test_empty_prefix(self):
         parms = self.tempurl_parms(
@@ -461,7 +463,7 @@ class TestContainerTempurlEnv(BaseEnv):
 
 class TestContainerTempurl(Base):
     env = TestContainerTempurlEnv
-    digest_name = 'sha1'
+    digest_name = 'sha256'
 
     def setUp(self):
         super(TestContainerTempurl, self).setUp()
@@ -736,7 +738,7 @@ class TestSloTempurlEnv(TestTempurlBaseEnv):
 
 class TestSloTempurl(Base):
     env = TestSloTempurlEnv
-    digest_name = 'sha1'
+    digest_name = 'sha256'
 
     def setUp(self):
         super(TestSloTempurl, self).setUp()
@@ -838,7 +840,7 @@ class TestTempurlAlgorithms(Base):
         else:
             raise ValueError('Unrecognized encoding: %r' % encoding)
 
-    def _do_test(self, digest, encoding, expect_failure=False):
+    def _do_test(self, digest, encoding):
         expires = int(time()) + 86400
         sig = self.get_sig(expires, digest, encoding)
 
@@ -850,24 +852,14 @@ class TestTempurlAlgorithms(Base):
 
         parms = {'temp_url_sig': sig, 'temp_url_expires': str(expires)}
 
-        if expect_failure:
-            with self.assertRaises(ResponseError):
-                self.env.obj.read(parms=parms, cfg={'no_auth_token': True})
-            self.assert_status([401])
+        contents = self.env.obj.read(
+            parms=parms,
+            cfg={'no_auth_token': True})
+        self.assertEqual(contents, b"obj contents")
 
-            # ditto for HEADs
-            with self.assertRaises(ResponseError):
-                self.env.obj.info(parms=parms, cfg={'no_auth_token': True})
-            self.assert_status([401])
-        else:
-            contents = self.env.obj.read(
-                parms=parms,
-                cfg={'no_auth_token': True})
-            self.assertEqual(contents, b"obj contents")
-
-            # GET tempurls also allow HEAD requests
-            self.assertTrue(self.env.obj.info(
-                parms=parms, cfg={'no_auth_token': True}))
+        # GET tempurls also allow HEAD requests
+        self.assertTrue(self.env.obj.info(
+            parms=parms, cfg={'no_auth_token': True}))
 
     @requires_digest('sha1')
     def test_sha1(self):
@@ -887,8 +879,7 @@ class TestTempurlAlgorithms(Base):
 
     @requires_digest('sha512')
     def test_sha512(self):
-        # 128 chars seems awfully long for a signature -- let's require base64
-        self._do_test('sha512', 'hex', expect_failure=True)
+        self._do_test('sha512', 'hex')
         self._do_test('sha512', 'base64')
         self._do_test('sha512', 'base64-no-padding')
         self._do_test('sha512', 'url-safe-base64')
