@@ -14,6 +14,9 @@
 # limitations under the License.
 
 
+import eventlet
+eventlet.monkey_patch()
+
 from test import get_config
 from swift.common.utils import config_true_value
 
@@ -21,3 +24,8 @@ from swift.common.utils import config_true_value
 config = get_config('probe_test')
 CHECK_SERVER_TIMEOUT = int(config.get('check_server_timeout', 30))
 VALIDATE_RSYNC = config_true_value(config.get('validate_rsync', False))
+PROXY_BASE_URL = config.get('proxy_base_url')
+if PROXY_BASE_URL is None:
+    # TODO: find and load an "appropriate" proxy-server.conf(.d), piece
+    # something together from bind_ip, bind_port, and cert_file
+    PROXY_BASE_URL = 'http://127.0.0.1:8080'
