@@ -31,6 +31,7 @@ from swift.common.http import HTTP_NOT_FOUND
 
 from swiftclient import client, get_auth, ClientException
 
+from test.probe import PROXY_BASE_URL
 from test.probe.common import ENABLED_POLICIES
 
 TIMEOUT = 60
@@ -50,7 +51,7 @@ def meta_command(name, bases, attrs):
     for attr, value in attrs.items():
         if getattr(value, '__command__', False):
             commands[attr] = value
-            # methods have always have a __doc__ attribute, sometimes empty
+            # methods always have a __doc__ attribute, sometimes empty
             docs[attr] = (getattr(value, '__doc__', None) or
                           'perform the %s command' % attr).strip()
     attrs['__commands__'] = commands
@@ -67,6 +68,7 @@ def command(f):
     return f
 
 
+@six.add_metaclass(meta_command)
 class BaseBrain(object):
     def _setup(self, account, container_name, object_name,
                server_type, policy):
@@ -297,7 +299,6 @@ class InternalBrainClient(object):
         return headers, b''.join(resp_iter)
 
 
-@six.add_metaclass(meta_command)
 class BrainSplitter(BaseBrain):
     def __init__(self, url, token, container_name='test', object_name='test',
                  server_type='container', policy=None):
@@ -306,7 +307,6 @@ class BrainSplitter(BaseBrain):
                     server_type, policy)
 
 
-@six.add_metaclass(meta_command)
 class InternalBrainSplitter(BaseBrain):
     def __init__(self, conf, container_name='test', object_name='test',
                  server_type='container', policy=None):
@@ -340,7 +340,7 @@ def main():
         if cmd not in BrainSplitter.__commands__:
             parser.print_help()
             return 'ERROR: unknown command %s' % cmd
-    url, token = get_auth('http://127.0.0.1:8080/auth/v1.0',
+    url, token = get_auth(PROXY_BASE_URL + '/auth/v1.0',
                           'test:tester', 'testing')
     if options.server_type == 'object' and not options.policy_name:
         options.policy_name = POLICIES.default.name
