@@ -42,17 +42,18 @@ import random
 import errno
 import xattr
 from io import BytesIO
+from uuid import uuid4
 
 import six
 import six.moves.cPickle as pickle
 from six.moves import range
 from six.moves.http_client import HTTPException
 
-from swift.common import storage_policy, swob, utils
+from swift.common import storage_policy, swob, utils, exceptions
 from swift.common.memcached import MemcacheConnectionError
 from swift.common.storage_policy import (StoragePolicy, ECStoragePolicy,
                                          VALID_EC_TYPES)
-from swift.common.utils import Timestamp, md5
+from swift.common.utils import Timestamp, md5, close_if_possible
 from test import get_config
 from test.debug_logger import FakeLogger
 from swift.common.header_key_dict import HeaderKeyDict
@@ -63,11 +64,7 @@ import functools
 from gzip import GzipFile
 import mock as mocklib
 import inspect
-import unittest
-
-
-class SkipTest(unittest.SkipTest):
-    pass
+from unittest import SkipTest
 
 
 EMPTY_ETAG = md5(usedforsecurity=False).hexdigest()
@@ -413,6 +410,7 @@ class FakeMemcache(object):
 
     def __init__(self, error_on_set=None, error_on_get=None):
         self.store = {}
+        self.times = {}
         self.calls = []
         self.error_on_incr = False
         self.error_on_get = error_on_get or []
@@ -443,6 +441,7 @@ class FakeMemcache(object):
         else:
             assert isinstance(value, (str, bytes))
         self.store[key] = value
+        self.times[key] = time
         return True
 
     @track
@@ -466,12 +465,14 @@ class FakeMemcache(object):
     def delete(self, key):
         try:
             del self.store[key]
+            del self.times[key]
         except Exception:
             pass
         return True
 
     def delete_all(self):
         self.store.clear()
+        self.times.clear()
 
 
 # This decorator only makes sense in the context of FakeMemcache;
@@ -515,6 +516,17 @@ def readuntil2crlfs(fd):
     return rv
 
 
+def readlength(fd, size, timeout=1.0):
+    buf = b''
+    with eventlet.Timeout(timeout):
+        while len(buf) < size:
+            chunk = fd.read(min(64, size - len(buf)))
+            buf += chunk
+            if len(buf) >= size:
+                break
+    return buf
+
+
 def connect_tcp(hostport):
     rv = socket.socket()
     rv.connect(hostport)
@@ -1091,7 +1103,7 @@ def requires_o_tmpfile_support_in_tmp(func):
 class StubResponse(object):
 
     def __init__(self, status, body=b'', headers=None, frag_index=None,
-                 slowdown=None):
+                 slowdown=None, slowdown_after=0):
         self.status = status
         self.body = body
         self.readable = BytesIO(body)
@@ -1099,13 +1111,17 @@ class StubResponse(object):
             self._slowdown = iter(slowdown)
         except TypeError:
             self._slowdown = iter([slowdown])
+        self.slowdown_after = slowdown_after
         self.headers = HeaderKeyDict(headers)
         if frag_index is not None:
             self.headers['X-Object-Sysmeta-Ec-Frag-Index'] = frag_index
         fake_reason = ('Fake', 'This response is a lie.')
         self.reason = swob.RESPONSE_REASONS.get(status, fake_reason)[0]
+        self.bytes_read = 0
 
     def slowdown(self):
+        if self.bytes_read < self.slowdown_after:
+            return
         try:
             wait = next(self._slowdown)
         except StopIteration:
@@ -1127,11 +1143,15 @@ class StubResponse(object):
 
     def read(self, amt=0):
         self.slowdown()
-        return self.readable.read(amt)
+        res = self.readable.read(amt)
+        self.bytes_read += len(res)
+        return res
 
     def readline(self, size=-1):
         self.slowdown()
-        return self.readable.readline(size)
+        res = self.readable.readline(size)
+        self.bytes_read += len(res)
+        return res
 
     def __repr__(self):
         info = ['Status: %s' % self.status]
@@ -1405,3 +1425,171 @@ def group_by_byte(contents):
     return [
         (char, sum(1 for _ in grp))
         for char, grp in itertools.groupby(byte_iter)]
+
+
+def generate_db_path(tempdir, server_type):
+    return os.path.join(
+        tempdir, '%ss' % server_type, 'part', 'suffix', 'hash',
+        '%s-%s.db' % (server_type, uuid4()))
+
+
+class ConfigAssertMixin(object):
+    """
+    Use this with a TestCase to get py2/3 compatible assert for DuplicateOption
+    """
+    def assertDuplicateOption(self, app_config, option_name, option_value):
+        """
+        PY3 added a DuplicateOptionError, PY2 didn't seem to care
+        """
+        if six.PY3:
+            self.assertDuplicateOptionError(app_config, option_name)
+        else:
+            self.assertDuplicateOptionOK(app_config, option_name, option_value)
+
+    def assertDuplicateOptionError(self, app_config, option_name):
+        with self.assertRaises(
+                utils.configparser.DuplicateOptionError) as ctx:
+            app_config()
+        msg = str(ctx.exception)
+        self.assertIn(option_name, msg)
+        self.assertIn('already exists', msg)
+
+    def assertDuplicateOptionOK(self, app_config, option_name, option_value):
+        app = app_config()
+        if hasattr(app, 'conf'):
+            found_value = app.conf[option_name]
+        else:
+            if hasattr(app, '_pipeline_final_app'):
+                # special case for proxy app!
+                app = app._pipeline_final_app
+            found_value = getattr(app, option_name)
+        self.assertEqual(found_value, option_value)
+
+
+class FakeSource(object):
+    def __init__(self, chunks, headers=None, body=b''):
+        self.chunks = list(chunks)
+        self.headers = headers or {}
+        self.status = 200
+        self.swift_conn = None
+        self.body = body
+
+    def read(self, _read_size):
+        if self.chunks:
+            chunk = self.chunks.pop(0)
+            if chunk is None:
+                raise exceptions.ChunkReadTimeout()
+            else:
+                return chunk
+        else:
+            return self.body
+
+    def getheader(self, header):
+        # content-length for the whole object is generated dynamically
+        # by summing non-None chunks
+        if header.lower() == "content-length":
+            if self.chunks:
+                return str(sum(len(c) for c in self.chunks
+                               if c is not None))
+            return len(self.read(-1))
+        return self.headers.get(header.lower())
+
+    def getheaders(self):
+        return [('content-length', self.getheader('content-length'))] + \
+               [(k, v) for k, v in self.headers.items()]
+
+
+class CaptureIterator(object):
+    """
+    Wraps an iterable, forwarding all calls to the wrapped iterable but
+    capturing the calls via a callback.
+
+    This class may be used to observe garbage collection, so tests should not
+    have to hold a reference to instances of this class because that would
+    prevent them being garbage collected. Calls are therefore captured via a
+    callback rather than being stashed locally.
+
+    :param wrapped: an iterable to wrap.
+    :param call_capture_callback: a function that will be called to capture
+        calls to this iterator.
+    """
+    def __init__(self, wrapped, call_capture_callback):
+        self.call_capture_callback = call_capture_callback
+        self.wrapped_iter = wrapped
+
+    def _capture_call(self):
+        # call home to capture the call
+        self.call_capture_callback(inspect.stack()[1][3])
+
+    def __iter__(self):
+        return self
+
+    def next(self):
+        self._capture_call()
+        return next(self.wrapped_iter)
+
+    __next__ = next
+
+    def __del__(self):
+        self._capture_call()
+
+    def close(self):
+        self._capture_call()
+        close_if_possible(self.wrapped_iter)
+
+
+class CaptureIteratorFactory(object):
+    """
+    Create instances of ``CaptureIterator`` to wrap a given iterable, and
+    provides a callback function for the ``CaptureIterator`` to capture its
+    calls.
+
+    :param wrapped: an iterable to wrap.
+    """
+    def __init__(self, wrapped):
+        self.wrapped = wrapped
+        self.instance_count = 0
+        self.captured_calls = defaultdict(list)
+
+    def log_call(self, instance_number, call):
+        self.captured_calls[instance_number].append(call)
+
+    def __call__(self, *args, **kwargs):
+        # note: do not keep a reference to the CaptureIterator because that
+        # would prevent it being garbage collected
+        self.instance_count += 1
+        return CaptureIterator(
+            self.wrapped(*args, **kwargs),
+            functools.partial(self.log_call, self.instance_count))
+
+
+def get_node_error_stats(proxy_app, ring_node):
+    node_key = proxy_app.error_limiter.node_key(ring_node)
+    return proxy_app.error_limiter.stats.get(node_key) or {}
+
+
+def node_error_count(proxy_app, ring_node):
+    # Reach into the proxy's internals to get the error count for a
+    # particular node
+    return get_node_error_stats(proxy_app, ring_node).get('errors', 0)
+
+
+def node_error_counts(proxy_app, ring_nodes):
+    # Reach into the proxy's internals to get the error counts for a
+    # list of nodes
+    return sorted([get_node_error_stats(proxy_app, node).get('errors', 0)
+                   for node in ring_nodes], reverse=True)
+
+
+def node_last_error(proxy_app, ring_node):
+    # Reach into the proxy's internals to get the last error for a
+    # particular node
+    return get_node_error_stats(proxy_app, ring_node).get('last_error')
+
+
+def set_node_errors(proxy_app, ring_node, value, last_error):
+    # Set the node's error count to value
+    node_key = proxy_app.error_limiter.node_key(ring_node)
+    stats = {'errors': value,
+             'last_error': last_error}
+    proxy_app.error_limiter.stats[node_key] = stats
