@@ -29,6 +29,7 @@ from swift.common.exceptions import ClientException
 from swift.common.utils import normalize_timestamp, Timestamp
 
 from test import unit
+from test.debug_logger import debug_logger
 from swift.common.storage_policy import StoragePolicy, POLICIES
 
 
@@ -50,7 +51,12 @@ class FakeAccountBroker(object):
                 'delete_timestamp': time.time() - 10}
         return info
 
-    def list_containers_iter(self, limit, marker, *args):
+    def list_containers_iter(self, limit, marker, *args, **kwargs):
+        if not kwargs.pop('allow_reserved'):
+            raise RuntimeError('Expected allow_reserved to be True!')
+        if kwargs:
+            raise RuntimeError('Got unexpected keyword arguments: %r' % (
+                kwargs, ))
         for cont in self.containers:
             if cont > marker:
                 yield cont, None, None, None, None
@@ -203,7 +209,7 @@ class TestReaper(unittest.TestCase):
         r = reaper.AccountReaper(conf)
         r.myips = myips
         if fakelogger:
-            r.logger = unit.debug_logger('test-reaper')
+            r.logger = debug_logger('test-reaper')
         return r
 
     def fake_reap_account(self, *args, **kwargs):
@@ -297,7 +303,7 @@ class TestReaper(unittest.TestCase):
         conf = {
             'mount_check': 'false',
         }
-        r = reaper.AccountReaper(conf, logger=unit.debug_logger())
+        r = reaper.AccountReaper(conf, logger=debug_logger())
         mock_path = 'swift.account.reaper.direct_delete_object'
         for policy in POLICIES:
             r.reset_stats()
@@ -317,7 +323,8 @@ class TestReaper(unittest.TestCase):
                             'X-Container-Partition': 'partition',
                             'X-Container-Device': device,
                             'X-Backend-Storage-Policy-Index': policy.idx,
-                            'X-Timestamp': '1429117638.86767'
+                            'X-Timestamp': '1429117638.86767',
+                            'x-backend-use-replication-network': 'true',
                         }
                         ring = r.get_object_ring(policy.idx)
                         expected = call(dict(ring.devs[i], index=i), 0,
@@ -437,7 +444,8 @@ class TestReaper(unittest.TestCase):
                     'X-Account-Partition': 'partition',
                     'X-Account-Device': device,
                     'X-Account-Override-Deleted': 'yes',
-                    'X-Timestamp': '1429117639.67676'
+                    'X-Timestamp': '1429117639.67676',
+                    'x-backend-use-replication-network': 'true',
                 }
                 ring = r.get_object_ring(policy.idx)
                 expected = call(dict(ring.devs[i], index=i), 0, 'a', 'c',
@@ -461,7 +469,8 @@ class TestReaper(unittest.TestCase):
                 patch('swift.account.reaper.AccountReaper.reap_object',
                       self.fake_reap_object):
             r.reap_container('a', 'partition', acc_nodes, 'c')
-        self.assertEqual(r.logger.get_increment_counts()['return_codes.4'], 1)
+        self.assertEqual(
+            r.logger.statsd_client.get_increment_counts()['return_codes.4'], 1)
         self.assertEqual(r.stats_containers_deleted, 1)
 
     def test_reap_container_partial_fail(self):
@@ -480,7 +489,8 @@ class TestReaper(unittest.TestCase):
                 patch('swift.account.reaper.AccountReaper.reap_object',
                       self.fake_reap_object):
             r.reap_container('a', 'partition', acc_nodes, 'c')
-        self.assertEqual(r.logger.get_increment_counts()['return_codes.4'], 4)
+        self.assertEqual(
+            r.logger.statsd_client.get_increment_counts()['return_codes.4'], 4)
         self.assertEqual(r.stats_containers_possibly_remaining, 1)
 
     def test_reap_container_full_fail(self):
@@ -499,7 +509,8 @@ class TestReaper(unittest.TestCase):
                 patch('swift.account.reaper.AccountReaper.reap_object',
                       self.fake_reap_object):
             r.reap_container('a', 'partition', acc_nodes, 'c')
-        self.assertEqual(r.logger.get_increment_counts()['return_codes.4'], 5)
+        self.assertEqual(
+            r.logger.statsd_client.get_increment_counts()['return_codes.4'], 5)
         self.assertEqual(r.stats_containers_remaining, 1)
 
     def test_reap_container_get_object_timeout(self):
@@ -559,7 +570,7 @@ class TestReaper(unittest.TestCase):
 
     def test_reap_account(self):
         containers = ('c1', 'c2', 'c3', 'c4')
-        broker = FakeAccountBroker(containers, unit.FakeLogger())
+        broker = FakeAccountBroker(containers, debug_logger())
         self.called_amount = 0
         self.r = r = self.init_reaper({}, fakelogger=True)
         r.start_time = time.time()
@@ -595,7 +606,7 @@ class TestReaper(unittest.TestCase):
         self.assertEqual(len(self.r.account_ring.devs), 3)
 
     def test_reap_account_no_container(self):
-        broker = FakeAccountBroker(tuple(), unit.FakeLogger())
+        broker = FakeAccountBroker(tuple(), debug_logger())
         self.r = r = self.init_reaper({}, fakelogger=True)
         self.called_amount = 0
         r.start_time = time.time()
@@ -710,11 +721,16 @@ class TestReaper(unittest.TestCase):
         devices = self.prepare_data_dir()
         self.called_amount = 0
         conf = {'devices': devices}
-        r = self.init_reaper(conf, myips=['10.10.10.2'])
+        r = self.init_reaper(conf, myips=['10.10.10.2'], fakelogger=True)
 
         container_reaped = [0]
 
-        def fake_list_containers_iter(self, *args):
+        def fake_list_containers_iter(self, *args, **kwargs):
+            if not kwargs.pop('allow_reserved'):
+                raise RuntimeError('Expected allow_reserved to be True!')
+            if kwargs:
+                raise RuntimeError('Got unexpected keyword arguments: %r' % (
+                    kwargs, ))
             for container in self.containers:
                 if container in self.containers_yielded:
                     continue
@@ -727,7 +743,7 @@ class TestReaper(unittest.TestCase):
             container_reaped[0] += 1
 
         fake_ring = FakeRing()
-        fake_logger = unit.FakeLogger()
+        fake_logger = debug_logger()
         with patch('swift.account.reaper.AccountBroker',
                    FakeAccountBroker), \
                 patch(
@@ -792,7 +808,7 @@ class TestReaper(unittest.TestCase):
         self.assertFalse(foo.called)
 
         with patch('swift.account.reaper.AccountReaper.reap_device') as foo:
-            r.logger = unit.debug_logger('test-reaper')
+            r.logger = debug_logger('test-reaper')
             r.devices = 'thisdeviceisbad'
             r.run_once()
         self.assertTrue(r.logger.get_lines_for_level(
