@@ -1,3 +1,4 @@
+
 # Copyright (c) 2011-2014 OpenStack Foundation.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -26,18 +27,20 @@ from paste.deploy import loadwsgi
 from six.moves.urllib.parse import unquote, quote
 
 import swift.common.middleware.s3api
+from swift.common.middleware.s3api.s3response import ErrorResponse, \
+    AccessDenied
 from swift.common.middleware.s3api.utils import Config
 from swift.common.middleware.keystoneauth import KeystoneAuth
 from swift.common import swob, registry
 from swift.common.swob import Request
-from swift.common.utils import md5
+from swift.common.utils import md5, get_logger, UTC
 
 from keystonemiddleware.auth_token import AuthProtocol
 from keystoneauth1.access import AccessInfoV2
 
-from test.debug_logger import debug_logger
+from test.debug_logger import debug_logger, FakeStatsdClient
 from test.unit.common.middleware.s3api import S3ApiTestCase
-from test.unit.common.middleware.s3api.helpers import FakeSwift
+from test.unit.common.middleware.helpers import FakeSwift
 from test.unit.common.middleware.s3api.test_s3token import \
     GOOD_RESPONSE_V2, GOOD_RESPONSE_V3
 from swift.common.middleware.s3api.s3request import SigV4Request, S3Request
@@ -212,6 +215,30 @@ class TestS3ApiMiddleware(S3ApiTestCase):
             loader.assert_called_with(loadwsgi.APP, 'proxy-conf-file')
             pipeline.assert_called_with(context)
 
+    def test_init_logger(self):
+        proxy_logger = get_logger({}, log_route='proxy-server').logger
+
+        s3api = S3ApiMiddleware(None, {})
+        self.assertEqual('s3api', s3api.logger.name)
+        self.assertEqual('s3api', s3api.logger.logger.name)
+        self.assertIsNot(s3api.logger.logger, proxy_logger)
+        self.assertEqual('swift', s3api.logger.server)
+        self.assertIsNone(s3api.logger.logger.statsd_client)
+
+        with mock.patch('swift.common.utils.StatsdClient', FakeStatsdClient):
+            s3api = S3ApiMiddleware(None, {'log_name': 'proxy-server',
+                                           'log_statsd_host': '1.2.3.4'})
+            s3api.logger.increment('test-metric')
+        self.assertEqual('s3api', s3api.logger.name)
+        self.assertEqual('s3api', s3api.logger.logger.name)
+        self.assertIsNot(s3api.logger.logger, proxy_logger)
+        self.assertEqual('proxy-server', s3api.logger.server)
+        self.assertEqual('s3api.', s3api.logger.logger.statsd_client._prefix)
+        client = s3api.logger.logger.statsd_client
+        self.assertEqual({'test-metric': 1}, client.get_increment_counts())
+        self.assertEqual(1, len(client.sendto_calls))
+        self.assertEqual(b's3api.test-metric:1|c', client.sendto_calls[0][0])
+
     def test_non_s3_request_passthrough(self):
         req = Request.blank('/something')
         status, headers, body = self.call_s3api(req)
@@ -223,6 +250,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                                      'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'AccessDenied')
+        self.assertEqual(
+            {'403.AccessDenied.invalid_header_auth': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_bad_method(self):
         req = Request.blank('/',
@@ -231,6 +261,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                                      'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'MethodNotAllowed')
+        self.assertEqual(
+            {'405.MethodNotAllowed': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_bad_method_but_method_exists_in_controller(self):
         req = Request.blank(
@@ -240,6 +273,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                      'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'MethodNotAllowed')
+        self.assertEqual(
+            {'405.MethodNotAllowed': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_path_info_encode(self):
         bucket_name = 'b%75cket'
@@ -367,16 +403,19 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                             'AWSAccessKeyId=test:tester' % expire,
                             environ={'REQUEST_METHOD': 'GET'},
                             headers={'Date': self.get_date_header()})
-        req.headers['Date'] = datetime.utcnow()
+        req.headers['Date'] = datetime.now(UTC)
         req.content_type = 'text/plain'
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'AccessDenied')
+        self.assertEqual(
+            {'403.AccessDenied.expired': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_signed_urls(self):
         # Set expire to last 32b timestamp value
         # This number can't be higher, because it breaks tests on 32b systems
         expire = '2147483647'  # 19 Jan 2038 03:14:07
-        utc_date = datetime.utcnow()
+        utc_date = datetime.now(UTC)
         req = Request.blank('/bucket/object?Signature=X&Expires=%s&'
                             'AWSAccessKeyId=test:tester&Timestamp=%s' %
                             (expire, utc_date.isoformat().rsplit('.')[0]),
@@ -413,10 +452,13 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                             'AWSAccessKeyId=test:tester' % expire,
                             environ={'REQUEST_METHOD': 'GET'},
                             headers={'Date': self.get_date_header()})
-        req.headers['Date'] = datetime.utcnow()
+        req.headers['Date'] = datetime.now(UTC)
         req.content_type = 'text/plain'
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'AccessDenied')
+        self.assertEqual(
+            {'403.AccessDenied.invalid_expires': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_signed_urls_no_sign(self):
         expire = '2147483647'  # 19 Jan 2038 03:14:07
@@ -424,20 +466,26 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                             'AWSAccessKeyId=test:tester' % expire,
                             environ={'REQUEST_METHOD': 'GET'},
                             headers={'Date': self.get_date_header()})
-        req.headers['Date'] = datetime.utcnow()
+        req.headers['Date'] = datetime.now(UTC)
         req.content_type = 'text/plain'
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'AccessDenied')
+        self.assertEqual(
+            {'403.AccessDenied.invalid_query_auth': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_signed_urls_no_access(self):
         expire = '2147483647'  # 19 Jan 2038 03:14:07
         req = Request.blank('/bucket/object?Expires=%s&'
                             'AWSAccessKeyId=' % expire,
                             environ={'REQUEST_METHOD': 'GET'})
-        req.headers['Date'] = datetime.utcnow()
+        req.headers['Date'] = datetime.now(UTC)
         req.content_type = 'text/plain'
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'AccessDenied')
+        self.assertEqual(
+            {'403.AccessDenied.invalid_query_auth': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_signed_urls_v4(self):
         req = Request.blank(
@@ -477,12 +525,16 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                 headers={'Date': self.get_date_header()},
                 environ={'REQUEST_METHOD': 'GET'})
             req.content_type = 'text/plain'
+            self.s3api.logger.logger.clear()
             status, headers, body = self.call_s3api(req)
             self.assertEqual(status.split()[0], '400', body)
             self.assertEqual(self._get_error_code(body),
                              'AuthorizationQueryParametersError')
             self.assertEqual(self._get_error_message(body), message)
             self.assertIn(extra, body)
+            self.assertEqual(
+                {'400.AuthorizationQueryParametersError': 1},
+                self.s3api.logger.logger.statsd_client.get_increment_counts())
 
         dt = self.get_v4_amz_date_header().split('T', 1)[0]
         test('test:tester/not-a-date/us-east-1/s3/aws4_request',
@@ -511,6 +563,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
         req.content_type = 'text/plain'
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'AccessDenied')
+        self.assertEqual(
+            {'403.AccessDenied.invalid_date': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_signed_urls_v4_invalid_algorithm(self):
         req = Request.blank(
@@ -526,6 +581,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
         req.content_type = 'text/plain'
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'InvalidArgument')
+        self.assertEqual(
+            {'400.InvalidArgument': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_signed_urls_v4_missing_signed_headers(self):
         req = Request.blank(
@@ -541,6 +599,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body),
                          'AuthorizationHeaderMalformed')
+        self.assertEqual(
+            {'400.AuthorizationHeaderMalformed': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_signed_urls_v4_invalid_credentials(self):
         req = Request.blank('/bucket/object'
@@ -555,6 +616,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
         req.content_type = 'text/plain'
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'AccessDenied')
+        self.assertEqual(
+            {'403.AccessDenied.invalid_credential': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_signed_urls_v4_missing_signature(self):
         req = Request.blank(
@@ -569,6 +633,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
         req.content_type = 'text/plain'
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'AccessDenied')
+        self.assertEqual(
+            {'403.AccessDenied.invalid_query_auth': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_bucket_virtual_hosted_style(self):
         req = Request.blank('/',
@@ -665,6 +732,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                                      'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'InvalidURI')
+        self.assertEqual(
+            {'400.InvalidURI': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_object_create_bad_md5_unreadable(self):
         req = Request.blank('/bucket/object',
@@ -674,6 +744,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                             headers={'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'InvalidDigest')
+        self.assertEqual(
+            {'400.InvalidDigest': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_object_create_bad_md5_too_short(self):
         too_short_digest = md5(b'hey', usedforsecurity=False).digest()[:-1]
@@ -688,6 +761,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
             headers={'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'InvalidDigest')
+        self.assertEqual(
+            {'400.InvalidDigest': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_object_create_bad_md5_bad_padding(self):
         too_short_digest = md5(b'hey', usedforsecurity=False).digest()
@@ -702,6 +778,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
             headers={'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'InvalidDigest')
+        self.assertEqual(
+            {'400.InvalidDigest': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_object_create_bad_md5_too_long(self):
         too_long_digest = md5(
@@ -717,6 +796,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
             headers={'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'InvalidDigest')
+        self.assertEqual(
+            {'400.InvalidDigest': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_invalid_metadata_directive(self):
         req = Request.blank('/',
@@ -727,6 +809,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                             headers={'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'InvalidArgument')
+        self.assertEqual(
+            {'400.InvalidArgument': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_invalid_storage_class(self):
         req = Request.blank('/',
@@ -736,6 +821,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                             headers={'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'InvalidStorageClass')
+        self.assertEqual(
+            {'400.InvalidStorageClass': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_invalid_ssc(self):
         req = Request.blank('/',
@@ -745,6 +833,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                                      'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'InvalidArgument')
+        self.assertEqual(
+            {'400.InvalidArgument': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def _test_unsupported_header(self, header, value=None):
         if value is None:
@@ -754,8 +845,12 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                                      'HTTP_AUTHORIZATION': 'AWS X:Y:Z'},
                             headers={header: value,
                                      'Date': self.get_date_header()})
+        self.s3api.logger.logger.clear()
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'NotImplemented')
+        self.assertEqual(
+            {'501.NotImplemented': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_mfa(self):
         self._test_unsupported_header('x-amz-mfa')
@@ -815,6 +910,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                             headers={'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'NotImplemented')
+        self.assertEqual(
+            {'501.NotImplemented': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_notification(self):
         self._test_unsupported_resource('notification')
@@ -841,18 +939,31 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                                      'Date': self.get_date_header()})
         status, headers, body = self.call_s3api(req)
         self.assertEqual(status.split()[0], '200')
+        self.assertEqual(
+            {},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
+
         req = Request.blank('/bucket?tagging',
                             environ={'REQUEST_METHOD': 'PUT'},
                             headers={'Authorization': 'AWS test:tester:hmac',
                                      'Date': self.get_date_header()})
+        self.s3api.logger.logger.clear()
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'NotImplemented')
+        self.assertEqual(
+            {'501.NotImplemented': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
+
         req = Request.blank('/bucket?tagging',
                             environ={'REQUEST_METHOD': 'DELETE'},
                             headers={'Authorization': 'AWS test:tester:hmac',
                                      'Date': self.get_date_header()})
+        self.s3api.logger.logger.clear()
         status, headers, body = self.call_s3api(req)
         self.assertEqual(self._get_error_code(body), 'NotImplemented')
+        self.assertEqual(
+            {'501.NotImplemented': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_restore(self):
         self._test_unsupported_resource('restore')
@@ -867,6 +978,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
         self.assertEqual(elem.find('./Code').text, 'MethodNotAllowed')
         self.assertEqual(elem.find('./Method').text, 'POST')
         self.assertEqual(elem.find('./ResourceType').text, 'ACL')
+        self.assertEqual(
+            {'405.MethodNotAllowed': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     @mock.patch.object(registry, '_sensitive_headers', set())
     @mock.patch.object(registry, '_sensitive_params', set())
@@ -916,9 +1030,6 @@ class TestS3ApiMiddleware(S3ApiTestCase):
             pipeline.return_value = 's3api s3token keystoneauth proxy-server'
             self.s3api.check_pipeline(self.conf)
 
-            pipeline.return_value = 's3api swauth proxy-server'
-            self.s3api.check_pipeline(self.conf)
-
             # Note that authtoken would need to have delay_auth_decision=True
             pipeline.return_value = 's3api authtoken s3token keystoneauth ' \
                 'proxy-server'
@@ -952,9 +1063,6 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                 'proxy-server'
             self.s3api.check_pipeline(self.conf)
 
-            pipeline.return_value = 's3api swauth proxy-server'
-            self.s3api.check_pipeline(self.conf)
-
             pipeline.return_value = 's3api authtoken s3token keystoneauth ' \
                 'proxy-server'
             self.s3api.check_pipeline(self.conf)
@@ -1005,6 +1113,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
         status, headers, body = self.call_s3api(req)
         self.assertEqual(status.split()[0], '403')
         self.assertEqual(self._get_error_code(body), 'AccessDenied')
+        self.assertEqual(
+            {'403.AccessDenied.invalid_date': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_signature_v4_no_payload(self):
         environ = {
@@ -1024,9 +1135,12 @@ class TestS3ApiMiddleware(S3ApiTestCase):
         self.assertEqual(
             self._get_error_message(body),
             'Missing required header for this request: x-amz-content-sha256')
+        self.assertEqual(
+            {'400.InvalidRequest': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_signature_v4_bad_authorization_string(self):
-        def test(auth_str, error, msg, extra=b''):
+        def test(auth_str, error, msg, metric, extra=b''):
             environ = {
                 'REQUEST_METHOD': 'GET'}
             headers = {
@@ -1036,15 +1150,20 @@ class TestS3ApiMiddleware(S3ApiTestCase):
             req = Request.blank('/bucket/object', environ=environ,
                                 headers=headers)
             req.content_type = 'text/plain'
+            self.s3api.logger.logger.clear()
             status, headers, body = self.call_s3api(req)
             self.assertEqual(self._get_error_code(body), error)
             self.assertEqual(self._get_error_message(body), msg)
             self.assertIn(extra, body)
+            self.assertEqual(
+                {metric: 1},
+                self.s3api.logger.logger.statsd_client.get_increment_counts())
 
         auth_str = ('AWS4-HMAC-SHA256 '
                     'SignedHeaders=host;x-amz-date,'
                     'Signature=X')
-        test(auth_str, 'AccessDenied', 'Access Denied.')
+        test(auth_str, 'AccessDenied', 'Access Denied.',
+             '403.AccessDenied.invalid_credential')
 
         auth_str = (
             'AWS4-HMAC-SHA256 '
@@ -1053,7 +1172,7 @@ class TestS3ApiMiddleware(S3ApiTestCase):
         test(auth_str, 'AuthorizationHeaderMalformed',
              'The authorization header is malformed; the authorization '
              'header requires three components: Credential, SignedHeaders, '
-             'and Signature.')
+             'and Signature.', '400.AuthorizationHeaderMalformed')
 
         auth_str = ('AWS4-HMAC-SHA256 '
                     'Credential=test:tester/%s/us-west-2/s3/aws4_request, '
@@ -1062,7 +1181,7 @@ class TestS3ApiMiddleware(S3ApiTestCase):
         test(auth_str, 'AuthorizationHeaderMalformed',
              "The authorization header is malformed; "
              "the region 'us-west-2' is wrong; expecting 'us-east-1'",
-             b'<Region>us-east-1</Region>')
+             '400.AuthorizationHeaderMalformed', b'<Region>us-east-1</Region>')
 
         auth_str = ('AWS4-HMAC-SHA256 '
                     'Credential=test:tester/%s/us-east-1/not-s3/aws4_request, '
@@ -1070,7 +1189,8 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                     self.get_v4_amz_date_header().split('T', 1)[0])
         test(auth_str, 'AuthorizationHeaderMalformed',
              'The authorization header is malformed; '
-             'incorrect service "not-s3". This endpoint belongs to "s3".')
+             'incorrect service "not-s3". This endpoint belongs to "s3".',
+             '400.AuthorizationHeaderMalformed')
 
         auth_str = ('AWS4-HMAC-SHA256 '
                     'Credential=test:tester/%s/us-east-1/s3/not-aws4_request, '
@@ -1079,13 +1199,15 @@ class TestS3ApiMiddleware(S3ApiTestCase):
         test(auth_str, 'AuthorizationHeaderMalformed',
              'The authorization header is malformed; '
              'incorrect terminal "not-aws4_request". '
-             'This endpoint uses "aws4_request".')
+             'This endpoint uses "aws4_request".',
+             '400.AuthorizationHeaderMalformed')
 
         auth_str = (
             'AWS4-HMAC-SHA256 '
             'Credential=test:tester/20130524/us-east-1/s3/aws4_request, '
             'SignedHeaders=host;x-amz-date')
-        test(auth_str, 'AccessDenied', 'Access Denied.')
+        test(auth_str, 'AccessDenied', 'Access Denied.',
+             '403.AccessDenied.invalid_header_auth')
 
     def test_canonical_string_v4(self):
         def _get_req(path, environ):
@@ -1297,6 +1419,9 @@ class TestS3ApiMiddleware(S3ApiTestCase):
         # FIXME: should this failed as 400 or pass via query auth?
         # for now, 403 forbidden for safety
         self.assertEqual(status.split()[0], '403', body)
+        self.assertEqual(
+            {'403.AccessDenied.invalid_expires': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
         # But if we are missing Signature in query param
         req = Request.blank(
@@ -1307,8 +1432,12 @@ class TestS3ApiMiddleware(S3ApiTestCase):
             environ={'REQUEST_METHOD': 'GET'},
             headers=headers)
         req.content_type = 'text/plain'
+        self.s3api.logger.logger.clear()
         status, headers, body = self.call_s3api(req)
         self.assertEqual(status.split()[0], '403', body)
+        self.assertEqual(
+            {'403.AccessDenied.invalid_expires': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
 
     def test_s3api_with_only_s3_token(self):
         self.swift = FakeSwift()
@@ -1420,6 +1549,10 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                 self.assertEqual(1, mock_req.call_count)
                 # it never even tries to contact keystone
                 self.assertEqual(0, mock_fetch.call_count)
+                statsd_client = self.s3api.logger.logger.statsd_client
+                self.assertEqual(
+                    {'403.SignatureDoesNotMatch': 1},
+                    statsd_client.get_increment_counts())
 
     def test_s3api_with_only_s3_token_in_s3acl(self):
         self.swift = FakeSwift()
@@ -1442,8 +1575,6 @@ class TestS3ApiMiddleware(S3ApiTestCase):
         # after PUT container so we need to register the resposne here
         self.swift.register('POST', '/v1/AUTH_TENANT_ID/bucket',
                             swob.HTTPNoContent, {}, None)
-        self.swift.register('TEST', '/v1/AUTH_TENANT_ID',
-                            swob.HTTPMethodNotAllowed, {}, None)
         with patch.object(self.s3_token, '_json_request') as mock_req:
             mock_resp = requests.Response()
             mock_resp._content = json.dumps(GOOD_RESPONSE_V2).encode('ascii')
@@ -1457,6 +1588,83 @@ class TestS3ApiMiddleware(S3ApiTestCase):
                              req.environ['swift.backend_path'])
             self.assertEqual(1, mock_req.call_count)
 
+    def test_s3api_with_time_skew(self):
+        def do_test(skew):
+            req = Request.blank(
+                '/object',
+                environ={'HTTP_HOST': 'bucket.localhost:80',
+                         'REQUEST_METHOD': 'GET',
+                         'HTTP_AUTHORIZATION':
+                             'AWS test:tester:hmac'},
+                headers={'Date': self.get_date_header(skew=skew)})
+            self.s3api.logger.logger.clear()
+            return self.call_s3api(req)
+
+        status, _, body = do_test(800)
+        self.assertEqual('200 OK', status)
+        self.assertFalse(
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
+
+        status, _, body = do_test(-800)
+        self.assertEqual('200 OK', status)
+        self.assertFalse(
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
+
+        status, _, body = do_test(1000)
+        self.assertEqual('403 Forbidden', status)
+        self.assertEqual(self._get_error_code(body), 'RequestTimeTooSkewed')
+        self.assertEqual(
+            {'403.RequestTimeTooSkewed': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
+
+        status, _, body = do_test(-1000)
+        self.assertEqual('403 Forbidden', status)
+        self.assertEqual(self._get_error_code(body), 'RequestTimeTooSkewed')
+        self.assertEqual(
+            {'403.RequestTimeTooSkewed': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
+
+        self.s3api.conf.allowable_clock_skew = 100
+        status, _, body = do_test(800)
+        self.assertEqual('403 Forbidden', status)
+        self.assertEqual(self._get_error_code(body), 'RequestTimeTooSkewed')
+        self.assertEqual(
+            {'403.RequestTimeTooSkewed': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
+
+    def test_s3api_error_metric(self):
+        class KaboomResponse(ErrorResponse):
+            _code = 'ka boom'
+
+        def do_test(err_response):
+            req = Request.blank(
+                '/object',
+                environ={'HTTP_HOST': 'bucket.localhost:80',
+                         'REQUEST_METHOD': 'GET',
+                         'HTTP_AUTHORIZATION':
+                             'AWS test:tester:hmac'},
+                headers={'Date': self.get_date_header()})
+            self.s3api.logger.logger.clear()
+            with mock.patch.object(
+                    self.s3api, 'handle_request', side_effect=err_response):
+                self.call_s3api(req)
+
+        do_test(ErrorResponse(status=403, msg='not good', reason='bad'))
+        self.assertEqual(
+            {'403.ErrorResponse.bad': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
+
+        do_test(AccessDenied(msg='no entry', reason='invalid_date'))
+        self.assertEqual(
+            {'403.AccessDenied.invalid_date': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
+
+        # check whitespace replaced with underscore
+        do_test(KaboomResponse(status=400, msg='boom', reason='boom boom'))
+        self.assertEqual(
+            {'400.ka_boom.boom_boom': 1},
+            self.s3api.logger.logger.statsd_client.get_increment_counts())
+
 
 if __name__ == '__main__':
     unittest.main()
