@@ -13,12 +13,13 @@
 
 import unittest
 
-from swift.common.swob import Request, wsgify, HTTPForbidden
+from swift.common.swob import Request, wsgify, HTTPForbidden, HTTPOk, \
+    HTTPServiceUnavailable, HTTPNotFound
 
 from swift.common.middleware import account_quotas, copy
 
-from swift.proxy.controllers.base import get_cache_key, \
-    headers_to_account_info, headers_to_object_info
+from test.unit import patch_policies
+from test.unit.common.middleware.helpers import FakeSwift
 
 
 class FakeCache(object):
@@ -32,46 +33,6 @@ class FakeCache(object):
         pass
 
 
-class FakeBadApp(object):
-    def __init__(self, headers=None):
-        if headers is None:
-            headers = []
-        self.headers = headers
-
-    def __call__(self, env, start_response):
-        start_response('404 NotFound', self.headers)
-        return []
-
-
-class FakeApp(object):
-    def __init__(self, headers=None):
-        if headers is None:
-            headers = []
-        self.headers = headers
-
-    def __call__(self, env, start_response):
-        if 'swift.authorize' in env:
-            aresp = env['swift.authorize'](Request(env))
-            if aresp:
-                return aresp(env, start_response)
-        if env['REQUEST_METHOD'] == "HEAD" and \
-                env['PATH_INFO'] == '/v1/a/c2/o2':
-            cache_key = get_cache_key('a', 'c2', 'o2')
-            env.setdefault('swift.infocache', {})[cache_key] = \
-                headers_to_object_info(self.headers, 200)
-            start_response('200 OK', self.headers)
-        elif env['REQUEST_METHOD'] == "HEAD" and \
-                env['PATH_INFO'] == '/v1/a/c2/o3':
-            start_response('404 Not Found', [])
-        else:
-            # Cache the account_info (same as a real application)
-            cache_key = get_cache_key('a')
-            env.setdefault('swift.infocache', {})[cache_key] = \
-                headers_to_account_info(self.headers, 200)
-            start_response('200 OK', self.headers)
-        return []
-
-
 class FakeAuthFilter(object):
 
     def __init__(self, app):
@@ -89,9 +50,17 @@ class FakeAuthFilter(object):
 
 class TestAccountQuota(unittest.TestCase):
 
+    def setUp(self):
+        self.app = FakeSwift()
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '1000'})
+        self.app.register('HEAD', '/v1/a/c', HTTPOk, {
+            'x-backend-storage-policy-index': '1'})
+        self.app.register('POST', '/v1/a', HTTPOk, {})
+        self.app.register('PUT', '/v1/a/c/o', HTTPOk, {})
+
     def test_unauthorized(self):
-        headers = [('x-account-bytes-used', '1000'), ]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
@@ -101,8 +70,7 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 200)
 
     def test_no_quotas(self):
-        headers = [('x-account-bytes-used', '1000'), ]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
@@ -114,8 +82,7 @@ class TestAccountQuota(unittest.TestCase):
         # If you try to set X-Account-Meta-* on an object, it's ignored, so
         # the quota middleware shouldn't complain about it even if we're not a
         # reseller admin.
-        headers = [('x-account-bytes-used', '1000')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             headers={'X-Account-Meta-Quota-Bytes': '99999'},
@@ -127,8 +94,8 @@ class TestAccountQuota(unittest.TestCase):
     def test_container_request_ignores_attempt_to_set_quotas(self):
         # As with an object, if you try to set X-Account-Meta-* on a
         # container, it's ignored.
-        headers = [('x-account-bytes-used', '1000')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        self.app.register('PUT', '/v1/a/c', HTTPOk, {})
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c',
                             headers={'X-Account-Meta-Quota-Bytes': '99999'},
@@ -140,9 +107,10 @@ class TestAccountQuota(unittest.TestCase):
     def test_bogus_quota_is_ignored(self):
         # This can happen if the metadata was set by a user prior to the
         # activation of the account-quota middleware
-        headers = [('x-account-bytes-used', '1000'),
-                   ('x-account-meta-quota-bytes', 'pasty-plastogene')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '1000',
+            'x-account-meta-quota-bytes': 'pasty-plastogene'})
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
@@ -151,9 +119,10 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 200)
 
     def test_exceed_bytes_quota(self):
-        headers = [('x-account-bytes-used', '1000'),
-                   ('x-account-meta-quota-bytes', '0')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '1000',
+            'x-account-meta-quota-bytes': '0'})
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
@@ -162,11 +131,53 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 413)
         self.assertEqual(res.body, b'Upload exceeds quota.')
 
+    @patch_policies
+    def test_exceed_per_policy_quota(self):
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '100',
+            'x-account-storage-policy-unu-bytes-used': '100',
+            'x-account-sysmeta-quota-bytes-policy-1': '10',
+            'x-account-meta-quota-bytes': '1000'})
+        app = account_quotas.AccountQuotaMiddleware(self.app)
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a/c/o',
+                            environ={'REQUEST_METHOD': 'PUT',
+                                     'swift.cache': cache})
+        res = req.get_response(app)
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, b'Upload exceeds policy quota.')
+
+    @patch_policies
+    def test_policy_quota_translation(self):
+        def do_test(method):
+            self.app.register(method, '/v1/a', HTTPOk, {
+                'x-account-bytes-used': '100',
+                'x-account-storage-policy-unu-bytes-used': '100',
+                'x-account-sysmeta-quota-bytes-policy-1': '10',
+                'x-account-meta-quota-bytes': '1000'})
+            app = account_quotas.AccountQuotaMiddleware(self.app)
+            cache = FakeCache(None)
+            req = Request.blank('/v1/a', method=method, environ={
+                'swift.cache': cache})
+            res = req.get_response(app)
+            self.assertEqual(res.status_int, 200)
+            self.assertEqual(res.headers.get(
+                'X-Account-Meta-Quota-Bytes'), '1000')
+            self.assertEqual(res.headers.get(
+                'X-Account-Sysmeta-Quota-Bytes-Policy-1'), '10')
+            self.assertEqual(res.headers.get(
+                'X-Account-Quota-Bytes-Policy-Unu'), '10')
+            self.assertEqual(res.headers.get(
+                'X-Account-Storage-Policy-Unu-Bytes-Used'), '100')
+
+        do_test('GET')
+        do_test('HEAD')
+
     def test_exceed_quota_not_authorized(self):
-        headers = [('x-account-bytes-used', '1000'),
-                   ('x-account-meta-quota-bytes', '0')]
-        app = FakeAuthFilter(
-            account_quotas.AccountQuotaMiddleware(FakeApp(headers)))
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '1000',
+            'x-account-meta-quota-bytes': '0'})
+        app = FakeAuthFilter(account_quotas.AccountQuotaMiddleware(self.app))
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o', method='PUT',
                             headers={'x-auth-token': 'bad-secret'},
@@ -175,10 +186,10 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 403)
 
     def test_exceed_quota_authorized(self):
-        headers = [('x-account-bytes-used', '1000'),
-                   ('x-account-meta-quota-bytes', '0')]
-        app = FakeAuthFilter(
-            account_quotas.AccountQuotaMiddleware(FakeApp(headers)))
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '1000',
+            'x-account-meta-quota-bytes': '0'})
+        app = FakeAuthFilter(account_quotas.AccountQuotaMiddleware(self.app))
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o', method='PUT',
                             headers={'x-auth-token': 'secret'},
@@ -187,10 +198,10 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 413)
 
     def test_under_quota_not_authorized(self):
-        headers = [('x-account-bytes-used', '0'),
-                   ('x-account-meta-quota-bytes', '1000')]
-        app = FakeAuthFilter(
-            account_quotas.AccountQuotaMiddleware(FakeApp(headers)))
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '0',
+            'x-account-meta-quota-bytes': '1000'})
+        app = FakeAuthFilter(account_quotas.AccountQuotaMiddleware(self.app))
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o', method='PUT',
                             headers={'x-auth-token': 'bad-secret'},
@@ -199,10 +210,10 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 403)
 
     def test_under_quota_authorized(self):
-        headers = [('x-account-bytes-used', '0'),
-                   ('x-account-meta-quota-bytes', '1000')]
-        app = FakeAuthFilter(
-            account_quotas.AccountQuotaMiddleware(FakeApp(headers)))
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '0',
+            'x-account-meta-quota-bytes': '1000'})
+        app = FakeAuthFilter(account_quotas.AccountQuotaMiddleware(self.app))
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o', method='PUT',
                             headers={'x-auth-token': 'secret'},
@@ -210,10 +221,40 @@ class TestAccountQuota(unittest.TestCase):
         res = req.get_response(app)
         self.assertEqual(res.status_int, 200)
 
+    def test_exceed_quota_bytes_on_empty_account_not_authorized(self):
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '0',
+            'x-account-meta-quota-bytes': '10'})
+        app = FakeAuthFilter(account_quotas.AccountQuotaMiddleware(self.app))
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a/c/o', method='PUT',
+                            headers={'x-auth-token': 'secret',
+                                     'content-length': '100'},
+                            environ={'swift.cache': cache})
+        res = req.get_response(app)
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, b'Upload exceeds quota.')
+
+    def test_exceed_quota_bytes_not_authorized(self):
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '100',
+            'x-account-meta-quota-bytes': '1000'})
+        app = FakeAuthFilter(account_quotas.AccountQuotaMiddleware(self.app))
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a/c/o', method='PUT',
+                            headers={'x-auth-token': 'secret',
+                                     'content-length': '901'},
+                            environ={'swift.cache': cache})
+        res = req.get_response(app)
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, b'Upload exceeds quota.')
+
     def test_over_quota_container_create_still_works(self):
-        headers = [('x-account-bytes-used', '1001'),
-                   ('x-account-meta-quota-bytes', '1000')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '1001',
+            'x-account-meta-quota-bytes': '1000'})
+        self.app.register('PUT', '/v1/a/new_container', HTTPOk, {})
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a/new_container',
                             environ={'REQUEST_METHOD': 'PUT',
@@ -223,9 +264,11 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 200)
 
     def test_over_quota_container_post_still_works(self):
-        headers = [('x-account-bytes-used', '1001'),
-                   ('x-account-meta-quota-bytes', '1000')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '1001',
+            'x-account-meta-quota-bytes': '1000'})
+        self.app.register('POST', '/v1/a/new_container', HTTPOk, {})
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a/new_container',
                             environ={'REQUEST_METHOD': 'POST',
@@ -235,9 +278,11 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 200)
 
     def test_over_quota_obj_post_still_works(self):
-        headers = [('x-account-bytes-used', '1001'),
-                   ('x-account-meta-quota-bytes', '1000')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '1001',
+            'x-account-meta-quota-bytes': '1000'})
+        self.app.register('POST', '/v1/a/c/o', HTTPOk, {})
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'POST',
@@ -247,9 +292,11 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 200)
 
     def test_exceed_bytes_quota_reseller(self):
-        headers = [('x-account-bytes-used', '1000'),
-                   ('x-account-meta-quota-bytes', '0')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '1000',
+            'x-account-meta-quota-bytes': '0'})
+        self.app.register('PUT', '/v1/a', HTTPOk, {})
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a',
                             environ={'REQUEST_METHOD': 'PUT',
@@ -259,10 +306,13 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 200)
 
     def test_exceed_bytes_quota_reseller_copy_from(self):
-        headers = [('x-account-bytes-used', '500'),
-                   ('x-account-meta-quota-bytes', '1000'),
-                   ('content-length', '1000')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '500',
+            'x-account-meta-quota-bytes': '1000'})
+        self.app.register('GET', '/v1/a/c2/o2', HTTPOk, {
+            'content-length': '1000'}, b'a' * 1000)
+        app = copy.filter_factory({})(
+            account_quotas.AccountQuotaMiddleware(self.app))
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
@@ -273,10 +323,13 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 200)
 
     def test_exceed_bytes_quota_reseller_copy_verb(self):
-        headers = [('x-account-bytes-used', '500'),
-                   ('x-account-meta-quota-bytes', '1000'),
-                   ('content-length', '1000')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '500',
+            'x-account-meta-quota-bytes': '1000'})
+        self.app.register('GET', '/v1/a/c2/o2', HTTPOk, {
+            'content-length': '1000'}, b'a' * 1000)
+        app = copy.filter_factory({})(
+            account_quotas.AccountQuotaMiddleware(self.app))
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c2/o2',
                             environ={'REQUEST_METHOD': 'COPY',
@@ -287,8 +340,8 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 200)
 
     def test_bad_application_quota(self):
-        headers = []
-        app = account_quotas.AccountQuotaMiddleware(FakeBadApp(headers))
+        self.app.register('PUT', '/v1/a/c/o', HTTPNotFound, {})
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
@@ -297,8 +350,7 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 404)
 
     def test_no_info_quota(self):
-        headers = []
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
@@ -307,9 +359,10 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 200)
 
     def test_not_exceed_bytes_quota(self):
-        headers = [('x-account-bytes-used', '1000'),
-                   ('x-account-meta-quota-bytes', 2000)]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        self.app.register('HEAD', '/v1/a', HTTPOk, {
+            'x-account-bytes-used': '1000',
+            'x-account-meta-quota-bytes': '2000'})
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
@@ -318,8 +371,7 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 200)
 
     def test_invalid_quotas(self):
-        headers = [('x-account-bytes-used', '0'), ]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a',
                             environ={'REQUEST_METHOD': 'POST',
@@ -328,10 +380,23 @@ class TestAccountQuota(unittest.TestCase):
                                      'reseller_request': True})
         res = req.get_response(app)
         self.assertEqual(res.status_int, 400)
+        self.assertEqual(self.app.calls, [])
+
+    @patch_policies
+    def test_invalid_policy_quota(self):
+        app = account_quotas.AccountQuotaMiddleware(self.app)
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a', environ={
+            'REQUEST_METHOD': 'POST',
+            'swift.cache': cache,
+            'HTTP_X_ACCOUNT_QUOTA_BYTES_POLICY_NULO': 'abc',
+            'reseller_request': True})
+        res = req.get_response(app)
+        self.assertEqual(res.status_int, 400)
+        self.assertEqual(self.app.calls, [])
 
     def test_valid_quotas_admin(self):
-        headers = [('x-account-bytes-used', '0'), ]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a',
                             environ={'REQUEST_METHOD': 'POST',
@@ -339,10 +404,22 @@ class TestAccountQuota(unittest.TestCase):
                                      'HTTP_X_ACCOUNT_META_QUOTA_BYTES': '100'})
         res = req.get_response(app)
         self.assertEqual(res.status_int, 403)
+        self.assertEqual(self.app.calls, [])
+
+    @patch_policies
+    def test_valid_policy_quota_admin(self):
+        app = account_quotas.AccountQuotaMiddleware(self.app)
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a', environ={
+            'REQUEST_METHOD': 'POST',
+            'swift.cache': cache,
+            'HTTP_X_ACCOUNT_QUOTA_BYTES_POLICY_UNU': '100'})
+        res = req.get_response(app)
+        self.assertEqual(res.status_int, 403)
+        self.assertEqual(self.app.calls, [])
 
     def test_valid_quotas_reseller(self):
-        headers = [('x-account-bytes-used', '0'), ]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a',
                             environ={'REQUEST_METHOD': 'POST',
@@ -351,10 +428,28 @@ class TestAccountQuota(unittest.TestCase):
                                      'reseller_request': True})
         res = req.get_response(app)
         self.assertEqual(res.status_int, 200)
+        self.assertEqual(self.app.calls_with_headers, [
+            ('POST', '/v1/a', {'Host': 'localhost:80',
+                               'X-Account-Meta-Quota-Bytes': '100'})])
+
+    @patch_policies
+    def test_valid_policy_quota_reseller(self):
+        app = account_quotas.AccountQuotaMiddleware(self.app)
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a', environ={
+            'REQUEST_METHOD': 'POST',
+            'swift.cache': cache,
+            'HTTP_X_ACCOUNT_QUOTA_BYTES_POLICY_NULO': '100',
+            'reseller_request': True})
+        res = req.get_response(app)
+        self.assertEqual(res.status_int, 200)
+        self.assertEqual(self.app.calls_with_headers, [
+            ('POST', '/v1/a', {
+                'Host': 'localhost:80',
+                'X-Account-Sysmeta-Quota-Bytes-Policy-0': '100'})])
 
     def test_delete_quotas(self):
-        headers = [('x-account-bytes-used', '0'), ]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a',
                             environ={'REQUEST_METHOD': 'POST',
@@ -364,8 +459,7 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 403)
 
     def test_delete_quotas_with_remove_header(self):
-        headers = [('x-account-bytes-used', '0'), ]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a', environ={
             'REQUEST_METHOD': 'POST',
@@ -375,8 +469,7 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 403)
 
     def test_delete_quotas_reseller(self):
-        headers = [('x-account-bytes-used', '0'), ]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         req = Request.blank('/v1/a',
                             environ={'REQUEST_METHOD': 'POST',
                                      'HTTP_X_ACCOUNT_META_QUOTA_BYTES': '',
@@ -385,8 +478,7 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 200)
 
     def test_delete_quotas_with_remove_header_reseller(self):
-        headers = [('x-account-bytes-used', '0'), ]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1/a', environ={
             'REQUEST_METHOD': 'POST',
@@ -397,29 +489,32 @@ class TestAccountQuota(unittest.TestCase):
         self.assertEqual(res.status_int, 200)
 
     def test_invalid_request_exception(self):
-        headers = [('x-account-bytes-used', '1000'), ]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
+        self.app.register('PUT', '/v1', HTTPServiceUnavailable, {})
+        app = account_quotas.AccountQuotaMiddleware(self.app)
         cache = FakeCache(None)
         req = Request.blank('/v1',
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        # Response code of 200 because authentication itself is not done here
-        self.assertEqual(res.status_int, 200)
+        self.assertEqual(res.status_int, 503)
 
 
 class AccountQuotaCopyingTestCases(unittest.TestCase):
 
     def setUp(self):
-        self.app = FakeApp()
+        self.headers = []
+        self.app = FakeSwift()
+        self.app.register('HEAD', '/v1/a', HTTPOk, self.headers)
+        self.app.register('HEAD', '/v1/a/c', HTTPOk, {
+            'x-backend-storage-policy-index': '1'})
+        self.app.register('GET', '/v1/a/c2/o2', HTTPOk, {
+            'content-length': '1000'})
         self.aq_filter = account_quotas.filter_factory({})(self.app)
         self.copy_filter = copy.filter_factory({})(self.aq_filter)
 
     def test_exceed_bytes_quota_copy_from(self):
-        headers = [('x-account-bytes-used', '500'),
-                   ('x-account-meta-quota-bytes', '1000'),
-                   ('content-length', '1000')]
-        self.app.headers = headers
+        self.headers[:] = [('x-account-bytes-used', '500'),
+                           ('x-account-meta-quota-bytes', '1000')]
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
@@ -430,10 +525,8 @@ class AccountQuotaCopyingTestCases(unittest.TestCase):
         self.assertEqual(res.body, b'Upload exceeds quota.')
 
     def test_exceed_bytes_quota_copy_verb(self):
-        headers = [('x-account-bytes-used', '500'),
-                   ('x-account-meta-quota-bytes', '1000'),
-                   ('content-length', '1000')]
-        self.app.headers = headers
+        self.headers[:] = [('x-account-bytes-used', '500'),
+                           ('x-account-meta-quota-bytes', '1000')]
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c2/o2',
                             environ={'REQUEST_METHOD': 'COPY',
@@ -444,10 +537,9 @@ class AccountQuotaCopyingTestCases(unittest.TestCase):
         self.assertEqual(res.body, b'Upload exceeds quota.')
 
     def test_not_exceed_bytes_quota_copy_from(self):
-        headers = [('x-account-bytes-used', '0'),
-                   ('x-account-meta-quota-bytes', '1000'),
-                   ('content-length', '1000')]
-        self.app.headers = headers
+        self.app.register('PUT', '/v1/a/c/o', HTTPOk, {})
+        self.headers[:] = [('x-account-bytes-used', '0'),
+                           ('x-account-meta-quota-bytes', '1000')]
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
@@ -457,10 +549,9 @@ class AccountQuotaCopyingTestCases(unittest.TestCase):
         self.assertEqual(res.status_int, 200)
 
     def test_not_exceed_bytes_quota_copy_verb(self):
-        headers = [('x-account-bytes-used', '0'),
-                   ('x-account-meta-quota-bytes', '1000'),
-                   ('content-length', '1000')]
-        self.app.headers = headers
+        self.app.register('PUT', '/v1/a/c/o', HTTPOk, {})
+        self.headers[:] = [('x-account-bytes-used', '0'),
+                           ('x-account-meta-quota-bytes', '1000')]
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c2/o2',
                             environ={'REQUEST_METHOD': 'COPY',
@@ -469,22 +560,9 @@ class AccountQuotaCopyingTestCases(unittest.TestCase):
         res = req.get_response(self.copy_filter)
         self.assertEqual(res.status_int, 200)
 
-    def test_quota_copy_from_no_src(self):
-        headers = [('x-account-bytes-used', '0'),
-                   ('x-account-meta-quota-bytes', '1000')]
-        self.app.headers = headers
-        cache = FakeCache(None)
-        req = Request.blank('/v1/a/c/o',
-                            environ={'REQUEST_METHOD': 'PUT',
-                                     'swift.cache': cache},
-                            headers={'x-copy-from': '/c2/o3'})
-        res = req.get_response(self.copy_filter)
-        self.assertEqual(res.status_int, 200)
-
     def test_quota_copy_from_bad_src(self):
-        headers = [('x-account-bytes-used', '0'),
-                   ('x-account-meta-quota-bytes', '1000')]
-        self.app.headers = headers
+        self.headers[:] = [('x-account-bytes-used', '0'),
+                           ('x-account-meta-quota-bytes', '1000')]
         cache = FakeCache(None)
         req = Request.blank('/v1/a/c/o',
                             environ={'REQUEST_METHOD': 'PUT',
@@ -493,5 +571,11 @@ class AccountQuotaCopyingTestCases(unittest.TestCase):
         res = req.get_response(self.copy_filter)
         self.assertEqual(res.status_int, 412)
 
+        self.headers[:] = [('x-account-bytes-used', '1000'),
+                           ('x-account-meta-quota-bytes', '0')]
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 412)
+
+
 if __name__ == '__main__':
     unittest.main()
