@@ -14,7 +14,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import hashlib
 import json
 import mock
 import shutil
@@ -26,7 +25,7 @@ import unittest
 from swift.common import swob
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.middleware import dlo
-from swift.common.utils import closing_if_possible
+from swift.common.utils import closing_if_possible, md5
 from test.unit.common.middleware.helpers import FakeSwift
 
 
@@ -36,7 +35,7 @@ LIMIT = 'swift.common.constraints.CONTAINER_LISTING_LIMIT'
 def md5hex(s):
     if not isinstance(s, bytes):
         s = s.encode('utf-8')
-    return hashlib.md5(s).hexdigest()
+    return md5(s, usedforsecurity=False).hexdigest()
 
 
 class DloTestCase(unittest.TestCase):
@@ -315,6 +314,7 @@ class TestDloGetManifest(DloTestCase):
         self.assertEqual(status, '200 OK')
         self.assertEqual(body, b'useful stuff here')
         self.assertEqual(self.app.call_count, 1)
+        self.assertFalse(self.app.unread_requests)
 
     def test_get_manifest_passthrough(self):
         # reregister it with the query param
@@ -331,6 +331,17 @@ class TestDloGetManifest(DloTestCase):
         headers = HeaderKeyDict(headers)
         self.assertEqual(headers["Etag"], "manifest-etag")
         self.assertEqual(body, b'manifest-contents')
+        self.assertFalse(self.app.unread_requests)
+
+        # HEAD query param worked, since GET with query param is registered
+        req = swob.Request.blank(
+            '/v1/AUTH_test/mancon/manifest',
+            environ={'REQUEST_METHOD': 'HEAD',
+                     'QUERY_STRING': 'multipart-manifest=get'})
+        status, headers, body = self.call_dlo(req)
+        headers = HeaderKeyDict(headers)
+        self.assertEqual(headers["Etag"], "manifest-etag")
+        self.assertEqual(body, b'')
 
     def test_error_passthrough(self):
         self.app.register(
@@ -341,6 +352,20 @@ class TestDloGetManifest(DloTestCase):
         status, headers, body = self.call_dlo(req)
         self.assertEqual(status, '404 Not Found')
 
+        # ... and multipart-manifest=get also returns registered 404 response
+        req = swob.Request.blank('/v1/AUTH_test/gone/404ed',
+                                 method='GET',
+                                 params={'multipart-manifest': 'get'})
+        status, headers, body = self.call_dlo(req)
+        self.assertEqual(status, '404 Not Found')
+
+        # HEAD with same params find same registered GET
+        req = swob.Request.blank('/v1/AUTH_test/gone/404ed',
+                                 method='HEAD',
+                                 params={'multipart-manifest': 'get'})
+        status, headers, body = self.call_dlo(req)
+        self.assertEqual(status, '404 Not Found')
+
     def test_get_range(self):
         req = swob.Request.blank('/v1/AUTH_test/mancon/manifest',
                                  environ={'REQUEST_METHOD': 'GET'},
@@ -354,6 +379,29 @@ class TestDloGetManifest(DloTestCase):
             md5hex("aaaaa") + md5hex("bbbbb") + md5hex("ccccc") +
             md5hex("ddddd") + md5hex("eeeee"))
         self.assertEqual(headers.get("Etag"), expected_etag)
+        self.assertEqual(self.app.unread_requests, {})
+
+    def test_get_big_manifest(self):
+        self.app.register(
+            'GET', '/v1/AUTH_test/mancon/big-manifest',
+            swob.HTTPOk, {'Content-Length': '17000', 'Etag': 'manifest-etag',
+                          'X-Object-Manifest': 'c/seg'},
+            b'manifest-contents' * 1000)
+        req = swob.Request.blank('/v1/AUTH_test/mancon/big-manifest',
+                                 environ={'REQUEST_METHOD': 'GET'})
+        status, headers, body = self.call_dlo(req)
+        headers = HeaderKeyDict(headers)
+        self.assertEqual(status, "200 OK")
+        self.assertEqual(headers["Content-Length"], "25")
+        self.assertEqual(body, b'aaaaabbbbbcccccdddddeeeee')
+        expected_etag = '"%s"' % md5hex(
+            md5hex("aaaaa") + md5hex("bbbbb") + md5hex("ccccc") +
+            md5hex("ddddd") + md5hex("eeeee"))
+        self.assertEqual(headers.get("Etag"), expected_etag)
+        self.assertEqual(self.app.unread_requests, {
+            # Since we don't know how big this will be, we just disconnect
+            ('GET', '/v1/AUTH_test/mancon/big-manifest'): 1,
+        })
 
     def test_get_range_on_segment_boundaries(self):
         req = swob.Request.blank('/v1/AUTH_test/mancon/manifest',
@@ -558,7 +606,23 @@ class TestDloGetManifest(DloTestCase):
             self.assertFalse('If-Modified-Since' in hdrs)
             self.assertFalse('If-Unmodified-Since' in hdrs)
 
-    def test_error_fetching_first_segment(self):
+    def test_server_error_fetching_first_segment(self):
+        self.app.register(
+            'GET', '/v1/AUTH_test/c/seg_01',
+            swob.HTTPServiceUnavailable, {}, None)
+
+        req = swob.Request.blank('/v1/AUTH_test/mancon/manifest',
+                                 environ={'REQUEST_METHOD': 'GET'})
+        status, headers, body = self.call_dlo(req)
+        self.assertEqual(status, "503 Service Unavailable")
+        self.assertEqual(self.app.unread_requests, {})
+        self.assertEqual(self.dlo.logger.get_lines_for_level('error'), [
+            'While processing manifest /v1/AUTH_test/mancon/manifest, '
+            'got 503 (<html><h1>Service Unavailable</h1><p>The server is '
+            'curren...) while retrieving /v1/AUTH_test/c/seg_01',
+        ])
+
+    def test_client_error_fetching_first_segment(self):
         self.app.register(
             'GET', '/v1/AUTH_test/c/seg_01',
             swob.HTTPForbidden, {}, None)
@@ -567,9 +631,11 @@ class TestDloGetManifest(DloTestCase):
                                  environ={'REQUEST_METHOD': 'GET'})
         status, headers, body = self.call_dlo(req)
         self.assertEqual(status, "409 Conflict")
+        self.assertEqual(self.app.unread_requests, {})
         self.assertEqual(self.dlo.logger.get_lines_for_level('error'), [
             'While processing manifest /v1/AUTH_test/mancon/manifest, '
-            'got 403 while retrieving /v1/AUTH_test/c/seg_01',
+            'got 403 (<html><h1>Forbidden</h1><p>Access was denied to this '
+            'reso...) while retrieving /v1/AUTH_test/c/seg_01',
         ])
 
     def test_error_fetching_second_segment(self):
@@ -585,9 +651,11 @@ class TestDloGetManifest(DloTestCase):
         self.assertEqual(status, "200 OK")
         # first segment made it out
         self.assertEqual(body, b'aaaaa')
+        self.assertEqual(self.app.unread_requests, {})
         self.assertEqual(self.dlo.logger.get_lines_for_level('error'), [
             'While processing manifest /v1/AUTH_test/mancon/manifest, '
-            'got 403 while retrieving /v1/AUTH_test/c/seg_02',
+            'got 403 (<html><h1>Forbidden</h1><p>Access was denied to this '
+            'reso...) while retrieving /v1/AUTH_test/c/seg_02',
         ])
 
     def test_error_listing_container_first_listing_request(self):
@@ -634,7 +702,7 @@ class TestDloGetManifest(DloTestCase):
         self.app.register(
             'GET', '/v1/AUTH_test/c/seg_02',
             swob.HTTPOk, {'Content-Length': '5', 'Etag': md5hex("bbbbb")},
-            'bbWRONGbb')
+            'WRONG')
 
         req = swob.Request.blank('/v1/AUTH_test/mancon/manifest',
                                  environ={'REQUEST_METHOD': 'GET'})
@@ -642,8 +710,35 @@ class TestDloGetManifest(DloTestCase):
         headers = HeaderKeyDict(headers)
 
         self.assertEqual(status, "200 OK")
+        self.assertEqual(headers['Content-Length'], "25")
         # stop after error
-        self.assertEqual(body, b"aaaaabbWRONGbb")
+        self.assertEqual(body, b"aaaaaWRONG")
+        log_lines = self.dlo.logger.get_lines_for_level('error')
+        self.assertEqual(len(log_lines), 1,
+                         'Expected one log line, got %r' % log_lines)
+        self.assertEqual(log_lines[0][:21], 'Bad MD5 checksum for ')
+
+    def test_mismatched_length_fetching_second_segment(self):
+        self.app.register(
+            'GET', '/v1/AUTH_test/c/seg_02',
+            swob.HTTPOk, {'Content-Length': '5', 'Etag': md5hex("bbbb")},
+            # Use a list so we can get a discrepency between content-length and
+            # number of bytes in the app_iter
+            [b'b' * 4])
+
+        req = swob.Request.blank('/v1/AUTH_test/mancon/manifest',
+                                 environ={'REQUEST_METHOD': 'GET'})
+        status, headers, body = self.call_dlo(req)
+        headers = HeaderKeyDict(headers)
+
+        self.assertEqual(status, "200 OK")
+        self.assertEqual(headers['Content-Length'], "25")
+        # stop after error
+        self.assertEqual(body, b"aaaaabbbb")
+        log_lines = self.dlo.logger.get_lines_for_level('error')
+        self.assertEqual(len(log_lines), 1,
+                         'Expected one log line, got %r' % log_lines)
+        self.assertEqual(log_lines[0][:24], 'Bad response length for ')
 
     def test_etag_comparison_ignores_quotes(self):
         # a little future-proofing here in case we ever fix this in swob
@@ -666,7 +761,8 @@ class TestDloGetManifest(DloTestCase):
         status, headers, body = self.call_dlo(req)
         headers = HeaderKeyDict(headers)
         self.assertEqual(headers["Etag"],
-                         '"' + hashlib.md5(b"abcdef").hexdigest() + '"')
+                         '"' + md5(b"abcdef",
+                                   usedforsecurity=False).hexdigest() + '"')
 
     def test_object_prefix_quoting(self):
         self.app.register(
