@@ -36,7 +36,7 @@ from test.unit.common.middleware.helpers import FakeAppThatExcepts
 class FakeApp(object):
 
     def __init__(self, body=None, response_str='200 OK', policy_idx='0',
-                 chunked=False):
+                 chunked=False, environ_updates=None):
         if body is None:
             body = [b'FAKE APP']
         elif isinstance(body, six.binary_type):
@@ -46,6 +46,7 @@ class FakeApp(object):
         self.response_str = response_str
         self.policy_idx = policy_idx
         self.chunked = chunked
+        self.environ_updates = environ_updates or {}
 
     def __call__(self, env, start_response):
         try:
@@ -65,10 +66,11 @@ class FakeApp(object):
         if is_container_or_object_req and self.policy_idx is not None:
             headers.append(('X-Backend-Storage-Policy-Index',
                            str(self.policy_idx)))
-
         start_response(self.response_str, headers)
         while env['wsgi.input'].read(5):
             pass
+        # N.B. mw can set this anytime before the resp is finished
+        env.update(self.environ_updates)
         return self.body
 
 
@@ -123,7 +125,7 @@ class TestProxyLogging(unittest.TestCase):
             return info_calls[0][0][0].split(' ')
 
     def assertTiming(self, exp_metric, app, exp_timing=None):
-        timing_calls = app.access_logger.log_dict['timing']
+        timing_calls = app.access_logger.statsd_client.calls['timing']
         found = False
         for timing_call in timing_calls:
             self.assertEqual({}, timing_call[1])
@@ -138,12 +140,13 @@ class TestProxyLogging(unittest.TestCase):
                 exp_metric, timing_calls))
 
     def assertNotTiming(self, not_exp_metric, app):
-        timing_calls = app.access_logger.log_dict['timing']
+        timing_calls = app.access_logger.statsd_client.calls['timing']
         for timing_call in timing_calls:
             self.assertNotEqual(not_exp_metric, timing_call[0][0])
 
     def assertUpdateStats(self, exp_metrics_and_values, app):
-        update_stats_calls = sorted(app.access_logger.log_dict['update_stats'])
+        update_stats_calls = sorted(
+            app.access_logger.statsd_client.calls['update_stats'])
         got_metrics_values_and_kwargs = [(usc[0][0], usc[0][1], usc[1])
                                          for usc in update_stats_calls]
         exp_metrics_values_and_kwargs = [(emv[0], emv[1], {})
@@ -239,7 +242,7 @@ class TestProxyLogging(unittest.TestCase):
                     self.assertUpdateStats([('%s.GET.321.xfer' % exp_type,
                                              4 + 7),
                                             ('object.policy.0.GET.321.xfer',
-                                            4 + 7)],
+                                             4 + 7)],
                                            app)
                 else:
                     self.assertUpdateStats([('%s.GET.321.xfer' % exp_type,
@@ -335,8 +338,8 @@ class TestProxyLogging(unittest.TestCase):
                     self.assertNotTiming(
                         '%s.PUT.314.first-byte.timing' % exp_type, app)
                     # No results returned for the non-existent policy
-                    self.assertUpdateStats([('object.PUT.314.xfer', 6 + 8)],
-                                           app)
+                    self.assertUpdateStats(
+                        [('object.PUT.314.xfer', 6 + 8)], app)
 
     def test_log_request_stat_method_filtering_default(self):
         method_map = {
@@ -423,9 +426,10 @@ class TestProxyLogging(unittest.TestCase):
                 'template which can be edited in config: '
                 '{protocol} {path} {method} '
                 '{path.anonymized} {container.anonymized} '
-                '{request_time} {start_time.datetime} {end_time} {ttfb}')})
+                '{request_time} {start_time.datetime} {end_time} {ttfb} '
+                '{domain}')})
         app.access_logger = debug_logger()
-        req = Request.blank('/', environ={'REQUEST_METHOD': 'GET'})
+        req = Request.blank('/', headers={'Host': 'example.com'})
         with mock.patch('time.time',
                         mock.MagicMock(
                             side_effect=[10000000.0, 10000000.5, 10000001.0])):
@@ -443,6 +447,7 @@ class TestProxyLogging(unittest.TestCase):
         self.assertEqual(log_parts[13], '26/Apr/1970/17/46/40')
         self.assertEqual(log_parts[14], '10000001.000000000')
         self.assertEqual(log_parts[15], '0.5')
+        self.assertEqual(log_parts[16], 'example.com')
         self.assertEqual(resp_body, b'FAKE APP')
 
     def test_log_msg_template_s3api(self):
@@ -504,7 +509,8 @@ class TestProxyLogging(unittest.TestCase):
         self.assertEqual(log_parts[6], '200')
         self.assertEqual(resp_body, b'somechunksof data')
         self.assertEqual(log_parts[11], str(len(resp_body)))
-        self.assertUpdateStats([('SOS.GET.200.xfer', len(resp_body))], app)
+        self.assertUpdateStats([('SOS.GET.200.xfer', len(resp_body))],
+                               app)
 
     def test_log_headers(self):
         for conf_key in ['access_log_headers', 'log_headers']:
@@ -787,6 +793,193 @@ class TestProxyLogging(unittest.TestCase):
         self.assertEqual(log_parts[6], '499')
         self.assertEqual(log_parts[10], '-')  # read length
 
+    def test_environ_has_proxy_logging_status(self):
+        conf = {'log_msg_template':
+                '{method} {path} {status_int} {wire_status_int}'}
+
+        def do_test(environ_updates):
+            fake_app = FakeApp(body=[b'Slow Down'],
+                               response_str='503 Slow Down',
+                               environ_updates=environ_updates)
+            app = proxy_logging.ProxyLoggingMiddleware(fake_app, conf)
+            app.access_logger = debug_logger()
+            req = Request.blank('/v1/a/c')
+            captured_start_resp = mock.MagicMock()
+            try:
+                resp = app(req.environ, captured_start_resp)
+                b''.join(resp)  # read body
+            except IOError:
+                pass
+            captured_start_resp.assert_called_once_with(
+                '503 Slow Down', mock.ANY, None)
+            return self._log_parts(app)
+
+        # control case, logged status == wire status
+        environ_updates = {}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '503', '503'])
+
+        # logged status is forced to other value
+        environ_updates = {'swift.proxy_logging_status': 429}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '429', '503'])
+
+        environ_updates = {'swift.proxy_logging_status': '429'}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '429', '503'])
+
+        environ_updates = {'swift.proxy_logging_status': None}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '-', '503'])
+
+        # middleware should use an int like the docs tell them too, but we
+        # won't like ... "blow up" or anything
+        environ_updates = {'swift.proxy_logging_status': ''}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '-', '503'])
+
+        environ_updates = {'swift.proxy_logging_status': True}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', 'True', '503'])
+
+        environ_updates = {'swift.proxy_logging_status': False}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '-', '503'])
+
+        environ_updates = {'swift.proxy_logging_status': 'parsing ok'}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', 'parsing%20ok', '503'])
+
+    def test_body_iter_updates_environ_proxy_logging_status(self):
+        conf = {'log_msg_template':
+                '{method} {path} {status_int} {wire_status_int}'}
+
+        def do_test(req, body_iter, updated_status):
+            fake_app = FakeApp(body=body_iter,
+                               response_str='205 Weird')
+            app = proxy_logging.ProxyLoggingMiddleware(fake_app, conf)
+            app.access_logger = debug_logger()
+            captured_start_resp = mock.MagicMock()
+            try:
+                resp = app(req.environ, captured_start_resp)
+                b''.join(resp)  # read body
+            except IOError:
+                pass
+            captured_start_resp.assert_called_once_with(
+                '205 Weird', mock.ANY, None)
+            self.assertEqual(self._log_parts(app),
+                             ['GET', '/v1/a/c', updated_status, '205'])
+
+        # sanity
+        req = Request.blank('/v1/a/c')
+        do_test(req, [b'normal', b'chunks'], '205')
+
+        def update_in_middle_chunk_gen():
+            yield b'foo'
+            yield b'bar'
+            req.environ['swift.proxy_logging_status'] = 209
+            yield b'baz'
+
+        req = Request.blank('/v1/a/c')
+        do_test(req, update_in_middle_chunk_gen(), '209')
+
+        def update_in_finally_chunk_gen():
+            try:
+                for i in range(3):
+                    yield ('foo%s' % i).encode()
+            finally:
+                req.environ['swift.proxy_logging_status'] = 210
+
+        req = Request.blank('/v1/a/c')
+        do_test(req, update_in_finally_chunk_gen(), '210')
+
+    def test_environ_has_proxy_logging_status_unread_body(self):
+        conf = {'log_msg_template':
+                '{method} {path} {status_int} {wire_status_int}'}
+
+        def do_test(environ_updates):
+            fake_app = FakeApp(body=[b'Slow Down'],
+                               response_str='503 Slow Down',
+                               environ_updates=environ_updates)
+            app = proxy_logging.ProxyLoggingMiddleware(fake_app, conf)
+            app.access_logger = debug_logger()
+            req = Request.blank('/v1/a/c')
+            captured_start_resp = mock.MagicMock()
+            resp = app(req.environ, captured_start_resp)
+            # read first chunk
+            next(resp)
+            resp.close()  # raise a GeneratorExit in middleware app_iter loop
+            captured_start_resp.assert_called_once_with(
+                '503 Slow Down', mock.ANY, None)
+            return self._log_parts(app)
+
+        # control case, logged status is 499
+        environ_updates = {}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '499', '503'])
+
+        # logged status is forced to 499 despite swift.proxy_logging_status
+        environ_updates = {'swift.proxy_logging_status': '429'}
+        self.assertEqual(do_test(environ_updates),
+                         ['GET', '/v1/a/c', '499', '503'])
+
+    def test_environ_has_proxy_logging_status_and_app_explodes(self):
+        # verify exception overrides proxy_logging_status
+        conf = {'log_msg_template':
+                '{method} {path} {status_int} {wire_status_int}'}
+
+        class ExplodingFakeApp(object):
+
+            def __call__(self, env, start_response):
+                # this is going to be so great!
+                env['swift.proxy_logging_status'] = '456'
+                start_response('568 Bespoke', [('X-Special', 'fun')])
+                raise Exception('oops!')
+
+        fake_app = ExplodingFakeApp()
+        app = proxy_logging.ProxyLoggingMiddleware(fake_app, conf)
+        app.access_logger = debug_logger()
+        req = Request.blank('/v1/a/c')
+        captured_start_resp = mock.MagicMock()
+        with self.assertRaises(Exception) as cm:
+            app(req.environ, captured_start_resp)
+        captured_start_resp.assert_not_called()
+        self.assertEqual('oops!', str(cm.exception))
+        self.assertEqual(self._log_parts(app),
+                         ['GET', '/v1/a/c', '500', '500'])
+
+    def test_environ_has_proxy_logging_status_and_body_explodes(self):
+        # verify exception overrides proxy_logging_status
+        conf = {'log_msg_template':
+                '{method} {path} {status_int} {wire_status_int}'}
+
+        def exploding_body():
+            yield 'some'
+            yield 'stuff'
+            raise Exception('oops!')
+
+        class ExplodingFakeApp(object):
+
+            def __call__(self, env, start_response):
+                # this is going to be so great!
+                env['swift.proxy_logging_status'] = '456'
+                start_response('568 Bespoke', [('X-Special', 'fun')])
+                return exploding_body()
+
+        fake_app = ExplodingFakeApp()
+        app = proxy_logging.ProxyLoggingMiddleware(fake_app, conf)
+        app.access_logger = debug_logger()
+        req = Request.blank('/v1/a/c')
+        captured_start_resp = mock.MagicMock()
+        app_iter = app(req.environ, captured_start_resp)
+        with self.assertRaises(Exception) as cm:
+            b''.join(app_iter)
+        captured_start_resp.assert_called_once_with(
+            '568 Bespoke', [('X-Special', 'fun')], None)
+        self.assertEqual('oops!', str(cm.exception))
+        self.assertEqual(self._log_parts(app),
+                         ['GET', '/v1/a/c', '500', '568'])
+
     def test_app_exception(self):
         app = proxy_logging.ProxyLoggingMiddleware(
             FakeAppThatExcepts(), {})
