@@ -23,7 +23,7 @@ import six
 from six.moves.urllib.parse import quote, urlparse
 from swift.common.middleware import tempauth as auth
 from swift.common.middleware.acl import format_acl
-from swift.common.swob import Request, Response
+from swift.common.swob import Request, Response, bytes_to_wsgi
 from swift.common.utils import split_path, StatsdClient
 from test.unit import FakeMemcache
 
@@ -627,6 +627,15 @@ class TestAuth(unittest.TestCase):
                                auth.DEFAULT_TOKEN_LIFE - 0.5, delta=0.5)
         self.assertGreater(len(resp.headers['x-auth-token']), 10)
 
+    def test_get_token_memcache_error(self):
+        test_auth = auth.filter_factory({'user_ac_user': 'testing'})(FakeApp())
+        req = self._make_request(
+            '/auth/v1.0',
+            headers={'X-Auth-User': 'ac:user', 'X-Auth-Key': 'testing'})
+        req.environ['swift.cache'] = FakeMemcache(error_on_set=[True])
+        resp = req.get_response(test_auth)
+        self.assertEqual(resp.status_int, 503)
+
     def test_get_token_success_other_auth_prefix(self):
         test_auth = auth.filter_factory({'user_ac_user': 'testing',
                                          'auth_prefix': '/other/'})(FakeApp())
@@ -1000,10 +1009,11 @@ class TestAuth(unittest.TestCase):
         quoted_acct = quote(u'/v1/AUTH_t\u00e9st'.encode('utf8'))
         memcache = FakeMemcache()
 
+        wsgi_user = bytes_to_wsgi(u't\u00e9st:t\u00e9ster'.encode('utf8'))
         req = self._make_request(
             '/auth/v1.0',
-            headers={'X-Auth-User': u't\u00e9st:t\u00e9ster',
-                     'X-Auth-Key': u'p\u00e1ss'})
+            headers={'X-Auth-User': wsgi_user,
+                     'X-Auth-Key': bytes_to_wsgi(u'p\u00e1ss'.encode('utf8'))})
         req.environ['swift.cache'] = memcache
         resp = req.get_response(ath)
         self.assertEqual(resp.status_int, 200)
@@ -1013,8 +1023,8 @@ class TestAuth(unittest.TestCase):
 
         req = self._make_request(
             '/auth/v1.0',
-            headers={'X-Auth-User': u't\u00e9st:t\u00e9ster',
-                     'X-Auth-Key': u'p\u00e1ss'})
+            headers={'X-Auth-User': wsgi_user,
+                     'X-Auth-Key': bytes_to_wsgi(u'p\u00e1ss'.encode('utf8'))})
         req.environ['swift.cache'] = memcache
         resp = req.get_response(ath)
         self.assertEqual(resp.status_int, 200)
