@@ -78,6 +78,7 @@ class TestTempURL(unittest.TestCase):
         self.app = FakeApp()
         self.auth = tempauth.filter_factory({'reseller_prefix': ''})(self.app)
         self.tempurl = tempurl.filter_factory({})(self.auth)
+        self.logger = self.tempurl.logger = debug_logger()
 
     def _make_request(self, path, environ=None, keys=(), container_keys=None,
                       **kwargs):
@@ -157,6 +158,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
+
         sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
         self.assert_valid_sig(expires, path, [key], sig)
 
@@ -171,6 +173,12 @@ class TestTempURL(unittest.TestCase):
             key, hmac_body, hashlib.sha512).digest())
         self.assert_valid_sig(expires, path, [key], b'sha512:' + sig)
 
+        self.assertEqual(self.logger.statsd_client.get_increment_counts(), {
+            'tempurl.digests.sha1': 1,
+            'tempurl.digests.sha256': 2,
+            'tempurl.digests.sha512': 1
+        })
+
     def test_get_valid_key2(self):
         method = 'GET'
         expires = int(time() + 86400)
@@ -178,8 +186,8 @@ class TestTempURL(unittest.TestCase):
         key1 = b'abc123'
         key2 = b'def456'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig1 = hmac.new(key1, hmac_body, hashlib.sha1).hexdigest()
-        sig2 = hmac.new(key2, hmac_body, hashlib.sha1).hexdigest()
+        sig1 = hmac.new(key1, hmac_body, hashlib.sha256).hexdigest()
+        sig2 = hmac.new(key2, hmac_body, hashlib.sha256).hexdigest()
         for sig in (sig1, sig2):
             self.assert_valid_sig(expires, path, [key1, key2], sig)
 
@@ -201,8 +209,8 @@ class TestTempURL(unittest.TestCase):
         key1 = b'me'
         key2 = b'other'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig1 = hmac.new(key1, hmac_body, hashlib.sha1).hexdigest()
-        sig2 = hmac.new(key2, hmac_body, hashlib.sha1).hexdigest()
+        sig1 = hmac.new(key1, hmac_body, hashlib.sha256).hexdigest()
+        sig2 = hmac.new(key2, hmac_body, hashlib.sha256).hexdigest()
         account_keys = []
         for sig in (sig1, sig2):
             self.assert_valid_sig(expires, path, account_keys, sig, environ)
@@ -210,11 +218,13 @@ class TestTempURL(unittest.TestCase):
     def test_signature_trim(self):
         # Insert proxy logging into the pipeline
         p_logging = proxy_logging.filter_factory({})(self.app)
-        self.auth = tempauth.filter_factory({'reseller_prefix': ''})(
-            p_logging)
-        self.tempurl = tempurl.filter_factory({})(self.auth)
+        self.auth = tempauth.filter_factory({
+            'reseller_prefix': ''})(p_logging)
+        self.tempurl = tempurl.filter_factory({
+            'allowed_digests': 'sha1'})(self.auth)
 
-        sig = 'valid_sigs_will_be_exactly_40_characters'
+        # valid sig should be exactly 40 hex chars
+        sig = 'deadbeefdeadbeefdeadbeefdeadbeefdeadbeef'
         expires = int(time() + 1000)
         p_logging.access_logger.logger = debug_logger('fake')
         resp = self._make_request(
@@ -239,7 +249,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(path, keys=[key], environ={
             'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
             'filename=bob%%20%%22killer%%22.txt' % (sig, expires)})
@@ -261,7 +271,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(path, keys=[key], environ={
             'REQUEST_METHOD': 'HEAD',
             'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
@@ -279,7 +289,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(path, keys=[key], environ={
             'REQUEST_METHOD': 'HEAD',
             'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s'
@@ -290,7 +300,7 @@ class TestTempURL(unittest.TestCase):
         get_method = 'GET'
         get_hmac_body = ('%s\n%i\n%s' %
                          (get_method, expires, path)).encode('utf-8')
-        get_sig = hmac.new(key, get_hmac_body, hashlib.sha1).hexdigest()
+        get_sig = hmac.new(key, get_hmac_body, hashlib.sha256).hexdigest()
         get_req = self._make_request(path, keys=[key], environ={
             'REQUEST_METHOD': 'GET',
             'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s'
@@ -306,7 +316,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(path, keys=[key], environ={
             'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
             'filename=bob%%20%%22killer%%22.txt&inline=' % (sig, expires)})
@@ -328,7 +338,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(path, keys=[key], environ={
             'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
             'inline=' % (sig, expires)})
@@ -349,16 +359,40 @@ class TestTempURL(unittest.TestCase):
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' %
                      (method, expires, sig_path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         self.assert_valid_sig(expires, query_path, [key], sig, prefix=prefix)
 
         query_path = query_path[:-1] + 'p3/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' %
                      (method, expires, sig_path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         self.assert_valid_sig(expires, query_path, [key], sig, prefix=prefix)
 
+    def test_get_valid_with_prefix_and_staticweb(self):
+        method = 'GET'
+        expires = int(time() + 86400)
+        prefix = 'p1/p2/'
+        sig_path = 'prefix:/v1/a/c/' + prefix
+        query_path = '/v1/a/c/' + prefix + 'o'
+        key = b'abc'
+        hmac_body = ('%s\n%i\n%s' %
+                     (method, expires, sig_path)).encode('utf-8')
+        sig = hmac.new(key, hmac_body, hashlib.sha512).hexdigest()
+        req = self._make_request(query_path, keys=[key], environ={
+            'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
+            'temp_url_prefix=%s' % (sig, expires, prefix)})
+        self.tempurl.app = FakeApp(iter([('200 Ok', {
+            'X-Backend-Content-Generator': 'staticweb'}, b'123')]))
+        resp = req.get_response(self.tempurl)
+        self.assertEqual(resp.status_int, 200)
+        # This is the key thing: if the response came from staticweb, assume
+        # the client is a browser and doesn't want a download prompt
+        self.assertEqual(resp.headers['content-disposition'], 'inline')
+        self.assertIn('expires', resp.headers)
+        self.assertEqual(req.environ['swift.authorize_override'], True)
+        self.assertEqual(req.environ['REMOTE_USER'], '.wsgi.tempurl')
+
     def test_get_valid_with_prefix_empty(self):
         method = 'GET'
         expires = int(time() + 86400)
@@ -367,7 +401,7 @@ class TestTempURL(unittest.TestCase):
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' %
                      (method, expires, sig_path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         self.assert_valid_sig(expires, query_path, [key], sig, prefix='')
 
     def test_obj_odd_chars(self):
@@ -376,7 +410,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/a\r\nb'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(quote(path), keys=[key], environ={
             'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
                 sig, expires)})
@@ -396,7 +430,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(path, keys=[key], environ={
             'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
                 sig, expires)})
@@ -416,7 +450,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o/'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(path, keys=[key], environ={
             'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
                 sig, expires)})
@@ -436,7 +470,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(path, keys=[key], environ={
             'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
             'filename=/i/want/this/just/as/it/is/' % (sig, expires)})
@@ -457,7 +491,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
@@ -475,7 +509,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'REQUEST_METHOD': 'PUT',
@@ -492,7 +526,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'REQUEST_METHOD': 'PUT',
@@ -526,7 +560,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'REQUEST_METHOD': 'PUT',
@@ -545,7 +579,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
@@ -561,7 +595,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'QUERY_STRING': 'temp_url_expires=%s' % expires})
@@ -576,7 +610,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'QUERY_STRING': 'temp_url_sig=%s' % sig})
@@ -591,7 +625,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
@@ -607,7 +641,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[],
             environ={'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
@@ -623,7 +657,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'REQUEST_METHOD': 'HEAD',
@@ -640,7 +674,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'REQUEST_METHOD': 'HEAD',
@@ -657,7 +691,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'REQUEST_METHOD': 'HEAD',
@@ -674,7 +708,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         # Deliberately fudge expires to show HEADs aren't just automatically
         # allowed.
         expires += 1
@@ -694,7 +728,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'REQUEST_METHOD': 'POST',
@@ -712,7 +746,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'REQUEST_METHOD': 'DELETE',
@@ -729,7 +763,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'REQUEST_METHOD': 'DELETE',
@@ -744,7 +778,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'REQUEST_METHOD': 'UNKNOWN',
@@ -772,7 +806,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
 
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(b'account-key', hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(b'account-key', hmac_body, hashlib.sha256).hexdigest()
         qs = '?temp_url_sig=%s&temp_url_expires=%s' % (sig, expires)
 
         # make request will setup the environ cache for us
@@ -794,7 +828,7 @@ class TestTempURL(unittest.TestCase):
         # the container level; a different container in the same account is
         # out of scope and thus forbidden.
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(b'container-key', hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(b'container-key', hmac_body, hashlib.sha256).hexdigest()
         qs = '?temp_url_sig=%s&temp_url_expires=%s' % (sig, expires)
 
         req = self._make_request(path + qs, **key_kwargs)
@@ -815,7 +849,7 @@ class TestTempURL(unittest.TestCase):
         # account-level tempurls by reusing one of the account's keys on a
         # container.
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(b'shared-key', hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(b'shared-key', hmac_body, hashlib.sha256).hexdigest()
         qs = '?temp_url_sig=%s&temp_url_expires=%s' % (sig, expires)
 
         req = self._make_request(path + qs, **key_kwargs)
@@ -836,7 +870,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path + '2', keys=[key],
             environ={'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
@@ -852,7 +886,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         if sig[-1] != '0':
             sig = sig[:-1] + '0'
         else:
@@ -872,7 +906,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
@@ -936,7 +970,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key + b'2'],
             environ={'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
@@ -954,7 +988,7 @@ class TestTempURL(unittest.TestCase):
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' %
                      (method, expires, sig_path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             query_path, keys=[key],
             environ={'QUERY_STRING':
@@ -972,7 +1006,7 @@ class TestTempURL(unittest.TestCase):
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' %
                      (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'QUERY_STRING':
@@ -993,7 +1027,7 @@ class TestTempURL(unittest.TestCase):
                                ('X-Symlink-Target', 'cont/symlink')]:
                 hmac_body = ('%s\n%i\n%s' %
                              (method, expires, path)).encode('utf-8')
-                sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+                sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
                 req = self._make_request(
                     path, method=method, keys=[key],
                     headers={hdr: value},
@@ -1014,7 +1048,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             headers={'x-remove-this': 'value'},
@@ -1033,7 +1067,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             headers={'x-remove-this-one': 'value1',
@@ -1055,7 +1089,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             headers={'x-conflict-header': 'value'},
@@ -1074,7 +1108,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             headers={'x-conflict-header-test': 'value'},
@@ -1092,7 +1126,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
@@ -1111,7 +1145,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             environ={'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
@@ -1131,7 +1165,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             headers={},
@@ -1153,7 +1187,7 @@ class TestTempURL(unittest.TestCase):
         path = '/v1/a/c/o'
         key = b'abc'
         hmac_body = ('%s\n%i\n%s' % (method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(
             path, keys=[key],
             headers={},
@@ -1188,6 +1222,16 @@ class TestTempURL(unittest.TestCase):
         self.assertEqual(self.tempurl._get_path_parts({
             'REQUEST_METHOD': 'GET', 'PATH_INFO': '/v1/a/c/'}),
             (None, None, None))
+        self.assertEqual(
+            self.tempurl._get_path_parts(
+                {'REQUEST_METHOD': 'GET', 'PATH_INFO': '/v1/a/c/'},
+                allow_container_root=True),
+            ('a', 'c', ''))
+        self.assertEqual(
+            self.tempurl._get_path_parts(
+                {'REQUEST_METHOD': 'GET', 'PATH_INFO': '/v1/a/c/'},
+                allow_container_root=False),
+            (None, None, None))
         self.assertEqual(self.tempurl._get_path_parts({
             'REQUEST_METHOD': 'GET', 'PATH_INFO': '/v1/a/c//////'}),
             (None, None, None))
@@ -1214,71 +1258,88 @@ class TestTempURL(unittest.TestCase):
         s = 'f5d5051bddf5df7e27c628818738334f'
         e_ts = int(time() + 86400)
         e_8601 = strftime(tempurl.EXPIRES_ISO8601_FORMAT, gmtime(e_ts))
-        for e in (e_ts, e_8601):
+        for e in (str(e_ts), e_8601):
             self.assertEqual(
-                self.tempurl._get_temp_url_info(
+                tempurl.get_temp_url_info(
                     {'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
                         s, e)}),
-                (s, e_ts, None, None, None, None))
+                (s, e, None, None, None, None))
             self.assertEqual(
-                self.tempurl._get_temp_url_info(
+                tempurl.get_temp_url_info(
                     {'QUERY_STRING':
                      'temp_url_sig=%s&temp_url_expires=%s&temp_url_prefix=%s'
                      % (s, e, 'prefix')}),
-                (s, e_ts, 'prefix', None, None, None))
+                (s, e, 'prefix', None, None, None))
             self.assertEqual(
-                self.tempurl._get_temp_url_info(
+                tempurl.get_temp_url_info(
                     {'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
                      'filename=bobisyouruncle' % (s, e)}),
-                (s, e_ts, None, 'bobisyouruncle', None, None))
+                (s, e, None, 'bobisyouruncle', None, None))
             self.assertEqual(
-                self.tempurl._get_temp_url_info({}),
+                tempurl.get_temp_url_info({}),
                 (None, None, None, None, None, None))
             self.assertEqual(
-                self.tempurl._get_temp_url_info(
+                tempurl.get_temp_url_info(
                     {'QUERY_STRING': 'temp_url_expires=%s' % e}),
-                (None, e_ts, None, None, None, None))
+                (None, e, None, None, None, None))
             self.assertEqual(
-                self.tempurl._get_temp_url_info(
+                tempurl.get_temp_url_info(
                     {'QUERY_STRING': 'temp_url_sig=%s' % s}),
                 (s, None, None, None, None, None))
             self.assertEqual(
-                self.tempurl._get_temp_url_info(
+                tempurl.get_temp_url_info(
                     {'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=bad' % (
                         s)}),
-                (s, 0, None, None, None, None))
+                (s, 'bad', None, None, None, None))
             self.assertEqual(
-                self.tempurl._get_temp_url_info(
+                tempurl.get_temp_url_info(
                     {'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
                      'inline=' % (s, e)}),
-                (s, e_ts, None, None, True, None))
+                (s, e, None, None, True, None))
             self.assertEqual(
-                self.tempurl._get_temp_url_info(
+                tempurl.get_temp_url_info(
                     {'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
                      'filename=bobisyouruncle&inline=' % (s, e)}),
-                (s, e_ts, None, 'bobisyouruncle', True, None))
+                (s, e, None, 'bobisyouruncle', True, None))
             self.assertEqual(
-                self.tempurl._get_temp_url_info(
+                tempurl.get_temp_url_info(
                     {'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
                      'filename=bobisyouruncle&inline='
                      '&temp_url_ip_range=127.0.0.1' % (s, e)}),
-                (s, e_ts, None, 'bobisyouruncle', True, '127.0.0.1'))
+                (s, e, None, 'bobisyouruncle', True, '127.0.0.1'))
 
         e_ts = int(time() - 1)
         e_8601 = strftime(tempurl.EXPIRES_ISO8601_FORMAT, gmtime(e_ts))
-        for e in (e_ts, e_8601):
+        for e in (str(e_ts), e_8601):
             self.assertEqual(
-                self.tempurl._get_temp_url_info(
+                tempurl.get_temp_url_info(
                     {'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
                         s, e)}),
-                (s, 0, None, None, None, None))
-        # Offsets not supported (yet?).
+                (s, e, None, None, None, None))
         e_8601 = strftime('%Y-%m-%dT%H:%M:%S+0000', gmtime(e_ts))
         self.assertEqual(
-            self.tempurl._get_temp_url_info(
+            tempurl.get_temp_url_info(
                 {'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s' % (
-                    s, e_8601)}),
-            (s, 0, None, None, None, None))
+                    s, e_8601.replace('+', '%2B'))}),
+            (s, e_8601, None, None, None, None))
+
+    def test_normalize_temp_url_expires(self):
+        e_ts = int(time() + 86400)
+        self.assertEqual(e_ts, tempurl.normalize_temp_url_expires(e_ts))
+        self.assertEqual(e_ts, tempurl.normalize_temp_url_expires(str(e_ts)))
+
+        e_8601 = strftime(tempurl.EXPIRES_ISO8601_FORMAT, gmtime(e_ts))
+        self.assertEqual(e_ts, tempurl.normalize_temp_url_expires(e_8601))
+        # Offsets not supported (yet?).
+        e_8601 = strftime('%Y-%m-%dT%H:%M:%S+0000', gmtime(e_ts))
+        self.assertEqual(0, tempurl.normalize_temp_url_expires(e_8601))
+
+        self.assertEqual(None, tempurl.normalize_temp_url_expires(None))
+        self.assertEqual(0, tempurl.normalize_temp_url_expires('bad'))
+        e_ts = int(time() - 1)
+        self.assertEqual(0, tempurl.normalize_temp_url_expires(e_ts))
+        e_8601 = strftime(tempurl.EXPIRES_ISO8601_FORMAT, gmtime(e_ts))
+        self.assertEqual(0, tempurl.normalize_temp_url_expires(e_8601))
 
     def test_get_hmacs(self):
         self.assertEqual(
@@ -1467,7 +1528,7 @@ class TestTempURL(unittest.TestCase):
         ip_range = '127.0.0.0/29'
         hmac_body = ('ip=%s\n%s\n%i\n%s' %
                      (ip_range, method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(path, keys=[key], environ={
             'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
             'temp_url_ip_range=%s' % (sig, expires, ip_range),
@@ -1491,7 +1552,7 @@ class TestTempURL(unittest.TestCase):
         ip = '127.0.0.1'
         hmac_body = ('ip=%s\n%s\n%i\n%s' %
                      (ip, method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(path, keys=[key], environ={
             'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
             'temp_url_ip_range=%s' % (sig, expires, ip),
@@ -1536,7 +1597,7 @@ class TestTempURL(unittest.TestCase):
         ip = '2001:db8::'
         hmac_body = ('ip=%s\n%s\n%i\n%s' %
                      (ip, method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(path, keys=[key], environ={
             'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
             'temp_url_ip_range=%s' % (sig, expires, ip),
@@ -1560,7 +1621,7 @@ class TestTempURL(unittest.TestCase):
         ip_range = '2001:db8::/127'
         hmac_body = ('ip=%s\n%s\n%i\n%s' %
                      (ip_range, method, expires, path)).encode('utf-8')
-        sig = hmac.new(key, hmac_body, hashlib.sha1).hexdigest()
+        sig = hmac.new(key, hmac_body, hashlib.sha256).hexdigest()
         req = self._make_request(path, keys=[key], environ={
             'QUERY_STRING': 'temp_url_sig=%s&temp_url_expires=%s&'
             'temp_url_ip_range=%s' % (sig, expires, ip_range),
@@ -1615,6 +1676,7 @@ class TestSwiftInfo(unittest.TestCase):
         self.assertEqual(set(info['outgoing_allow_headers']),
                          set(('x-object-meta-public-*',)))
         self.assertEqual(info['allowed_digests'], ['sha1', 'sha256', 'sha512'])
+        self.assertEqual(info['deprecated_digests'], ['sha1'])
 
     def test_non_default_methods(self):
         tempurl.filter_factory({
@@ -1623,7 +1685,7 @@ class TestSwiftInfo(unittest.TestCase):
             'incoming_allow_headers': 'x-timestamp x-versions-location',
             'outgoing_remove_headers': 'x-*',
             'outgoing_allow_headers': 'x-object-meta-* content-type',
-            'allowed_digests': 'sha512 md5 not-a-valid-digest',
+            'allowed_digests': 'sha1 sha512 md5 not-a-valid-digest',
         })
         swift_info = registry.get_swift_info()
         self.assertIn('tempurl', swift_info)
@@ -1636,7 +1698,25 @@ class TestSwiftInfo(unittest.TestCase):
         self.assertEqual(set(info['outgoing_remove_headers']), set(('x-*', )))
         self.assertEqual(set(info['outgoing_allow_headers']),
                          set(('x-object-meta-*', 'content-type')))
-        self.assertEqual(info['allowed_digests'], ['sha512'])
+        self.assertEqual(info['allowed_digests'], ['sha1', 'sha512'])
+        self.assertEqual(info['deprecated_digests'], ['sha1'])
+
+    def test_no_deprecated_digests(self):
+        tempurl.filter_factory({'allowed_digests': 'sha256 sha512'})
+        swift_info = registry.get_swift_info()
+        self.assertIn('tempurl', swift_info)
+        info = swift_info['tempurl']
+        self.assertEqual(set(info['methods']),
+                         set(('GET', 'HEAD', 'PUT', 'POST', 'DELETE')))
+        self.assertEqual(set(info['incoming_remove_headers']),
+                         set(('x-timestamp',)))
+        self.assertEqual(set(info['incoming_allow_headers']), set())
+        self.assertEqual(set(info['outgoing_remove_headers']),
+                         set(('x-object-meta-*',)))
+        self.assertEqual(set(info['outgoing_allow_headers']),
+                         set(('x-object-meta-public-*',)))
+        self.assertEqual(info['allowed_digests'], ['sha256', 'sha512'])
+        self.assertNotIn('deprecated_digests', info)
 
     def test_bad_config(self):
         with self.assertRaises(ValueError):
@@ -1645,5 +1725,19 @@ class TestSwiftInfo(unittest.TestCase):
             })
 
 
+class TestTempurlWarning(unittest.TestCase):
+
+    def test_deprecation_warning(self):
+        logger = debug_logger()
+        with mock.patch('swift.common.middleware.tempurl.get_logger',
+                        lambda *a, **kw: logger):
+            tempurl.filter_factory({'allowed_digests': 'sha1'})
+        log_lines = logger.get_lines_for_level('warning')
+        self.assertIn(
+            'The following digest algorithms are configured but deprecated:'
+            ' sha1. Support will be removed in a future release.',
+            log_lines)
+
+
 if __name__ == '__main__':
     unittest.main()
