@@ -22,12 +22,15 @@ import time
 
 from io import BytesIO
 
+import mock
 import six
 from six.moves.urllib.parse import quote
 
 import swift.common.swob as swob
 from swift.common import utils, exceptions
 
+from test.unit.common.middleware.helpers import LeakTrackingIter
+
 
 class TestHeaderEnvironProxy(unittest.TestCase):
     def test_proxy(self):
@@ -832,7 +835,7 @@ class TestRequest(unittest.TestCase):
         req = swob.Request.blank('/hi/there', body='hi')
 
         req.if_unmodified_since = 0
-        self.assertTrue(isinstance(req.if_unmodified_since, datetime.datetime))
+        self.assertIsInstance(req.if_unmodified_since, datetime.datetime)
         if_unmodified_since = req.if_unmodified_since
         req.if_unmodified_since = if_unmodified_since
         self.assertEqual(if_unmodified_since, req.if_unmodified_since)
@@ -1232,16 +1235,100 @@ class TestResponse(unittest.TestCase):
         The Response object's __call__ method should be able to reify a
         Request object from the env it gets passed.
         """
+        tracking = {
+            'closed': 0,
+            'read': 0,
+        }
+
+        def mark_closed(*args):
+            tracking['closed'] += 1
+
+        def mark_read(*args):
+            tracking['read'] += 1
+
         def test_app(environ, start_response):
             start_response('200 OK', [])
-            return [b'hi']
+            body = [b'hi']
+            return LeakTrackingIter(body, mark_closed, mark_read, None)
         req = swob.Request.blank('/')
         req.method = 'HEAD'
         status, headers, app_iter = req.call_application(test_app)
         resp = swob.Response(status=status, headers=dict(headers),
                              app_iter=app_iter)
         output_iter = resp(req.environ, lambda *_: None)
-        self.assertEqual(list(output_iter), [b''])
+        with utils.closing_if_possible(output_iter):
+            body = b''.join(output_iter)
+        self.assertEqual(body, b'')
+        self.assertEqual(tracking, {
+            'closed': 1,
+            'read': 1,
+        })
+
+    def test_swob_drains_small_HEAD_resp_iter(self):
+        tracking = {
+            'closed': 0,
+            'read': 0,
+        }
+
+        def mark_closed(*args):
+            tracking['closed'] += 1
+
+        def mark_read(*args):
+            tracking['read'] += 1
+
+        def test_app(environ, start_response):
+            start_response('200 OK', [])
+            body = [b'hello', b'world']
+            return LeakTrackingIter(body, mark_closed, mark_read, None)
+
+        req = swob.Request.blank('/', method='HEAD')
+        status, headers, app_iter = req.call_application(test_app)
+        resp = swob.Response(status=status, headers=dict(headers),
+                             app_iter=app_iter)
+        # sanity, swob drains small HEAD responses
+        output_iter = resp(req.environ, lambda *_: None)
+        with utils.closing_if_possible(output_iter):
+            # regardless what the app returns swob's HEAD response is empty
+            body = b''.join(output_iter)
+        self.assertEqual(body, b'')
+        self.assertEqual(tracking, {
+            'closed': 1,
+            'read': 1,
+        })
+
+    def test_swob_closes_large_HEAD_resp_iter(self):
+        tracking = {
+            'closed': 0,
+            'read': 0,
+        }
+
+        def mark_closed(*args):
+            tracking['closed'] += 1
+
+        def mark_read(*args):
+            tracking['read'] += 1
+
+        def test_app(environ, start_response):
+            start_response('200 OK', [])
+            body = [b'hello', b'world']
+            return LeakTrackingIter(body, mark_closed, mark_read, None)
+
+        req = swob.Request.blank('/', method='HEAD')
+        status, headers, app_iter = req.call_application(test_app)
+        resp = swob.Response(status=status, headers=dict(headers),
+                             app_iter=app_iter)
+        # N.B. if we call next a third time (i.e. len(helloworld) < read_limit)
+        # then leak tracker will notice StopIteration and count it drained.
+        with mock.patch.object(utils, 'DEFAULT_DRAIN_LIMIT', 10):
+            output_iter = resp(req.environ, lambda *_: None)
+        with utils.closing_if_possible(output_iter):
+            # regardless what the app returns swob's HEAD response is empty
+            body = b''.join(output_iter)
+        self.assertEqual(body, b'')
+        self.assertEqual(tracking, {
+            'closed': 1,
+            'read': 0,
+        })
 
     def test_call_preserves_closeability(self):
         def test_app(environ, start_response):
