@@ -43,7 +43,7 @@ from swift.common.storage_policy import POLICIES
 from test import listen_zero
 from test.debug_logger import debug_logger
 from test.unit import (
-    temptree, with_tempdir, write_fake_ring, patch_policies)
+    temptree, with_tempdir, write_fake_ring, patch_policies, ConfigAssertMixin)
 
 from paste.deploy import loadwsgi
 
@@ -60,7 +60,7 @@ def _fake_rings(tmpdir):
 
 
 @patch_policies
-class TestWSGI(unittest.TestCase):
+class TestWSGI(unittest.TestCase, ConfigAssertMixin):
     """Tests for swift.common.wsgi"""
 
     def test_init_request_processor(self):
@@ -133,14 +133,39 @@ class TestWSGI(unittest.TestCase):
     def test_loadapp_from_file(self, tempdir):
         conf_path = os.path.join(tempdir, 'object-server.conf')
         conf_body = """
+        [DEFAULT]
+        CONN_timeout = 10
+        client_timeout = 1
         [app:main]
         use = egg:swift#object
+        conn_timeout = 5
+        client_timeout = 2
+        CLIENT_TIMEOUT = 3
         """
         contents = dedent(conf_body)
         with open(conf_path, 'w') as f:
             f.write(contents)
         app = wsgi.loadapp(conf_path)
         self.assertIsInstance(app, obj_server.ObjectController)
+        # N.B. paste config loading from *file* is already case-sensitive,
+        # so, CLIENT_TIMEOUT/client_timeout are unique options
+        self.assertEqual(1, app.client_timeout)
+        self.assertEqual(5, app.conn_timeout)
+
+    @with_tempdir
+    def test_loadapp_from_file_with_duplicate_var(self, tempdir):
+        conf_path = os.path.join(tempdir, 'object-server.conf')
+        conf_body = """
+        [app:main]
+        use = egg:swift#object
+        client_timeout = 2
+        client_timeout = 3
+        """
+        contents = dedent(conf_body)
+        with open(conf_path, 'w') as f:
+            f.write(contents)
+        app_config = lambda: wsgi.loadapp(conf_path)
+        self.assertDuplicateOption(app_config, 'client_timeout', 3.0)
 
     @with_tempdir
     def test_loadapp_from_file_with_global_conf(self, tempdir):
@@ -204,11 +229,89 @@ class TestWSGI(unittest.TestCase):
 
     def test_loadapp_from_string(self):
         conf_body = """
+        [DEFAULT]
+        CONN_timeout = 10
+        client_timeout = 1
         [app:main]
         use = egg:swift#object
+        conn_timeout = 5
+        client_timeout = 2
         """
         app = wsgi.loadapp(wsgi.ConfigString(conf_body))
-        self.assertTrue(isinstance(app, obj_server.ObjectController))
+        self.assertIsInstance(app, obj_server.ObjectController)
+        self.assertEqual(1, app.client_timeout)
+        self.assertEqual(5, app.conn_timeout)
+
+    @with_tempdir
+    def test_loadapp_from_dir(self, tempdir):
+        conf_files = {
+            'pipeline': """
+            [pipeline:main]
+            pipeline = tempauth proxy-server
+            """,
+            'tempauth': """
+            [DEFAULT]
+            swift_dir = %s
+            random_VAR = foo
+            [filter:tempauth]
+            use = egg:swift#tempauth
+            random_var = bar
+            """ % tempdir,
+            'proxy': """
+            [DEFAULT]
+            conn_timeout = 5
+            client_timeout = 1
+            [app:proxy-server]
+            use = egg:swift#proxy
+            CONN_timeout = 10
+            client_timeout = 2
+            """,
+        }
+        _fake_rings(tempdir)
+        for filename, conf_body in conf_files.items():
+            path = os.path.join(tempdir, filename + '.conf')
+            with open(path, 'wt') as fd:
+                fd.write(dedent(conf_body))
+        app = wsgi.loadapp(tempdir)
+        # DEFAULT takes priority (!?)
+        self.assertEqual(5, app._pipeline_final_app.conn_timeout)
+        self.assertEqual(1, app._pipeline_final_app.client_timeout)
+        self.assertEqual('foo', app.app.app.app.conf['random_VAR'])
+        self.assertEqual('bar', app.app.app.app.conf['random_var'])
+
+    @with_tempdir
+    def test_loadapp_from_dir_with_duplicate_var(self, tempdir):
+        conf_files = {
+            'pipeline': """
+            [pipeline:main]
+            pipeline = tempauth proxy-server
+            """,
+            'tempauth': """
+            [DEFAULT]
+            swift_dir = %s
+            random_VAR = foo
+            [filter:tempauth]
+            use = egg:swift#tempauth
+            random_var = bar
+            """ % tempdir,
+            'proxy': """
+            [app:proxy-server]
+            use = egg:swift#proxy
+            client_timeout = 2
+            CLIENT_TIMEOUT = 1
+            conn_timeout = 3
+            conn_timeout = 4
+            """,
+        }
+        _fake_rings(tempdir)
+        for filename, conf_body in conf_files.items():
+            path = os.path.join(tempdir, filename + '.conf')
+            with open(path, 'wt') as fd:
+                fd.write(dedent(conf_body))
+        app_config = lambda: wsgi.loadapp(tempdir)
+        # N.B. our paste conf.d parsing re-uses readconf,
+        # so, CLIENT_TIMEOUT/client_timeout are unique options
+        self.assertDuplicateOption(app_config, 'conn_timeout', 4.0)
 
     @with_tempdir
     def test_load_app_config(self, tempdir):
@@ -329,8 +432,8 @@ class TestWSGI(unittest.TestCase):
                     conf_dir, 'proxy-server')
         # verify pipeline is catch_errors -> proxy-server
         expected = swift.common.middleware.catch_errors.CatchErrorMiddleware
-        self.assertTrue(isinstance(app, expected))
-        self.assertTrue(isinstance(app.app, swift.proxy.server.Application))
+        self.assertIsInstance(app, expected)
+        self.assertIsInstance(app.app, swift.proxy.server.Application)
         # config settings applied to app instance
         self.assertEqual(0.2, app.app.conn_timeout)
         # appconfig returns values from 'proxy-server' section
@@ -392,7 +495,7 @@ class TestWSGI(unittest.TestCase):
             # test
             sock = wsgi.get_socket(conf)
             # assert
-            self.assertTrue(isinstance(sock, MockSocket))
+            self.assertIsInstance(sock, MockSocket)
             expected_socket_opts = {
                 socket.SOL_SOCKET: {
                     socket.SO_KEEPALIVE: 1,
@@ -496,6 +599,7 @@ class TestWSGI(unittest.TestCase):
         config = """
         [DEFAULT]
         client_timeout = 30
+        keepalive_timeout = 10
         max_clients = 1000
         swift_dir = TEMPDIR
 
@@ -529,12 +633,13 @@ class TestWSGI(unittest.TestCase):
         args, kwargs = _wsgi.server.call_args
         server_sock, server_app, server_logger = args
         self.assertEqual(sock, server_sock)
-        self.assertTrue(isinstance(server_app, swift.proxy.server.Application))
+        self.assertIsInstance(server_app, swift.proxy.server.Application)
         self.assertEqual(20, server_app.client_timeout)
-        self.assertTrue(isinstance(server_logger, wsgi.NullLogger))
+        self.assertIsInstance(server_logger, wsgi.NullLogger)
         self.assertTrue('custom_pool' in kwargs)
         self.assertEqual(1000, kwargs['custom_pool'].size)
         self.assertEqual(30, kwargs['socket_timeout'])
+        self.assertEqual(10, kwargs['keepalive'])
 
         proto_class = kwargs['protocol']
         self.assertEqual(proto_class, wsgi.SwiftHttpProtocol)
@@ -579,12 +684,13 @@ class TestWSGI(unittest.TestCase):
         args, kwargs = _wsgi.server.call_args
         server_sock, server_app, server_logger = args
         self.assertEqual(sock, server_sock)
-        self.assertTrue(isinstance(server_app, swift.proxy.server.Application))
+        self.assertIsInstance(server_app, swift.proxy.server.Application)
         self.assertEqual(2.5, server_app.client_timeout)
-        self.assertTrue(isinstance(server_logger, wsgi.NullLogger))
+        self.assertIsInstance(server_logger, wsgi.NullLogger)
         self.assertTrue('custom_pool' in kwargs)
         self.assertEqual(10, kwargs['custom_pool'].size)
         self.assertEqual(2.5, kwargs['socket_timeout'])
+        self.assertNotIn('keepalive', kwargs)  # eventlet defaults to True
 
         proto_class = kwargs['protocol']
         self.assertEqual(proto_class, wsgi.SwiftHttpProxiedProtocol)
@@ -594,6 +700,7 @@ class TestWSGI(unittest.TestCase):
         config = """
         [DEFAULT]
         swift_dir = TEMPDIR
+        keepalive_timeout = 0
 
         [pipeline:main]
         pipeline = proxy-server
@@ -623,6 +730,7 @@ class TestWSGI(unittest.TestCase):
         self.assertTrue('protocol' in kwargs)
         self.assertEqual('HTTP/1.0',
                          kwargs['protocol'].default_request_version)
+        self.assertIs(False, kwargs['keepalive'])
 
     def test_run_server_conf_dir(self):
         config_dir = {
@@ -664,8 +772,8 @@ class TestWSGI(unittest.TestCase):
         args, kwargs = _wsgi.server.call_args
         server_sock, server_app, server_logger = args
         self.assertEqual(sock, server_sock)
-        self.assertTrue(isinstance(server_app, swift.proxy.server.Application))
-        self.assertTrue(isinstance(server_logger, wsgi.NullLogger))
+        self.assertIsInstance(server_app, swift.proxy.server.Application)
+        self.assertIsInstance(server_logger, wsgi.NullLogger)
         self.assertTrue('custom_pool' in kwargs)
         self.assertEqual(30, kwargs['socket_timeout'])
         self.assertTrue('protocol' in kwargs)
@@ -714,7 +822,7 @@ class TestWSGI(unittest.TestCase):
         args, kwargs = mock_server.call_args
         server_sock, server_app, server_logger = args
         self.assertEqual(sock, server_sock)
-        self.assertTrue(isinstance(server_app, swift.proxy.server.Application))
+        self.assertIsInstance(server_app, swift.proxy.server.Application)
         self.assertEqual(20, server_app.client_timeout)
         self.assertIsNone(server_logger)
         self.assertTrue('custom_pool' in kwargs)
@@ -867,6 +975,8 @@ class TestWSGI(unittest.TestCase):
                 mock.patch.object(wsgi, 'loadapp', _loadapp), \
                 mock.patch.object(wsgi, 'capture_stdio'), \
                 mock.patch.object(wsgi, 'run_server', _run_server), \
+                mock.patch(
+                    'swift.common.wsgi.systemd_notify') as mock_notify, \
                 mock.patch('swift.common.utils.eventlet') as _utils_evt:
             wsgi.run_wsgi('conf_file', 'app_section',
                           global_conf_callback=_global_conf_callback)
@@ -877,9 +987,53 @@ class TestWSGI(unittest.TestCase):
                                                            socket=True,
                                                            select=True,
                                                            thread=True)
+        self.assertEqual(mock_notify.mock_calls, [
+            mock.call('logger', "STOPPING=1"),
+        ])
 
     def test_run_server_success(self):
-        calls = defaultdict(lambda: 0)
+        calls = defaultdict(int)
+
+        def _initrp(conf_file, app_section, *args, **kwargs):
+            calls['_initrp'] += 1
+            return (
+                {'__file__': 'test', 'workers': 0, 'bind_port': 12345},
+                'logger',
+                'log_name')
+
+        def _loadapp(uri, name=None, **kwargs):
+            calls['_loadapp'] += 1
+
+        logging.logThreads = 1  # reset to default
+        with mock.patch.object(wsgi, '_initrp', _initrp), \
+                mock.patch.object(wsgi, 'get_socket'), \
+                mock.patch.object(wsgi, 'drop_privileges') as _d_privs, \
+                mock.patch.object(wsgi, 'clean_up_daemon_hygiene') as _c_hyg, \
+                mock.patch.object(wsgi, 'loadapp', _loadapp), \
+                mock.patch.object(wsgi, 'capture_stdio'), \
+                mock.patch.object(wsgi, 'run_server'), \
+                mock.patch(
+                    'swift.common.wsgi.systemd_notify') as mock_notify, \
+                mock.patch('swift.common.utils.eventlet') as _utils_evt:
+            rc = wsgi.run_wsgi('conf_file', 'app_section')
+        self.assertEqual(calls['_initrp'], 1)
+        self.assertEqual(calls['_loadapp'], 1)
+        self.assertEqual(rc, 0)
+        _utils_evt.patcher.monkey_patch.assert_called_with(all=False,
+                                                           socket=True,
+                                                           select=True,
+                                                           thread=True)
+        self.assertEqual(mock_notify.mock_calls, [
+            mock.call('logger', "STOPPING=1"),
+        ])
+        # run_wsgi() no longer calls drop_privileges() in the parent process,
+        # just clean_up_daemon_hygene()
+        self.assertEqual([], _d_privs.mock_calls)
+        self.assertEqual([mock.call()], _c_hyg.mock_calls)
+        self.assertEqual(0, logging.logThreads)  # fixed in our monkey_patch
+
+    def test_run_server_test_config(self):
+        calls = defaultdict(int)
 
         def _initrp(conf_file, app_section, *args, **kwargs):
             calls['_initrp'] += 1
@@ -892,14 +1046,14 @@ class TestWSGI(unittest.TestCase):
             calls['_loadapp'] += 1
 
         with mock.patch.object(wsgi, '_initrp', _initrp), \
-                mock.patch.object(wsgi, 'get_socket'), \
+                mock.patch.object(wsgi, 'get_socket') as _get_socket, \
                 mock.patch.object(wsgi, 'drop_privileges') as _d_privs, \
                 mock.patch.object(wsgi, 'clean_up_daemon_hygiene') as _c_hyg, \
                 mock.patch.object(wsgi, 'loadapp', _loadapp), \
                 mock.patch.object(wsgi, 'capture_stdio'), \
                 mock.patch.object(wsgi, 'run_server'), \
                 mock.patch('swift.common.utils.eventlet') as _utils_evt:
-            rc = wsgi.run_wsgi('conf_file', 'app_section')
+            rc = wsgi.run_wsgi('conf_file', 'app_section', test_config=True)
         self.assertEqual(calls['_initrp'], 1)
         self.assertEqual(calls['_loadapp'], 1)
         self.assertEqual(rc, 0)
@@ -907,10 +1061,11 @@ class TestWSGI(unittest.TestCase):
                                                            socket=True,
                                                            select=True,
                                                            thread=True)
-        # run_wsgi() no longer calls drop_privileges() in the parent process,
-        # just clean_up_deemon_hygene()
+        # run_wsgi() stops before calling clean_up_daemon_hygene() or
+        # creating sockets
         self.assertEqual([], _d_privs.mock_calls)
-        self.assertEqual([mock.call()], _c_hyg.mock_calls)
+        self.assertEqual([], _c_hyg.mock_calls)
+        self.assertEqual([], _get_socket.mock_calls)
 
     @mock.patch('swift.common.wsgi.run_server')
     @mock.patch('swift.common.wsgi.WorkersStrategy')
@@ -935,6 +1090,7 @@ class TestWSGI(unittest.TestCase):
         ]
         with mock.patch.object(wsgi, '_initrp', return_value=stub__initrp), \
                 mock.patch.object(wsgi, 'loadapp'), \
+                mock.patch('swift.common.utils.monkey_patch'), \
                 mock.patch.object(wsgi, 'capture_stdio'):
             for server_type in ('account-server', 'container-server',
                                 'object-server'):
@@ -997,6 +1153,40 @@ class TestWSGI(unittest.TestCase):
         self.assertEqual(calls['_loadapp'], 0)
         self.assertEqual(rc, 1)
 
+    def test_run_server_bad_bind_port(self):
+        def do_test(port):
+            calls = defaultdict(lambda: 0)
+            logger = debug_logger()
+
+            def _initrp(conf_file, app_section, *args, **kwargs):
+                calls['_initrp'] += 1
+                return (
+                    {'__file__': 'test', 'workers': 0, 'bind_port': port},
+                    logger,
+                    'log_name')
+
+            def _loadapp(uri, name=None, **kwargs):
+                calls['_loadapp'] += 1
+
+            with mock.patch.object(wsgi, '_initrp', _initrp), \
+                    mock.patch.object(wsgi, 'get_socket'), \
+                    mock.patch.object(wsgi, 'drop_privileges'), \
+                    mock.patch.object(wsgi, 'loadapp', _loadapp), \
+                    mock.patch.object(wsgi, 'capture_stdio'), \
+                    mock.patch.object(wsgi, 'run_server'):
+                rc = wsgi.run_wsgi('conf_file', 'app_section')
+            self.assertEqual(calls['_initrp'], 1)
+            self.assertEqual(calls['_loadapp'], 0)
+            self.assertEqual(rc, 1)
+            self.assertEqual(
+                ["bind_port wasn't properly set in the config file. "
+                 "It must be explicitly set to a valid port number."],
+                logger.get_lines_for_level('error')
+            )
+
+        do_test('bad')
+        do_test('80000')
+
     def test_pre_auth_req_with_empty_env_no_path(self):
         r = wsgi.make_pre_authed_request(
             {}, 'GET')
@@ -1271,6 +1461,35 @@ class TestServersPerPortStrategy(unittest.TestCase, CommonTestMixin):
         # This is one of the workers for port 6006 that already got reaped.
         self.assertIsNone(self.strategy.register_worker_exit(89))
 
+    def test_servers_per_port_in_container(self):
+        # normally there's no configured ring_ip
+        conf = {
+            'bind_ip': '1.2.3.4',
+        }
+        self.strategy = wsgi.ServersPerPortStrategy(conf, self.logger, 1)
+        self.assertEqual(self.mock_cache_class.call_args,
+                         mock.call('/etc/swift', '1.2.3.4'))
+        self.assertEqual({6006, 6007},
+                         self.strategy.cache.all_bind_ports_for_node())
+        ports = {item[1][0] for item in self.strategy.new_worker_socks()}
+        self.assertEqual({6006, 6007}, ports)
+
+        # but in a container we can override it
+        conf = {
+            'bind_ip': '1.2.3.4',
+            'ring_ip': '2.3.4.5'
+        }
+        self.strategy = wsgi.ServersPerPortStrategy(conf, self.logger, 1)
+        # N.B. our fake BindPortsCache always returns {6006, 6007}, but a real
+        # BindPortsCache would only return ports for devices that match the ip
+        # address in the ring
+        self.assertEqual(self.mock_cache_class.call_args,
+                         mock.call('/etc/swift', '2.3.4.5'))
+        self.assertEqual({6006, 6007},
+                         self.strategy.cache.all_bind_ports_for_node())
+        ports = {item[1][0] for item in self.strategy.new_worker_socks()}
+        self.assertEqual({6006, 6007}, ports)
+
     def test_shutdown_sockets(self):
         pid = 88
         for s, i in self.strategy.new_worker_socks():
@@ -1404,10 +1623,12 @@ class TestWSGIContext(unittest.TestCase):
         r = Request.blank('/')
         it = wc._app_call(r.environ)
         self.assertEqual(wc._response_status, '200 Ok')
+        self.assertEqual(wc._get_status_int(), 200)
         self.assertEqual(b''.join(it), b'Ok\n')
         r = Request.blank('/')
         it = wc._app_call(r.environ)
         self.assertEqual(wc._response_status, '404 Not Found')
+        self.assertEqual(wc._get_status_int(), 404)
         self.assertEqual(b''.join(it), b'Ok\n')
 
     def test_app_iter_is_closable(self):
@@ -1426,6 +1647,7 @@ class TestWSGIContext(unittest.TestCase):
         r = Request.blank('/')
         iterable = wc._app_call(r.environ)
         self.assertEqual(wc._response_status, '200 OK')
+        self.assertEqual(wc._get_status_int(), 200)
 
         iterator = iter(iterable)
         self.assertEqual(b'aaaaa', next(iterator))
@@ -1446,6 +1668,7 @@ class TestWSGIContext(unittest.TestCase):
         it = wc._app_call(r.environ)
         wc.update_content_length(35)
         self.assertEqual(wc._response_status, '200 Ok')
+        self.assertEqual(wc._get_status_int(), 200)
         self.assertEqual(b''.join(it), b'Ok\n')
         self.assertEqual(wc._response_headers, [('Content-Length', '35')])
 
@@ -1461,6 +1684,7 @@ class TestWSGIContext(unittest.TestCase):
         it = wc._app_call(r.environ)
         wc._response_headers.append(('X-Trans-Id', 'txn'))
         self.assertEqual(wc._response_status, '200 Ok')
+        self.assertEqual(wc._get_status_int(), 200)
         self.assertEqual(b''.join(it), b'Ok\n')
         self.assertEqual(wc._response_headers, [
             ('Content-Length', '3'),
@@ -1604,15 +1828,23 @@ class TestPipelineModification(unittest.TestCase):
                     modify_func):
                 app = wsgi.loadapp(conf_file, global_conf={})
             exp = swift.common.middleware.catch_errors.CatchErrorMiddleware
-            self.assertTrue(isinstance(app, exp), app)
+            self.assertIsInstance(app, exp)
             exp = swift.common.middleware.healthcheck.HealthCheckMiddleware
-            self.assertTrue(isinstance(app.app, exp), app.app)
+            self.assertIsInstance(app.app, exp)
             exp = swift.proxy.server.Application
-            self.assertTrue(isinstance(app.app.app, exp), app.app.app)
+            self.assertIsInstance(app.app.app, exp)
             # Everybody gets a reference to the final app, too
             self.assertIs(app.app.app, app._pipeline_final_app)
+            self.assertIs(app.app.app, app._pipeline_request_logging_app)
             self.assertIs(app.app.app, app.app._pipeline_final_app)
+            self.assertIs(app.app.app, app.app._pipeline_request_logging_app)
             self.assertIs(app.app.app, app.app.app._pipeline_final_app)
+            exp_pipeline = [app, app.app, app.app.app]
+            self.assertEqual(exp_pipeline, app._pipeline)
+            self.assertEqual(exp_pipeline, app.app._pipeline)
+            self.assertEqual(exp_pipeline, app.app.app._pipeline)
+            self.assertIs(app._pipeline, app.app._pipeline)
+            self.assertIs(app._pipeline, app.app.app._pipeline)
 
             # make sure you can turn off the pipeline modification if you want
             def blow_up(*_, **__):
@@ -1626,9 +1858,74 @@ class TestPipelineModification(unittest.TestCase):
 
             # the pipeline was untouched
             exp = swift.common.middleware.healthcheck.HealthCheckMiddleware
-            self.assertTrue(isinstance(app, exp), app)
+            self.assertIsInstance(app, exp)
             exp = swift.proxy.server.Application
-            self.assertTrue(isinstance(app.app, exp), app.app)
+            self.assertIsInstance(app.app, exp)
+
+    def test_load_app_request_logging_app(self):
+        config = """
+        [DEFAULT]
+        swift_dir = TEMPDIR
+
+        [pipeline:main]
+        pipeline = catch_errors proxy_logging proxy-server
+
+        [app:proxy-server]
+        use = egg:swift#proxy
+        conn_timeout = 0.2
+
+        [filter:catch_errors]
+        use = egg:swift#catch_errors
+
+        [filter:proxy_logging]
+        use = egg:swift#proxy_logging
+        """
+
+        contents = dedent(config)
+        with temptree(['proxy-server.conf']) as t:
+            conf_file = os.path.join(t, 'proxy-server.conf')
+            with open(conf_file, 'w') as f:
+                f.write(contents.replace('TEMPDIR', t))
+            _fake_rings(t)
+            app = wsgi.loadapp(conf_file, global_conf={})
+
+            self.assertEqual(self.pipeline_modules(app),
+                             ['swift.common.middleware.catch_errors',
+                              'swift.common.middleware.gatekeeper',
+                              'swift.common.middleware.proxy_logging',
+                              'swift.common.middleware.listing_formats',
+                              'swift.common.middleware.copy',
+                              'swift.common.middleware.dlo',
+                              'swift.common.middleware.versioned_writes',
+                              'swift.proxy.server'])
+
+            pipeline = app._pipeline
+            logging_app = app._pipeline_request_logging_app
+            final_app = app._pipeline_final_app
+            # Sanity check -- loadapp returns the start of the pipeline
+            self.assertIs(app, pipeline[0])
+            # ... and the final_app is the end
+            self.assertIs(final_app, pipeline[-1])
+
+            # The logging app is its own special short pipeline
+            self.assertEqual(self.pipeline_modules(logging_app), [
+                'swift.common.middleware.proxy_logging',
+                'swift.proxy.server'])
+            self.assertNotIn(logging_app, pipeline)
+            self.assertIs(logging_app.app, final_app)
+
+            # All the apps in the main pipeline got decorated identically
+            for app in pipeline:
+                self.assertIs(app._pipeline, pipeline)
+                self.assertIs(app._pipeline_request_logging_app, logging_app)
+                self.assertIs(app._pipeline_final_app, final_app)
+
+            # Special logging app got them, too
+            self.assertIs(logging_app._pipeline_request_logging_app,
+                          logging_app)
+            self.assertIs(logging_app._pipeline_final_app, final_app)
+            # Though the pipeline's different -- may or may not matter?
+            self.assertEqual(logging_app._pipeline, [logging_app, final_app])
 
     def test_proxy_unmodified_wsgi_pipeline(self):
         # Make sure things are sane even when we modify nothing
@@ -2100,7 +2397,7 @@ class TestPipelineModification(unittest.TestCase):
             with open(conf_path, 'w') as f:
                 f.write(dedent(conf_body))
             app = wsgi.loadapp(conf_path)
-            self.assertTrue(isinstance(app, controller))
+            self.assertIsInstance(app, controller)
 
 
 if __name__ == '__main__':
