@@ -20,8 +20,8 @@ from textwrap import dedent
 
 import mock
 import errno
-from swift.common.utils import Timestamp
-from test.unit import debug_logger
+from swift.common.utils import Timestamp, readconf
+from test.debug_logger import debug_logger
 from swift.container import sync
 from swift.common.db import DatabaseConnectionError
 from swift.common import utils
@@ -134,7 +134,7 @@ class TestContainerSync(unittest.TestCase):
         self.assertTrue(cs.container_ring is cring)
         self.assertTrue(mock_ic.called)
         conf_path, name, retry = mock_ic.call_args[0]
-        self.assertTrue(isinstance(conf_path, ConfigString))
+        self.assertIsInstance(conf_path, ConfigString)
         self.assertEqual(conf_path.contents.getvalue(),
                          dedent(sync.ic_conf_body))
         self.assertLogMessage('warning', 'internal_client_conf_path')
@@ -154,9 +154,29 @@ class TestContainerSync(unittest.TestCase):
         sample_conf_filename = os.path.join(
             os.path.dirname(test.__file__),
             '../etc/internal-client.conf-sample')
-        with open(sample_conf_filename) as sample_conf_file:
-            sample_conf = sample_conf_file.read()
-        self.assertEqual(contents, sample_conf)
+        actual_conf = readconf(ConfigString(contents))
+        expected_conf = readconf(sample_conf_filename)
+        actual_conf.pop('__file__')
+        expected_conf.pop('__file__')
+        self.assertEqual(expected_conf, actual_conf)
+
+    def test_init_internal_client_log_name(self):
+        def _do_test_init_ic_log_name(conf, exp_internal_client_log_name):
+            with mock.patch(
+                    'swift.container.sync.InternalClient') \
+                    as mock_ic:
+                sync.ContainerSync(conf, container_ring='dummy object')
+            mock_ic.assert_called_once_with(
+                'conf-path',
+                'Swift Container Sync', 3,
+                global_conf={'log_name': exp_internal_client_log_name},
+                use_replication_network=True)
+
+        _do_test_init_ic_log_name({'internal_client_conf_path': 'conf-path'},
+                                  'container-sync-ic')
+        _do_test_init_ic_log_name({'internal_client_conf_path': 'conf-path',
+                                   'log_name': 'my-container-sync'},
+                                  'my-container-sync-ic')
 
     def test_run_forever(self):
         # This runs runs_forever with fakes to succeed for two loops, the first
