@@ -21,7 +21,7 @@ import logging
 import unittest
 import tempfile
 from shutil import rmtree
-from test.unit import debug_logger
+from test.debug_logger import debug_logger
 
 from swift.container.backend import DATADIR
 from swift.container import sync_store
@@ -69,7 +69,7 @@ class TestContainerSyncStore(unittest.TestCase):
                         with open(db_file, 'w') as outfile:
                             outfile.write('%s' % db_file)
 
-    def teardown(self):
+    def tearDown(self):
         rmtree(self.test_dir_prefix)
 
     def pick_dbfile(self):
