@@ -25,7 +25,7 @@ from contextlib import closing
 from gzip import GzipFile
 from tempfile import mkdtemp
 import time
-
+import warnings
 
 from eventlet import spawn, wsgi
 import mock
@@ -40,7 +40,7 @@ from swift.common.storage_policy import StoragePolicy, ECStoragePolicy
 from swift.common.middleware import listing_formats, proxy_logging
 from swift.common import utils
 from swift.common.utils import mkdirs, normalize_timestamp, NullLogger
-from swift.common.wsgi import SwiftHttpProtocol
+from swift.common.http_protocol import SwiftHttpProtocol
 from swift.container import server as container_server
 from swift.obj import server as object_server
 from swift.proxy import server as proxy_server
@@ -96,6 +96,7 @@ def setup_servers(the_object_server=object_server, extra_conf=None):
             'allow_versions': 't', 'node_timeout': 20}
     if extra_conf:
         conf.update(extra_conf)
+    context['conf'] = conf
     prolis = listen_zero()
     acc1lis = listen_zero()
     acc2lis = listen_zero()
@@ -215,6 +216,10 @@ def setup_servers(the_object_server=object_server, extra_conf=None):
     logging_prosv = proxy_logging.ProxyLoggingMiddleware(
         listing_formats.ListingFilter(prosrv, {}, logger=prosrv.logger),
         conf, logger=prosrv.logger)
+    # Yes, eventlet, we know -- we have to support bad clients, though
+    warnings.filterwarnings(
+        'ignore', module='eventlet',
+        message='capitalize_response_headers is disabled')
     prospa = spawn(wsgi.server, prolis, logging_prosv, nl,
                    protocol=SwiftHttpProtocol,
                    capitalize_response_headers=False)
@@ -262,7 +267,7 @@ def setup_servers(the_object_server=object_server, extra_conf=None):
                                                         {'X-Timestamp': ts,
                                                          'x-trans-id': 'test'})
         resp = conn.getresponse()
-        assert(resp.status == 201)
+        assert resp.status == 201
     # Create another account
     # used for account-to-account tests
     ts = normalize_timestamp(time.time())
@@ -276,7 +281,7 @@ def setup_servers(the_object_server=object_server, extra_conf=None):
                                                         {'X-Timestamp': ts,
                                                          'x-trans-id': 'test'})
         resp = conn.getresponse()
-        assert(resp.status == 201)
+        assert resp.status == 201
     # Create containers, 1 per test policy
     sock = connect_tcp(('localhost', prolis.getsockname()[1]))
     fd = sock.makefile('rwb')
