@@ -23,13 +23,14 @@ import six
 
 from swift.common import exceptions, utils
 from swift.common.storage_policy import POLICIES
+from swift.common.swob import wsgi_to_bytes, wsgi_to_str
 from swift.common.utils import Timestamp
 from swift.obj import ssync_sender, diskfile, ssync_receiver
 from swift.obj.replicator import ObjectReplicator
 
+from test.debug_logger import debug_logger
 from test.unit.obj.common import BaseTest
-from test.unit import patch_policies, make_timestamp_iter, skip_if_no_xattrs, \
-    debug_logger
+from test.unit import patch_policies, make_timestamp_iter, skip_if_no_xattrs
 
 
 class NullBufferedHTTPConnection(object):
@@ -102,8 +103,10 @@ class TestSender(BaseTest):
         self.daemon_logger = debug_logger('test-ssync-sender')
         self.daemon = ObjectReplicator(self.daemon_conf,
                                        self.daemon_logger)
-        job = {'policy': POLICIES.legacy}  # sufficient for Sender.__init__
-        self.sender = ssync_sender.Sender(self.daemon, None, job, None)
+        self.job = {'policy': POLICIES.legacy,
+                    'device': 'test-dev',
+                    'partition': '99'}  # sufficient for Sender.__init__
+        self.sender = ssync_sender.Sender(self.daemon, None, self.job, None)
 
     def test_call_catches_MessageTimeout(self):
 
@@ -788,13 +791,43 @@ class TestSender(BaseTest):
         self.assertEqual(response.readline(), b'')
         self.assertEqual(response.readline(), b'')
 
-    def test_missing_check_timeout(self):
+    def test_missing_check_timeout_start(self):
         connection = FakeConnection()
-        connection.send = lambda d: eventlet.sleep(1)
         response = FakeResponse()
         self.sender.daemon.node_timeout = 0.01
-        self.assertRaises(exceptions.MessageTimeout, self.sender.missing_check,
-                          connection, response)
+        self.assertFalse(self.sender.limited_by_max_objects)
+        with mock.patch.object(connection, 'send',
+                               side_effect=lambda *args: eventlet.sleep(1)):
+            with self.assertRaises(exceptions.MessageTimeout) as cm:
+                self.sender.missing_check(connection, response)
+        self.assertIn('0.01 seconds: missing_check start', str(cm.exception))
+        self.assertFalse(self.sender.limited_by_max_objects)
+
+    def test_missing_check_timeout_send_line(self):
+        def yield_hashes(device, partition, policy, suffixes=None, **kwargs):
+            yield (
+                '9d41d8cd98f00b204e9800998ecf0abc',
+                {'ts_data': Timestamp(1380144470.00000)})
+            yield (
+                '9d41d8cd98f00b204e9800998ecf0def',
+                {'ts_data': Timestamp(1380144471.00000)})
+        connection = FakeConnection()
+        response = FakeResponse()
+        # max_objects unlimited
+        self.sender = ssync_sender.Sender(self.daemon, None, self.job, None,
+                                          max_objects=0)
+        self.sender.daemon.node_timeout = 0.01
+        self.sender.df_mgr.yield_hashes = yield_hashes
+        self.assertFalse(self.sender.limited_by_max_objects)
+        sleeps = [0, 0, 1]
+        with mock.patch.object(
+                connection, 'send',
+                side_effect=lambda *args: eventlet.sleep(sleeps.pop(0))):
+            with self.assertRaises(exceptions.MessageTimeout) as cm:
+                self.sender.missing_check(connection, response)
+        self.assertIn('0.01 seconds: missing_check send line: '
+                      '1 lines (57 bytes) sent', str(cm.exception))
+        self.assertFalse(self.sender.limited_by_max_objects)
 
     def test_missing_check_has_empty_suffixes(self):
         def yield_hashes(device, partition, policy, suffixes=None, **kwargs):
@@ -818,6 +851,7 @@ class TestSender(BaseTest):
                 ':MISSING_CHECK: START\r\n'
                 ':MISSING_CHECK: END\r\n'))
         self.sender.df_mgr.yield_hashes = yield_hashes
+        self.assertFalse(self.sender.limited_by_max_objects)
         available_map, send_map = self.sender.missing_check(connection,
                                                             response)
         self.assertEqual(
@@ -826,6 +860,7 @@ class TestSender(BaseTest):
             b'15\r\n:MISSING_CHECK: END\r\n\r\n')
         self.assertEqual(send_map, {})
         self.assertEqual(available_map, {})
+        self.assertFalse(self.sender.limited_by_max_objects)
 
     def test_missing_check_has_suffixes(self):
         def yield_hashes(device, partition, policy, suffixes=None, **kwargs):
@@ -849,6 +884,10 @@ class TestSender(BaseTest):
                     'No match for %r %r %r %r' % (device, partition,
                                                   policy, suffixes))
 
+        # note: max_objects > number that would yield
+        self.sender = ssync_sender.Sender(self.daemon, None, self.job, None,
+                                          max_objects=4)
+
         connection = FakeConnection()
         self.sender.job = {
             'device': 'dev',
@@ -861,6 +900,7 @@ class TestSender(BaseTest):
                 ':MISSING_CHECK: START\r\n'
                 ':MISSING_CHECK: END\r\n'))
         self.sender.df_mgr.yield_hashes = yield_hashes
+        self.assertFalse(self.sender.limited_by_max_objects)
         available_map, send_map = self.sender.missing_check(connection,
                                                             response)
         self.assertEqual(
@@ -883,6 +923,126 @@ class TestSender(BaseTest):
                             ts_meta=Timestamp(1380144475.44444),
                             ts_ctype=Timestamp(1380144474.44448)))]
         self.assertEqual(available_map, dict(candidates))
+        self.assertEqual([], self.daemon_logger.get_lines_for_level('info'))
+        self.assertFalse(self.sender.limited_by_max_objects)
+
+    def test_missing_check_max_objects_less_than_actual_objects(self):
+        def yield_hashes(device, partition, policy, suffixes=None, **kwargs):
+            # verify missing_check stops after 2 objects even though more
+            # objects would yield
+            if (device == 'dev' and partition == '9' and
+                    policy == POLICIES.legacy and
+                    suffixes == ['abc', 'def']):
+                yield (
+                    '9d41d8cd98f00b204e9800998ecf0abc',
+                    {'ts_data': Timestamp(1380144470.00000)})
+                yield (
+                    '9d41d8cd98f00b204e9800998ecf0def',
+                    {'ts_data': Timestamp(1380144472.22222),
+                     'ts_meta': Timestamp(1380144473.22222)})
+                yield (
+                    '9d41d8cd98f00b204e9800998ecf1def',
+                    {'ts_data': Timestamp(1380144474.44444),
+                     'ts_ctype': Timestamp(1380144474.44448),
+                     'ts_meta': Timestamp(1380144475.44444)})
+            else:
+                raise Exception(
+                    'No match for %r %r %r %r' % (device, partition,
+                                                  policy, suffixes))
+
+        # max_objects < number that would yield
+        self.sender = ssync_sender.Sender(self.daemon, None, self.job, None,
+                                          max_objects=2)
+
+        connection = FakeConnection()
+        self.sender.job = {
+            'device': 'dev',
+            'partition': '9',
+            'policy': POLICIES.legacy,
+        }
+        self.sender.suffixes = ['abc', 'def']
+        response = FakeResponse(
+            chunk_body=(
+                ':MISSING_CHECK: START\r\n'
+                ':MISSING_CHECK: END\r\n'))
+        self.sender.df_mgr.yield_hashes = yield_hashes
+        self.assertFalse(self.sender.limited_by_max_objects)
+        available_map, send_map = self.sender.missing_check(connection,
+                                                            response)
+        self.assertEqual(
+            b''.join(connection.sent),
+            b'17\r\n:MISSING_CHECK: START\r\n\r\n'
+            b'33\r\n9d41d8cd98f00b204e9800998ecf0abc 1380144470.00000\r\n\r\n'
+            b'3b\r\n9d41d8cd98f00b204e9800998ecf0def 1380144472.22222 '
+            b'm:186a0\r\n\r\n'
+            b'15\r\n:MISSING_CHECK: END\r\n\r\n')
+        self.assertEqual(send_map, {})
+        candidates = [('9d41d8cd98f00b204e9800998ecf0abc',
+                       dict(ts_data=Timestamp(1380144470.00000))),
+                      ('9d41d8cd98f00b204e9800998ecf0def',
+                       dict(ts_data=Timestamp(1380144472.22222),
+                            ts_meta=Timestamp(1380144473.22222)))]
+        self.assertEqual(available_map, dict(candidates))
+        self.assertEqual(
+            ['ssync missing_check truncated after 2 objects: device: dev, '
+             'part: 9, policy: 0, last object hash: '
+             '9d41d8cd98f00b204e9800998ecf0def'],
+            self.daemon_logger.get_lines_for_level('info'))
+        self.assertTrue(self.sender.limited_by_max_objects)
+
+    def test_missing_check_max_objects_exactly_actual_objects(self):
+        def yield_hashes(device, partition, policy, suffixes=None, **kwargs):
+            if (device == 'dev' and partition == '9' and
+                    policy == POLICIES.legacy and
+                    suffixes == ['abc', 'def']):
+                yield (
+                    '9d41d8cd98f00b204e9800998ecf0abc',
+                    {'ts_data': Timestamp(1380144470.00000)})
+                yield (
+                    '9d41d8cd98f00b204e9800998ecf0def',
+                    {'ts_data': Timestamp(1380144472.22222),
+                     'ts_meta': Timestamp(1380144473.22222)})
+            else:
+                raise Exception(
+                    'No match for %r %r %r %r' % (device, partition,
+                                                  policy, suffixes))
+
+        # max_objects == number that would yield
+        self.sender = ssync_sender.Sender(self.daemon, None, self.job, None,
+                                          max_objects=2)
+
+        connection = FakeConnection()
+        self.sender.job = {
+            'device': 'dev',
+            'partition': '9',
+            'policy': POLICIES.legacy,
+        }
+        self.sender.suffixes = ['abc', 'def']
+        response = FakeResponse(
+            chunk_body=(
+                ':MISSING_CHECK: START\r\n'
+                ':MISSING_CHECK: END\r\n'))
+        self.sender.df_mgr.yield_hashes = yield_hashes
+        self.assertFalse(self.sender.limited_by_max_objects)
+        available_map, send_map = self.sender.missing_check(connection,
+                                                            response)
+        self.assertEqual(
+            b''.join(connection.sent),
+            b'17\r\n:MISSING_CHECK: START\r\n\r\n'
+            b'33\r\n9d41d8cd98f00b204e9800998ecf0abc 1380144470.00000\r\n\r\n'
+            b'3b\r\n9d41d8cd98f00b204e9800998ecf0def 1380144472.22222 '
+            b'm:186a0\r\n\r\n'
+            b'15\r\n:MISSING_CHECK: END\r\n\r\n')
+        self.assertEqual(send_map, {})
+        candidates = [('9d41d8cd98f00b204e9800998ecf0abc',
+                       dict(ts_data=Timestamp(1380144470.00000))),
+                      ('9d41d8cd98f00b204e9800998ecf0def',
+                       dict(ts_data=Timestamp(1380144472.22222),
+                            ts_meta=Timestamp(1380144473.22222)))]
+        self.assertEqual(available_map, dict(candidates))
+        # nothing logged re: truncation
+        self.assertEqual([], self.daemon_logger.get_lines_for_level('info'))
+        self.assertFalse(self.sender.limited_by_max_objects)
 
     def test_missing_check_far_end_disconnect(self):
         def yield_hashes(device, partition, policy, suffixes=None, **kwargs):
@@ -905,6 +1065,7 @@ class TestSender(BaseTest):
         }
         self.sender.suffixes = ['abc']
         self.sender.df_mgr.yield_hashes = yield_hashes
+        self.assertFalse(self.sender.limited_by_max_objects)
         response = FakeResponse(chunk_body='\r\n')
         exc = None
         try:
@@ -917,6 +1078,7 @@ class TestSender(BaseTest):
             b'17\r\n:MISSING_CHECK: START\r\n\r\n'
             b'33\r\n9d41d8cd98f00b204e9800998ecf0abc 1380144470.00000\r\n\r\n'
             b'15\r\n:MISSING_CHECK: END\r\n\r\n')
+        self.assertFalse(self.sender.limited_by_max_objects)
 
     def test_missing_check_far_end_disconnect2(self):
         def yield_hashes(device, partition, policy, suffixes=None, **kwargs):
@@ -939,6 +1101,7 @@ class TestSender(BaseTest):
         }
         self.sender.suffixes = ['abc']
         self.sender.df_mgr.yield_hashes = yield_hashes
+        self.assertFalse(self.sender.limited_by_max_objects)
         response = FakeResponse(
             chunk_body=':MISSING_CHECK: START\r\n')
         exc = None
@@ -952,6 +1115,7 @@ class TestSender(BaseTest):
             b'17\r\n:MISSING_CHECK: START\r\n\r\n'
             b'33\r\n9d41d8cd98f00b204e9800998ecf0abc 1380144470.00000\r\n\r\n'
             b'15\r\n:MISSING_CHECK: END\r\n\r\n')
+        self.assertFalse(self.sender.limited_by_max_objects)
 
     def test_missing_check_far_end_unexpected(self):
         def yield_hashes(device, partition, policy, suffixes=None, **kwargs):
@@ -974,6 +1138,7 @@ class TestSender(BaseTest):
         }
         self.sender.suffixes = ['abc']
         self.sender.df_mgr.yield_hashes = yield_hashes
+        self.assertFalse(self.sender.limited_by_max_objects)
         response = FakeResponse(chunk_body='OH HAI\r\n')
         exc = None
         try:
@@ -986,6 +1151,7 @@ class TestSender(BaseTest):
             b'17\r\n:MISSING_CHECK: START\r\n\r\n'
             b'33\r\n9d41d8cd98f00b204e9800998ecf0abc 1380144470.00000\r\n\r\n'
             b'15\r\n:MISSING_CHECK: END\r\n\r\n')
+        self.assertFalse(self.sender.limited_by_max_objects)
 
     def test_missing_check_send_map(self):
         def yield_hashes(device, partition, policy, suffixes=None, **kwargs):
@@ -1013,6 +1179,7 @@ class TestSender(BaseTest):
                 '0123abc dm\r\n'
                 ':MISSING_CHECK: END\r\n'))
         self.sender.df_mgr.yield_hashes = yield_hashes
+        self.assertFalse(self.sender.limited_by_max_objects)
         available_map, send_map = self.sender.missing_check(connection,
                                                             response)
         self.assertEqual(
@@ -1024,6 +1191,7 @@ class TestSender(BaseTest):
         self.assertEqual(available_map,
                          dict([('9d41d8cd98f00b204e9800998ecf0abc',
                                 {'ts_data': Timestamp(1380144470.00000)})]))
+        self.assertFalse(self.sender.limited_by_max_objects)
 
     def test_missing_check_extra_line_parts(self):
         # check that sender tolerates extra parts in missing check
@@ -1053,12 +1221,14 @@ class TestSender(BaseTest):
                 '0123abc d extra response parts\r\n'
                 ':MISSING_CHECK: END\r\n'))
         self.sender.df_mgr.yield_hashes = yield_hashes
+        self.assertFalse(self.sender.limited_by_max_objects)
         available_map, send_map = self.sender.missing_check(connection,
                                                             response)
         self.assertEqual(send_map, {'0123abc': {'data': True}})
         self.assertEqual(available_map,
                          dict([('9d41d8cd98f00b204e9800998ecf0abc',
                                 {'ts_data': Timestamp(1380144470.00000)})]))
+        self.assertFalse(self.sender.limited_by_max_objects)
 
     def test_updates_timeout(self):
         connection = FakeConnection()
@@ -1221,7 +1391,7 @@ class TestSender(BaseTest):
         args, _kwargs = self.sender.send_put.call_args
         connection, path, df = args
         self.assertEqual(path, '/a/c/o')
-        self.assertTrue(isinstance(df, diskfile.DiskFile))
+        self.assertIsInstance(df, diskfile.DiskFile)
         self.assertEqual(expected, df.get_metadata())
         # note that the put line isn't actually sent since we mock send_put;
         # send_put is tested separately.
@@ -1355,7 +1525,7 @@ class TestSender(BaseTest):
         args, _kwargs = self.sender.send_put.call_args
         connection, path, df = args
         self.assertEqual(path, '/a/c/o')
-        self.assertTrue(isinstance(df, diskfile.DiskFile))
+        self.assertIsInstance(df, diskfile.DiskFile)
         self.assertEqual(expected, df.get_metadata())
         self.assertEqual(os.path.join(self.tx_testdir, 'dev/objects/9/',
                                       object_hash[-3:], object_hash),
@@ -1522,12 +1692,15 @@ class TestSender(BaseTest):
             exc = err
         self.assertEqual(str(exc), '0.01 seconds: send_put chunk')
 
-    def _check_send_put(self, obj_name, meta_value, durable=True):
+    def _check_send_put(self, obj_name, meta_value,
+                        meta_name='Unicode-Meta-Name', durable=True):
         ts_iter = make_timestamp_iter()
         t1 = next(ts_iter)
         body = b'test'
         extra_metadata = {'Some-Other-Header': 'value',
-                          u'Unicode-Meta-Name': meta_value}
+                          meta_name: meta_value}
+        # Note that diskfile expects obj_name to be a native string
+        # but metadata to be wsgi strings
         df = self._make_open_diskfile(obj=obj_name, body=body,
                                       timestamp=t1,
                                       extra_metadata=extra_metadata,
@@ -1536,12 +1709,13 @@ class TestSender(BaseTest):
         expected['body'] = body if six.PY2 else body.decode('ascii')
         expected['chunk_size'] = len(body)
         expected['meta'] = meta_value
-        wire_meta = meta_value if six.PY2 else meta_value.encode('utf8')
+        expected['meta_name'] = meta_name
         path = six.moves.urllib.parse.quote(expected['name'])
         expected['path'] = path
         no_commit = '' if durable else 'X-Backend-No-Commit: True\r\n'
         expected['no_commit'] = no_commit
-        length = 145 + len(path) + len(wire_meta) + len(no_commit)
+        length = 128 + len(path) + len(meta_value) + len(no_commit) + \
+            len(meta_name)
         expected['length'] = format(length, 'x')
         # .meta file metadata is not included in expected for data only PUT
         t2 = next(ts_iter)
@@ -1556,15 +1730,14 @@ class TestSender(BaseTest):
             'Content-Length: %(Content-Length)s\r\n'
             'ETag: %(ETag)s\r\n'
             'Some-Other-Header: value\r\n'
-            'Unicode-Meta-Name: %(meta)s\r\n'
+            '%(meta_name)s: %(meta)s\r\n'
             '%(no_commit)s'
             'X-Timestamp: %(X-Timestamp)s\r\n'
             '\r\n'
             '\r\n'
             '%(chunk_size)s\r\n'
             '%(body)s\r\n' % expected)
-        if not six.PY2:
-            expected = expected.encode('utf8')
+        expected = wsgi_to_bytes(expected)
         self.assertEqual(b''.join(connection.sent), expected)
 
     def test_send_put(self):
@@ -1574,12 +1747,14 @@ class TestSender(BaseTest):
         self._check_send_put('o', 'meta', durable=False)
 
     def test_send_put_unicode(self):
-        if six.PY2:
-            self._check_send_put(
-                'o_with_caract\xc3\xa8res_like_in_french', 'm\xc3\xa8ta')
-        else:
-            self._check_send_put(
-                'o_with_caract\u00e8res_like_in_french', 'm\u00e8ta')
+        self._check_send_put(
+            wsgi_to_str('o_with_caract\xc3\xa8res_like_in_french'),
+            'm\xc3\xa8ta')
+
+    def test_send_put_unicode_header_name(self):
+        self._check_send_put(
+            wsgi_to_str('o_with_caract\xc3\xa8res_like_in_french'),
+            'm\xc3\xa8ta', meta_name='X-Object-Meta-Nam\xc3\xa8')
 
     def _check_send_post(self, obj_name, meta_value):
         ts_iter = make_timestamp_iter()
@@ -1595,9 +1770,11 @@ class TestSender(BaseTest):
         ts_1 = next(ts_iter)
         newer_metadata = {u'X-Object-Meta-Foo': meta_value,
                           'X-Timestamp': ts_1.internal}
+        # Note that diskfile expects obj_name to be a native string
+        # but metadata to be wsgi strings
         df.write_metadata(newer_metadata)
         path = six.moves.urllib.parse.quote(df.read_metadata()['name'])
-        wire_meta = meta_value if six.PY2 else meta_value.encode('utf8')
+        wire_meta = wsgi_to_bytes(meta_value)
         length = format(61 + len(path) + len(wire_meta), 'x')
 
         connection = FakeConnection()
@@ -1618,12 +1795,9 @@ class TestSender(BaseTest):
         self._check_send_post('o', 'meta')
 
     def test_send_post_unicode(self):
-        if six.PY2:
-            self._check_send_post(
-                'o_with_caract\xc3\xa8res_like_in_french', 'm\xc3\xa8ta')
-        else:
-            self._check_send_post(
-                'o_with_caract\u00e8res_like_in_french', 'm\u00e8ta')
+        self._check_send_post(
+            wsgi_to_str('o_with_caract\xc3\xa8res_like_in_french'),
+            'm\xc3\xa8ta')
 
     def test_disconnect_timeout(self):
         connection = FakeConnection()
@@ -1871,6 +2045,19 @@ class TestModuleMethods(unittest.TestCase):
             ssync_sender.encode_missing(object_hash, t_data, t_meta, t_type,
                                         durable=True))
 
+        # timestamps have offsets
+        t_data_offset = utils.Timestamp(t_data, offset=99)
+        t_meta_offset = utils.Timestamp(t_meta, offset=1)
+        t_type_offset = utils.Timestamp(t_type, offset=2)
+        expected = ('%s %s m:%x__1,t:%x__2'
+                    % (object_hash, t_data_offset.internal, d_meta_data,
+                       d_type_data))
+        self.assertEqual(
+            expected.encode('ascii'),
+            ssync_sender.encode_missing(
+                object_hash, t_data_offset, t_meta_offset, t_type_offset,
+                durable=True))
+
         # test encode and decode functions invert
         expected = {'object_hash': object_hash, 'ts_meta': t_meta,
                     'ts_data': t_data, 'ts_ctype': t_type, 'durable': False}
@@ -1884,6 +2071,38 @@ class TestModuleMethods(unittest.TestCase):
         actual = ssync_receiver.decode_missing(msg)
         self.assertEqual(expected, actual)
 
+        # test encode and decode functions invert with offset
+        t_data_offset = utils.Timestamp(t_data, offset=1)
+        expected = {'object_hash': object_hash, 'ts_meta': t_meta,
+                    'ts_data': t_data_offset, 'ts_ctype': t_type,
+                    'durable': False}
+        msg = ssync_sender.encode_missing(**expected)
+        actual = ssync_receiver.decode_missing(msg)
+        self.assertEqual(expected, actual)
+
+        t_meta_offset = utils.Timestamp(t_data, offset=2)
+        expected = {'object_hash': object_hash, 'ts_meta': t_meta_offset,
+                    'ts_data': t_data, 'ts_ctype': t_type,
+                    'durable': False}
+        msg = ssync_sender.encode_missing(**expected)
+        actual = ssync_receiver.decode_missing(msg)
+        self.assertEqual(expected, actual)
+
+        t_type_offset = utils.Timestamp(t_type, offset=3)
+        expected = {'object_hash': object_hash, 'ts_meta': t_meta,
+                    'ts_data': t_data, 'ts_ctype': t_type_offset,
+                    'durable': False}
+        msg = ssync_sender.encode_missing(**expected)
+        actual = ssync_receiver.decode_missing(msg)
+        self.assertEqual(expected, actual)
+
+        expected = {'object_hash': object_hash, 'ts_meta': t_meta_offset,
+                    'ts_data': t_data_offset, 'ts_ctype': t_type_offset,
+                    'durable': False}
+        msg = ssync_sender.encode_missing(**expected)
+        actual = ssync_receiver.decode_missing(msg)
+        self.assertEqual(expected, actual)
+
     def test_decode_wanted(self):
         parts = ['d']
         expected = {'data': True}
