@@ -12,10 +12,9 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from queue import PriorityQueue
-
 import eventlet
 import six.moves.cPickle as pickle
+from six.moves.queue import PriorityQueue
 import mock
 import os
 import unittest
@@ -33,7 +32,6 @@ from test.debug_logger import debug_logger
 from test.unit import (
     make_timestamp_iter, patch_policies, mocked_http_conn)
 from time import time
-from distutils.dir_util import mkpath
 
 from eventlet import spawn, Timeout
 
@@ -233,7 +231,7 @@ class TestObjectUpdater(unittest.TestCase):
                 asyncdir = os.path.join(self.sda1, ASYNCDIR_BASE)
 
             prefix_dir = os.path.join(asyncdir, 'abc')
-            mkpath(prefix_dir)
+            mkdirs(prefix_dir)
 
             # A non-directory where directory is expected should just be
             # skipped, but should not stop processing of subsequent
@@ -307,7 +305,7 @@ class TestObjectUpdater(unittest.TestCase):
     def test_sweep_logs(self):
         asyncdir = os.path.join(self.sda1, ASYNCDIR_BASE)
         prefix_dir = os.path.join(asyncdir, 'abc')
-        mkpath(prefix_dir)
+        mkdirs(prefix_dir)
 
         for o, t in [('abc', 123), ('def', 234), ('ghi', 345),
                      ('jkl', 456), ('mno', 567)]:
@@ -379,7 +377,7 @@ class TestObjectUpdater(unittest.TestCase):
         for policy in _mocked_policies:
             asyncdir = os.path.join(self.sda1, get_async_dir(policy.idx))
             prefix_dir = os.path.join(asyncdir, 'abc')
-            mkpath(prefix_dir)
+            mkdirs(prefix_dir)
 
             for o, t in [('abc', 123), ('def', 234), ('ghi', 345)]:
                 ohash = hash_path('account', 'container%d' % policy.idx, o)
@@ -462,7 +460,7 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertEqual([
             mock.call(self.devices_dir, 'sda1', True),
         ], mock_check_drive.mock_calls)
-        self.assertEqual(ou.logger.get_increment_counts(), {})
+        self.assertEqual(ou.logger.statsd_client.get_increment_counts(), {})
 
     @mock.patch('swift.obj.updater.dump_recon_cache')
     @mock.patch.object(object_updater, 'check_drive')
@@ -526,16 +524,16 @@ class TestObjectUpdater(unittest.TestCase):
         ou.run_once()
         self.assertTrue(not os.path.exists(older_op_path))
         self.assertTrue(os.path.exists(op_path))
-        self.assertEqual(ou.logger.get_increment_counts(),
+        self.assertEqual(ou.logger.statsd_client.get_increment_counts(),
                          {'failures': 1, 'unlinks': 1})
         self.assertIsNone(pickle.load(open(op_path, 'rb')).get('successes'))
         self.assertEqual(
             ['ERROR with remote server 127.0.0.1:67890/sda1: '
              'Connection refused'] * 3,
             ou.logger.get_lines_for_level('error'))
-        self.assertEqual([args for args, kw in ou.logger.log_dict['timing']], [
-            ('updater.timing.status.500', mock.ANY),
-        ] * 3)
+        self.assertEqual(
+            sorted(ou.logger.statsd_client.calls['timing']),
+            sorted([(('updater.timing.status.500', mock.ANY), {}), ] * 3))
         ou.logger.clear()
 
         bindsock = listen_zero()
@@ -591,17 +589,17 @@ class TestObjectUpdater(unittest.TestCase):
         if err:
             raise err
         self.assertTrue(os.path.exists(op_path))
-        self.assertEqual(ou.logger.get_increment_counts(),
+        self.assertEqual(ou.logger.statsd_client.get_increment_counts(),
                          {'failures': 1})
         self.assertEqual([0],
                          pickle.load(open(op_path, 'rb')).get('successes'))
         self.assertEqual([], ou.logger.get_lines_for_level('error'))
         self.assertEqual(
-            sorted([args for args, kw in ou.logger.log_dict['timing']]),
+            sorted(ou.logger.statsd_client.calls['timing']),
             sorted([
-                ('updater.timing.status.201', mock.ANY),
-                ('updater.timing.status.500', mock.ANY),
-                ('updater.timing.status.500', mock.ANY),
+                (('updater.timing.status.201', mock.ANY), {}),
+                (('updater.timing.status.500', mock.ANY), {}),
+                (('updater.timing.status.500', mock.ANY), {}),
             ]))
 
         # only 1/2 updates succeeds
@@ -612,16 +610,16 @@ class TestObjectUpdater(unittest.TestCase):
         if err:
             raise err
         self.assertTrue(os.path.exists(op_path))
-        self.assertEqual(ou.logger.get_increment_counts(),
+        self.assertEqual(ou.logger.statsd_client.get_increment_counts(),
                          {'failures': 1})
         self.assertEqual([0, 2],
                          pickle.load(open(op_path, 'rb')).get('successes'))
         self.assertEqual([], ou.logger.get_lines_for_level('error'))
         self.assertEqual(
-            sorted([args for args, kw in ou.logger.log_dict['timing']]),
+            sorted(ou.logger.statsd_client.calls['timing']),
             sorted([
-                ('updater.timing.status.404', mock.ANY),
-                ('updater.timing.status.201', mock.ANY),
+                (('updater.timing.status.404', mock.ANY), {}),
+                (('updater.timing.status.201', mock.ANY), {}),
             ]))
 
         # final update has Timeout
@@ -631,7 +629,7 @@ class TestObjectUpdater(unittest.TestCase):
             mock_connect.return_value.getresponse.side_effect = exc
             ou.run_once()
         self.assertTrue(os.path.exists(op_path))
-        self.assertEqual(ou.logger.get_increment_counts(),
+        self.assertEqual(ou.logger.statsd_client.get_increment_counts(),
                          {'failures': 1})
         self.assertEqual([0, 2],
                          pickle.load(open(op_path, 'rb')).get('successes'))
@@ -639,9 +637,10 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertIn(
             'Timeout waiting on remote server 127.0.0.1:%d/sda1: 99 seconds'
             % bindsock.getsockname()[1], ou.logger.get_lines_for_level('info'))
-        self.assertEqual([args for args, kw in ou.logger.log_dict['timing']], [
-            ('updater.timing.status.499', mock.ANY),
-        ])
+        self.assertEqual(
+            sorted(ou.logger.statsd_client.calls['timing']),
+            sorted([
+                (('updater.timing.status.499', mock.ANY), {})]))
 
         # final update has ConnectionTimeout
         ou.logger.clear()
@@ -650,7 +649,7 @@ class TestObjectUpdater(unittest.TestCase):
             mock_connect.return_value.getresponse.side_effect = exc
             ou.run_once()
         self.assertTrue(os.path.exists(op_path))
-        self.assertEqual(ou.logger.get_increment_counts(),
+        self.assertEqual(ou.logger.statsd_client.get_increment_counts(),
                          {'failures': 1})
         self.assertEqual([0, 2],
                          pickle.load(open(op_path, 'rb')).get('successes'))
@@ -658,9 +657,11 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertIn(
             'Timeout connecting to remote server 127.0.0.1:%d/sda1: 9 seconds'
             % bindsock.getsockname()[1], ou.logger.get_lines_for_level('info'))
-        self.assertEqual([args for args, kw in ou.logger.log_dict['timing']], [
-            ('updater.timing.status.500', mock.ANY),
-        ])
+        self.assertEqual(
+            sorted(ou.logger.statsd_client.calls['timing']),
+            sorted([
+                (('updater.timing.status.500', mock.ANY), {})
+            ]))
 
         # final update succeeds
         event = spawn(accept, [201])
@@ -677,11 +678,13 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertTrue(os.path.exists(os.path.dirname(os.path.dirname(
             op_path))))
         self.assertEqual([], ou.logger.get_lines_for_level('error'))
-        self.assertEqual(ou.logger.get_increment_counts(),
+        self.assertEqual(ou.logger.statsd_client.get_increment_counts(),
                          {'unlinks': 1, 'successes': 1})
-        self.assertEqual([args for args, kw in ou.logger.log_dict['timing']], [
-            ('updater.timing.status.201', mock.ANY),
-        ])
+        self.assertEqual(
+            sorted(ou.logger.statsd_client.calls['timing']),
+            sorted([
+                (('updater.timing.status.201', mock.ANY), {}),
+            ]))
 
     def test_obj_put_legacy_updates(self):
         ts = (normalize_timestamp(t) for t in
@@ -699,7 +702,7 @@ class TestObjectUpdater(unittest.TestCase):
         account, container, obj = 'a', 'c', 'o'
         # write an async
         for op in ('PUT', 'DELETE'):
-            self.logger._clear()
+            self.logger.clear()
             daemon = object_updater.ObjectUpdater(conf, logger=self.logger)
             dfmanager = DiskFileManager(conf, daemon.logger)
             # don't include storage-policy-index in headers_out pickle
@@ -729,9 +732,9 @@ class TestObjectUpdater(unittest.TestCase):
                 self.assertEqual(method, op)
                 self.assertEqual(headers['X-Backend-Storage-Policy-Index'],
                                  str(int(policy)))
-            self.assertEqual(daemon.logger.get_increment_counts(),
-                             {'successes': 1, 'unlinks': 1,
-                              'async_pendings': 1})
+            self.assertEqual(
+                daemon.logger.statsd_client.get_increment_counts(),
+                {'successes': 1, 'unlinks': 1, 'async_pendings': 1})
 
     def _write_async_update(self, dfmanager, timestamp, policy,
                             headers=None, container_path=None):
@@ -792,7 +795,7 @@ class TestObjectUpdater(unittest.TestCase):
                 self.assertEqual(method, 'PUT')
                 self.assertDictEqual(expected, headers)
             self.assertEqual(
-                daemon.logger.get_increment_counts(),
+                daemon.logger.statsd_client.get_increment_counts(),
                 {'successes': 1, 'unlinks': 1, 'async_pendings': 1})
             self.assertFalse(os.listdir(async_dir))
             daemon.logger.clear()
@@ -909,7 +912,7 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertEqual(
             {'redirects': 1, 'successes': 1,
              'unlinks': 1, 'async_pendings': 1},
-            daemon.logger.get_increment_counts())
+            daemon.logger.statsd_client.get_increment_counts())
         self.assertFalse(os.listdir(async_dir))  # no async file
 
     def test_obj_put_async_root_update_redirected_previous_success(self):
@@ -941,7 +944,7 @@ class TestObjectUpdater(unittest.TestCase):
                          [req['path'] for req in conn.requests])
         self.assertEqual(
             {'failures': 1, 'async_pendings': 1},
-            daemon.logger.get_increment_counts())
+            daemon.logger.statsd_client.get_increment_counts())
         async_path, async_data = self._check_async_file(async_dir)
         self.assertEqual(dict(orig_async_data, successes=[1]), async_data)
 
@@ -969,7 +972,7 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertEqual(
             {'redirects': 1, 'successes': 1, 'failures': 1, 'unlinks': 1,
              'async_pendings': 1},
-            daemon.logger.get_increment_counts())
+            daemon.logger.statsd_client.get_increment_counts())
         self.assertFalse(os.listdir(async_dir))  # no async file
 
     def _check_async_file(self, async_dir):
@@ -1017,7 +1020,7 @@ class TestObjectUpdater(unittest.TestCase):
                          [req['path'] for req in conn.requests])
         self.assertEqual(
             {'failures': 1, 'async_pendings': 1},
-            daemon.logger.get_increment_counts())
+            daemon.logger.statsd_client.get_increment_counts())
         # async file still intact
         async_path, async_data = self._check_async_file(async_dir)
         self.assertEqual(orig_async_path, async_path)
@@ -1096,7 +1099,7 @@ class TestObjectUpdater(unittest.TestCase):
             [req['path'] for req in conn.requests])
         self.assertEqual(
             {'redirects': 2, 'async_pendings': 1},
-            daemon.logger.get_increment_counts())
+            daemon.logger.statsd_client.get_increment_counts())
         # update failed, we still have pending file with most recent redirect
         # response Location header value added to data
         async_path, async_data = self._check_async_file(async_dir)
@@ -1122,7 +1125,7 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertEqual(
             {'redirects': 2, 'successes': 1, 'unlinks': 1,
              'async_pendings': 1},
-            daemon.logger.get_increment_counts())
+            daemon.logger.statsd_client.get_increment_counts())
         self.assertFalse(os.listdir(async_dir))  # no async file
 
     def test_obj_put_async_update_redirection_loop(self):
@@ -1170,7 +1173,7 @@ class TestObjectUpdater(unittest.TestCase):
                          [req['path'] for req in conn.requests])
         self.assertEqual(
             {'redirects': 2, 'async_pendings': 1},
-            daemon.logger.get_increment_counts())
+            daemon.logger.statsd_client.get_increment_counts())
         # update failed, we still have pending file with most recent redirect
         # response Location header value added to data
         async_path, async_data = self._check_async_file(async_dir)
@@ -1202,7 +1205,7 @@ class TestObjectUpdater(unittest.TestCase):
             [req['path'] for req in conn.requests])
         self.assertEqual(
             {'redirects': 4, 'async_pendings': 1},
-            daemon.logger.get_increment_counts())
+            daemon.logger.statsd_client.get_increment_counts())
         # update failed, we still have pending file with most recent redirect
         # response Location header value from root added to persisted data
         async_path, async_data = self._check_async_file(async_dir)
@@ -1232,7 +1235,7 @@ class TestObjectUpdater(unittest.TestCase):
             [req['path'] for req in conn.requests])
         self.assertEqual(
             {'redirects': 6, 'async_pendings': 1},
-            daemon.logger.get_increment_counts())
+            daemon.logger.statsd_client.get_increment_counts())
         # update failed, we still have pending file, but container_path is None
         # because most recent redirect location was a repeat
         async_path, async_data = self._check_async_file(async_dir)
@@ -1256,7 +1259,7 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertEqual(
             {'redirects': 6, 'successes': 1, 'unlinks': 1,
              'async_pendings': 1},
-            daemon.logger.get_increment_counts())
+            daemon.logger.statsd_client.get_increment_counts())
         self.assertFalse(os.listdir(async_dir))  # no async file
 
     def test_obj_update_quarantine(self):
@@ -1288,7 +1291,7 @@ class TestObjectUpdater(unittest.TestCase):
 
         self.assertEqual(
             {'quarantines': 1},
-            daemon.logger.get_increment_counts())
+            daemon.logger.statsd_client.get_increment_counts())
         self.assertFalse(os.listdir(async_dir))  # no asyncs
 
     def test_obj_update_gone_missing(self):
@@ -1320,7 +1323,8 @@ class TestObjectUpdater(unittest.TestCase):
         with mocked_http_conn():
             with mock.patch('swift.obj.updater.dump_recon_cache'):
                 daemon._load_update(self.sda1, op_path)
-        self.assertEqual({}, daemon.logger.get_increment_counts())
+        self.assertEqual(
+            {}, daemon.logger.statsd_client.get_increment_counts())
         self.assertEqual(os.listdir(async_dir), [ohash[-3:]])
         self.assertFalse(os.listdir(odir))
 
@@ -1400,7 +1404,7 @@ class TestObjectUpdater(unittest.TestCase):
                       info_lines[-1])
         self.assertEqual({'skips': 9, 'successes': 2, 'unlinks': 2,
                           'deferrals': 9},
-                         self.logger.get_increment_counts())
+                         self.logger.statsd_client.get_increment_counts())
 
     @mock.patch('swift.obj.updater.dump_recon_cache')
     def test_per_container_rate_limit_unlimited(self, mock_recon):
@@ -1438,7 +1442,7 @@ class TestObjectUpdater(unittest.TestCase):
                       '0 errors, 0 redirects, 0 skips, 0 deferrals, 0 drains',
                       info_lines[-1])
         self.assertEqual({'successes': 11, 'unlinks': 11},
-                         self.logger.get_increment_counts())
+                         self.logger.statsd_client.get_increment_counts())
 
     @mock.patch('swift.obj.updater.dump_recon_cache')
     def test_per_container_rate_limit_some_limited(self, mock_recon):
@@ -1507,7 +1511,7 @@ class TestObjectUpdater(unittest.TestCase):
                       info_lines[-1])
         self.assertEqual({'skips': 2, 'successes': 2, 'unlinks': 2,
                           'deferrals': 2},
-                         self.logger.get_increment_counts())
+                         self.logger.statsd_client.get_increment_counts())
 
     @mock.patch('swift.obj.updater.dump_recon_cache')
     def test_per_container_rate_limit_defer_2_skip_1(self, mock_recon):
@@ -1557,7 +1561,8 @@ class TestObjectUpdater(unittest.TestCase):
 
         def fake_get_time(bucket_iter):
             captured_skips_stats.append(
-                daemon.logger.get_increment_counts().get('skips', 0))
+                daemon.logger.statsd_client.get_increment_counts().get(
+                    'skips', 0))
             captured_queues.append(list(bucket_iter.buckets[0].deque))
             # make each update delay before the iter being called again
             now[0] += latencies.pop(0)
@@ -1624,7 +1629,7 @@ class TestObjectUpdater(unittest.TestCase):
                       info_lines[-1])
         self.assertEqual(
             {'skips': 1, 'successes': 3, 'unlinks': 3, 'deferrals': 2,
-             'drains': 1}, self.logger.get_increment_counts())
+             'drains': 1}, self.logger.statsd_client.get_increment_counts())
 
     @mock.patch('swift.obj.updater.dump_recon_cache')
     def test_per_container_rate_limit_defer_3_skip_1(self, mock_recon):
@@ -1652,7 +1657,7 @@ class TestObjectUpdater(unittest.TestCase):
                          len(self._find_async_pending_files()))
         # indexes 0, 2 succeed; 1, 3, 4 deferred but 1 is bumped from deferral
         # queue by 4; 4, 3 are then drained
-        latencies = [0, 0.05, .051, 0, 0, 0, .11, .01]
+        latencies = [0, 0.05, .051, 0, 0, 0, .11]
         expected_success = 4
 
         contexts_fed_in = []
@@ -1674,7 +1679,8 @@ class TestObjectUpdater(unittest.TestCase):
 
         def fake_get_time(bucket_iter):
             captured_skips_stats.append(
-                daemon.logger.get_increment_counts().get('skips', 0))
+                daemon.logger.statsd_client.get_increment_counts().get(
+                    'skips', 0))
             captured_queues.append(list(bucket_iter.buckets[0].deque))
             # make each update delay before the iter being called again
             now[0] += latencies.pop(0)
@@ -1693,7 +1699,7 @@ class TestObjectUpdater(unittest.TestCase):
                                   fake_object_update), \
                 mock.patch('swift.obj.updater.RateLimitedIterator',
                            fake_rate_limited_iterator), \
-                mock.patch('swift.obj.updater.time.sleep') as mock_sleep:
+                mock.patch('swift.common.utils.eventlet.sleep') as mock_sleep:
             daemon.run_once()
         self.assertEqual(expected_success, daemon.stats.successes)
         expected_skipped = expected_total - expected_success
@@ -1719,7 +1725,7 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertEqual([aorder[o] for o in expected_updates_sent],
                          [aorder[o] for o in actual_updates_sent])
 
-        self.assertEqual([0, 0, 0, 0, 0, 1, 1, 1], captured_skips_stats)
+        self.assertEqual([0, 0, 0, 0, 0, 1, 1], captured_skips_stats)
 
         expected_deferrals = [
             [],
@@ -1729,7 +1735,6 @@ class TestObjectUpdater(unittest.TestCase):
             [objs_fed_in[1], objs_fed_in[3]],
             [objs_fed_in[3], objs_fed_in[4]],
             [objs_fed_in[3]],  # note: rightmost element is drained
-            [objs_fed_in[3]],
         ]
         self.assertEqual(
             expected_deferrals,
@@ -1745,7 +1750,7 @@ class TestObjectUpdater(unittest.TestCase):
                       info_lines[-1])
         self.assertEqual(
             {'skips': 1, 'successes': 4, 'unlinks': 4, 'deferrals': 3,
-             'drains': 2}, self.logger.get_increment_counts())
+             'drains': 2}, self.logger.statsd_client.get_increment_counts())
 
     @mock.patch('swift.obj.updater.dump_recon_cache')
     def test_per_container_rate_limit_unsent_deferrals(self, mock_recon):
@@ -1776,7 +1781,7 @@ class TestObjectUpdater(unittest.TestCase):
         # first pass: 0, 2 and 5 succeed, 1, 3, 4, 6 deferred
         # last 2 deferred items sent before interval elapses
         latencies = [0, .05, 0.051, 0, 0, .11, 0, 0,
-                     0.1, 0, 0.1, 0]  # total 0.42
+                     0.1, 0.1, 0]  # total 0.411
         expected_success = 5
 
         contexts_fed_in = []
@@ -1801,7 +1806,8 @@ class TestObjectUpdater(unittest.TestCase):
             if not captured_skips_stats:
                 daemon.begin = now[0]
             captured_skips_stats.append(
-                daemon.logger.get_increment_counts().get('skips', 0))
+                daemon.logger.statsd_client.get_increment_counts().get(
+                    'skips', 0))
             captured_queues.append(list(bucket_iter.buckets[0].deque))
             # insert delay each time iter is called
             now[0] += latencies.pop(0)
@@ -1820,7 +1826,7 @@ class TestObjectUpdater(unittest.TestCase):
                                   fake_object_update), \
                 mock.patch('swift.obj.updater.RateLimitedIterator',
                            fake_rate_limited_iterator), \
-                mock.patch('swift.obj.updater.time.sleep') as mock_sleep:
+                mock.patch('swift.common.utils.eventlet.sleep') as mock_sleep:
             daemon.run_once()
         self.assertEqual(expected_success, daemon.stats.successes)
         expected_skipped = expected_total - expected_success
@@ -1840,7 +1846,7 @@ class TestObjectUpdater(unittest.TestCase):
         self.assertEqual(expected_updates_sent, actual_updates_sent)
 
         # skips (un-drained deferrals) not reported until end of cycle
-        self.assertEqual([0] * 12, captured_skips_stats)
+        self.assertEqual([0] * 10, captured_skips_stats)
 
         objs_fed_in = [ctx['update']['obj'] for ctx in contexts_fed_in]
         expected_deferrals = [
@@ -1856,8 +1862,6 @@ class TestObjectUpdater(unittest.TestCase):
             # note: rightmost element is drained
             [objs_fed_in[1], objs_fed_in[3], objs_fed_in[4], objs_fed_in[6]],
             [objs_fed_in[1], objs_fed_in[3], objs_fed_in[4]],
-            [objs_fed_in[1], objs_fed_in[3], objs_fed_in[4]],
-            [objs_fed_in[1], objs_fed_in[3]],
             [objs_fed_in[1], objs_fed_in[3]],
         ]
         self.assertEqual(
@@ -1874,8 +1878,9 @@ class TestObjectUpdater(unittest.TestCase):
                       info_lines[-1])
         self.assertEqual(
             {'successes': 5, 'unlinks': 5, 'deferrals': 4, 'drains': 2},
-            self.logger.get_increment_counts())
-        self.assertEqual([('skips', 2)], self.logger.get_update_stats())
+            self.logger.statsd_client.get_increment_counts())
+        self.assertEqual(
+            2, self.logger.statsd_client.get_stats_counts()['skips'])
 
 
 class TestObjectUpdaterFunctions(unittest.TestCase):
@@ -1911,21 +1916,21 @@ class TestBucketizedUpdateSkippingLimiter(unittest.TestCase):
         it = object_updater.BucketizedUpdateSkippingLimiter(
             [3, 1], self.logger, self.stats, 1000, 10)
         self.assertEqual(1000, it.num_buckets)
-        self.assertEqual(0.1, it.bucket_update_delta)
+        self.assertEqual([10] * 1000, [b.max_rate for b in it.buckets])
         self.assertEqual([3, 1], [x for x in it.iterator])
 
         # rate of 0 implies unlimited
         it = object_updater.BucketizedUpdateSkippingLimiter(
             iter([3, 1]), self.logger, self.stats, 9, 0)
         self.assertEqual(9, it.num_buckets)
-        self.assertEqual(-1, it.bucket_update_delta)
+        self.assertEqual([0] * 9, [b.max_rate for b in it.buckets])
         self.assertEqual([3, 1], [x for x in it.iterator])
 
         # num_buckets is collared at 1
         it = object_updater.BucketizedUpdateSkippingLimiter(
             iter([3, 1]), self.logger, self.stats, 0, 1)
         self.assertEqual(1, it.num_buckets)
-        self.assertEqual(1, it.bucket_update_delta)
+        self.assertEqual([1], [b.max_rate for b in it.buckets])
         self.assertEqual([3, 1], [x for x in it.iterator])
 
     def test_iteration_unlimited(self):
@@ -1963,7 +1968,7 @@ class TestBucketizedUpdateSkippingLimiter(unittest.TestCase):
         # enough capacity for all deferrals
         with mock.patch('swift.obj.updater.time.time',
                         side_effect=[now, now, now, now, now, now]):
-            with mock.patch('swift.obj.updater.time.sleep') as mock_sleep:
+            with mock.patch('swift.common.utils.eventlet.sleep') as mock_sleep:
                 it = object_updater.BucketizedUpdateSkippingLimiter(
                     iter(update_ctxs[:3]), self.logger, self.stats, 1, 10,
                     max_deferred_elements=2,
@@ -1982,7 +1987,7 @@ class TestBucketizedUpdateSkippingLimiter(unittest.TestCase):
         # only space for one deferral
         with mock.patch('swift.obj.updater.time.time',
                         side_effect=[now, now, now, now, now]):
-            with mock.patch('swift.obj.updater.time.sleep') as mock_sleep:
+            with mock.patch('swift.common.utils.eventlet.sleep') as mock_sleep:
                 it = object_updater.BucketizedUpdateSkippingLimiter(
                     iter(update_ctxs[:3]), self.logger, self.stats, 1, 10,
                     max_deferred_elements=1,
@@ -2000,7 +2005,7 @@ class TestBucketizedUpdateSkippingLimiter(unittest.TestCase):
         # only time for one deferral
         with mock.patch('swift.obj.updater.time.time',
                         side_effect=[now, now, now, now, now + 20, now + 20]):
-            with mock.patch('swift.obj.updater.time.sleep') as mock_sleep:
+            with mock.patch('swift.common.utils.eventlet.sleep') as mock_sleep:
                 it = object_updater.BucketizedUpdateSkippingLimiter(
                     iter(update_ctxs[:3]), self.logger, self.stats, 1, 10,
                     max_deferred_elements=2,
@@ -2019,7 +2024,7 @@ class TestBucketizedUpdateSkippingLimiter(unittest.TestCase):
         with mock.patch('swift.obj.updater.time.time',
                         side_effect=[now, now, now, now, now,
                                      now + 20, now + 20]):
-            with mock.patch('swift.obj.updater.time.sleep') as mock_sleep:
+            with mock.patch('swift.common.utils.eventlet.sleep') as mock_sleep:
                 it = object_updater.BucketizedUpdateSkippingLimiter(
                     iter(update_ctxs), self.logger, self.stats, 1, 10,
                     max_deferred_elements=2,
@@ -2048,7 +2053,7 @@ class TestBucketizedUpdateSkippingLimiter(unittest.TestCase):
         # deferrals stick in both buckets
         with mock.patch('swift.obj.updater.time.time',
                         side_effect=[next(time_iter) for _ in range(12)]):
-            with mock.patch('swift.obj.updater.time.sleep') as mock_sleep:
+            with mock.patch('swift.common.utils.eventlet.sleep') as mock_sleep:
                 it = object_updater.BucketizedUpdateSkippingLimiter(
                     iter(update_ctxs_1 + update_ctxs_2),
                     self.logger, self.stats, 4, 10,
@@ -2073,7 +2078,7 @@ class TestBucketizedUpdateSkippingLimiter(unittest.TestCase):
         # oldest deferral bumped from one bucket due to max_deferrals == 3
         with mock.patch('swift.obj.updater.time.time',
                         side_effect=[next(time_iter) for _ in range(10)]):
-            with mock.patch('swift.obj.updater.time.sleep') as mock_sleep:
+            with mock.patch('swift.common.utils.eventlet.sleep') as mock_sleep:
                 it = object_updater.BucketizedUpdateSkippingLimiter(
                     iter(update_ctxs_1 + update_ctxs_2),
                     self.logger, self.stats, 4, 10,
@@ -2097,7 +2102,7 @@ class TestBucketizedUpdateSkippingLimiter(unittest.TestCase):
         # older deferrals bumped from one bucket due to max_deferrals == 2
         with mock.patch('swift.obj.updater.time.time',
                         side_effect=[next(time_iter) for _ in range(10)]):
-            with mock.patch('swift.obj.updater.time.sleep') as mock_sleep:
+            with mock.patch('swift.common.utils.eventlet.sleep') as mock_sleep:
                 it = object_updater.BucketizedUpdateSkippingLimiter(
                     iter(update_ctxs_1 + update_ctxs_2),
                     self.logger, self.stats, 4, 10,
@@ -2119,16 +2124,8 @@ class TestBucketizedUpdateSkippingLimiter(unittest.TestCase):
 
 
 class TestRateLimiterBucket(unittest.TestCase):
-    def test_wait_until(self):
-        b1 = object_updater.RateLimiterBucket(10)
-        self.assertEqual(10, b1.wait_until)
-        b1.last_time = b1.wait_until
-        self.assertEqual(20, b1.wait_until)
-        b1.last_time = 12345.678
-        self.assertEqual(12355.678, b1.wait_until)
-
     def test_len(self):
-        b1 = object_updater.RateLimiterBucket(10)
+        b1 = object_updater.RateLimiterBucket(0.1)
         b1.deque.append(1)
         b1.deque.append(2)
         self.assertEqual(2, len(b1))
@@ -2136,7 +2133,7 @@ class TestRateLimiterBucket(unittest.TestCase):
         self.assertEqual(1, len(b1))
 
     def test_bool(self):
-        b1 = object_updater.RateLimiterBucket(10)
+        b1 = object_updater.RateLimiterBucket(0.1)
         self.assertFalse(b1)
         b1.deque.append(1)
         self.assertTrue(b1)
@@ -2148,13 +2145,13 @@ class TestRateLimiterBucket(unittest.TestCase):
         b1 = object_updater.RateLimiterBucket(10)
         b2 = object_updater.RateLimiterBucket(10)
 
-        b2.last_time = next(time_iter)
+        b2.running_time = next(time_iter)
         buckets = PriorityQueue()
         buckets.put(b1)
         buckets.put(b2)
         self.assertEqual([b1, b2], [buckets.get_nowait() for _ in range(2)])
 
-        b1.last_time = next(time_iter)
+        b1.running_time = next(time_iter)
         buckets.put(b1)
         buckets.put(b2)
         self.assertEqual([b2, b1], [buckets.get_nowait() for _ in range(2)])
